<div class="slick-numbers">
  {% for post in posts %}
  <div class="item-numbers">
    <a href="{{ post.link }}">
      <img src="{{ post.thumbnail.src }}" alt="{{ post.thumbnail.alt }}">
    </a>
  </div>
  {% endfor %}
  <div class="item-numbers">
    <div class="item-numbers--anteriores bk-primary">
      <div class="item-numbers--data">
        <a href="{{function('get_post_type_archive_link','fm_magazine')}}">
          <h3 class="heading-3">Ver todos<br> los números<br>anteriores<i class="fas fa-long-arrow-alt-right ml-2"></i></h3>
        </a>
        <hr class="hr-black">
      </div>
    </div>
  </div>

</div>
