footer {
  padding: 40px 0px;
}
.footer{
  color: $grey;
  ul {
    margin: 0;
    padding: 0;
    text-decoration: none;
    list-style: none;
    line-height: 30px;
  }
  a {
    font-size: 16px;
    font-weight: 300;
    color: $grey;
  }
  &--copy {
    padding: 30px 0px;
    background-color: $grey;
    span {
      color: white;
      font-size: 14px;
    }
  }
  &--share {
    padding-top: 15px;
    ul {
      display: flex;
      @include media-breakpoint-down(lg){
        justify-content: left;
      }
    }
    a {
      width: 30px;
      height: 30px;
      border-radius: 100%;
      text-align: center;
      color: white;
      font-size: 16px;
      margin-right: 10px;
      line-height: 33px;
      display: inline-block;
    }
  }
  &--subscribe {
    h3 {
      text-transform: uppercase;
      font-weight: bold;
    }
    p {
      font-weight: 300;
      font-size: 12px;
      line-height: 21px;
    }
    button {
      padding: 5px 20px;
      margin-top: 20px;
      color: white;
    }
  }
}
