<?php

namespace Twig\Node\Expression\Test;

class_exists('Twig_Node_Expression_Test_Divisibleby');

if (\false) {
    class DivisiblebyTest extends \Twig_Node_Expression_Test_Divisibleby
    {
    }
}
