<div class="grilla-ediciones">
  <div class="container">
    <!-- FILA 1 -->
    {% for row in posts | batch(6) %}
    <div class="row">
      {% for post in row %}
      <div class="grilla-ediciones--tapa col-6 col-lg-2">
        <a href="{{post.link}}"><img src="{{ post.thumbnail.src }}" alt=""></a>
        <a href="{{post.link}}"><h4 class="heading-5 mt-2 font-normal">#{{post.magazine_number}} | {{ post.post_date | date("F Y") | capitalize }}</h4></a>
      </div>
      {% endfor %}
    </div>
    {% endfor %}

    <div class="grilla-ediciones--pagination">
      {% include 'fm/partials/elements/pagination.twig' with {'pagination' : posts.pagination} %}
    </div>
  </div>
</div>
