<head>
    <meta charset="{{ site.charset }}" />
    <meta name="description" content="{{ site.description }}">
    <link rel="stylesheet" href="{{ site.theme.link }}/style.css" type="text/css" media="screen" />
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="author" href="{{ site.theme.link }}/humans.txt" />
    <link rel="pingback" href="{{ site.pingback_url }}" />
    <link rel="profile" href="http://gmpg.org/xfn/11">
    {{function('wp_head')}}
    <!-- Google Tag Manager -->
    <script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start':
    new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0],
    j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
    'https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
    })(window,document,'script','dataLayer','GTM-PKS844L');</script>
    <!-- End Google Tag Manager -->
</head>
