
<?php
class PageFmThankYouPage extends CDController{

  public function __construct() {
    $this->view = 'fm/page/thank-you.twig';
  }

  public function get_context(){

    $this->data = CDTimber::get_context();

    $this->data['post']->hero_login = true;

  }

}
