<form id="form-change-password">
  <div class="form-group">
    <label for="password">Contraseña*</label>
    <input type="password" name="password" class="form-control">
  </div>
  <div class="form-group">
    <label for="repeat_password">Repetir Contraseña*</label>
    <input type="password" name="repeat_password" class="form-control" >
  </div>

  <div class="text-center mb-3">
    <button type="submit" class="btn-block btn-lg bk-primary text-white">Enviar</button>
  </div>

  <div class="form-message sending-message" style="display:none;">
    <div class="col alert alert-info" role="alert">
     <p>Enviando información...</p>
    </div>
  </div>
  <div class="form-message error-message" style="display:none;">
    <div class="col alert alert-danger" role="alert">
     <p></p>
    </div>
  </div>
  <div class="form-message success-message" style="display:none;">
    <div class="col alert alert-success" role="alert">
     <p>Ingreso completado! Actualizando....</p>
    </div>
  </div>
</form>
