<div class="articles"> 
  <div class="container-custom">
    <div class="row">
      <div class="articles--actual col-lg-4 align-self-center">
        <div class="borde p-4">
          <div class="articles--title">
            <span class="txt-primary">FM&WORKPLACES</span>
            <h3 class="heading-3 mt-2">{{module.title}}</h3>
            <hr class="hr-primary bk-primary">
          </div>
          <p class="mb-4">{{module.text}}</p>
        </div>
        <div class="articles--vermas">
          <a href="{{function('get_post_type_archive_link','fm_magazine')}}" class="txt-primary">Ver ediciones anteriores<i class="fas fa-long-arrow-alt-right txt-primary ml-2"></i></a>
        </div>
      </div>
      <div class="col-lg-8">
        {% include 'fm/partials/carousels/carousel-numbers.twig' with {'posts': module.other_magazines} %}
      </div>
    </div>
  </div>
</div>
