<?php

/*
 * This file is part of twig-cache-extension.
 *
 * (c) Alexander <iam.asm89@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Asm89\Twig\CacheExtension\Exception;

/**
 * @todo Replace \RuntimeException with \InvalidArgumentException at version 2.0
 */
class BaseException extends \RuntimeException
{

}
