<!doctype html>
	<!--[if lt IE 7]><html class="no-js no-svg ie ie6 lt-ie9 lt-ie8 lt-ie7" {{ site.language_attributes }}> <![endif]-->
	<!--[if IE 7]><html class="no-js no-svg ie ie7 lt-ie9 lt-ie8" {{ site.language_attributes }}> <![endif]-->
	<!--[if IE 8]><html class="no-js no-svg ie ie8 lt-ie9" {{ site.language_attributes }}> <![endif]-->
	<!--[if gt IE 8]><!--><html class="no-js no-svg" {{ site.language_attributes }}> <!--<![endif]-->

	{% block head %}
		{% include 'head.twig' %}
	{% endblock %}

	<body class="{{body_class}}" data-template="base.twig">
		<!-- Google Tag Manager (noscript) -->
		<noscript><iframe src="https://www.googletagmanager.com/ns.html?id=GTM-PKS844L"
		height="0" width="0" style="display:none;visibility:hidden"></iframe></noscript>
		<!-- End Google Tag Manager (noscript) -->
		
		<div class="wrapper">

			{% block header %}
				{% include 'header.twig' %}
			{% endblock %}

			{% block content %}
				Sorry, no content
			{% endblock %}

			{% block footer %}
				{% include 'footer.twig' %}
			{% endblock %}
		</div>
		{{ function('wp_footer') }}
	</body>
</html>
