<div class="hero" style="background-image: url('{{ Image(post.hero_home_image).src }}');">
  <div class="hero--navigation col-lg-3 p-0">
    {% include 'fm/header.twig' %}
  </div>
  <div class="hero--text col-xl-4 col-lg-6 text-right">
    <span>{{post.hero_home_lead}}</span>
    <h1>{{post.hero_home_title}}</h1>
    <p>{{post.hero_home_subtitle}}</p>
    {% if post.hero_home_button %}
    <a href="{{post.hero_home_button.url}}" class="bk-primary btn">{{post.hero_home_button.title}}</a>
    {% endif %}
  </div>
</div>
