<?php
class ModuleFmSubscribe extends CDController{

  public function __construct() {

    $this->view = 'fm/partials/sections/module-subscribe.twig';

  }

  public function get_context(){

    $this->data['module'] = get_field('option_module_fm_subscribe', 'option');

  }

}
