<div class="staff">
  <div class="staff_title">
    <h3 class="heading-3">STAFF</h3>
    <hr class="hr-primary bk-primary">
  </div>
  <div class="row">
    {% for person in staff %}
    <aside class="profile col-lg-6">
      <div class="staff_box align-self-center">
        <div class="staff_box--status">
          <span class="txt-primary">{{person.position}}</span>
        </div>
        <div class="staff_box--name">
          <h4 class="heaing-4">{{person.name}}</h4>
        </div>
        <hr class="hr-primary bk-primary">
        <div class="staff_box--email">
          <a href="mailto:{{person.email}}">{{person.email}}</a>
        </div>
      </div>
    </aside>
    {% endfor %}
  </div>
</div>
