//HOME
.hero {
  background-position: center;
  background-repeat: no-repeat;
  background-size: cover;
  height: 90vh;
  @include media-breakpoint-up(lg) {
    height: 800px;
  }
  position: relative;
  &::before {
    content: "";
    display: block;
    background-color:rgba(0, 0, 0, 0.2);
    position: absolute;
    left: 0;
    top: 0;
    height: 100%;
    width: 100%;
    z-index: 1;
  }
  &--text {
    color: white;
    right: 0px;
    bottom: 5%;
    position: absolute;
    z-index: 3;
    @include media-breakpoint-up(lg) {
      right: 30px;
      z-index: 2;
      top: 200px;
    }
    h1 {
      font-weight: bold;
      text-transform: uppercase;
      margin-top: 10px;
      margin-bottom: 10px;
      @include media-breakpoint-up(lg) {
        margin-top: 10px;
        line-height: 44px;
      }
    }
    span {
      font-size: 16px;
      font-weight: normal;
    }
    p {
      font-size: 18px;
      line-height: 22px;
    }
    .btn {
      padding: 8px 15px;
      text-transform: uppercase;
      color: white;
      font-size: 12px;
      margin-top: 15px;
      border-radius: 0;
      @include media-breakpoint-up(lg) {
        margin-top: 30px;
      }
      &:hover {
        background-color: black;
        @include transition;
      }
    }
  }
  &--suscribite {
    color: white;
    top: 50px;
    right: 40px;
    position: absolute;
    z-index: 2;
    display: none;
    @include media-breakpoint-up(lg) {
          display: flex;
    }
    button {
      padding: 5px 15px;
    }
    .btn-white {
      background: white;
      margin-right: 5px;
    }
  }
  &--navigation {
    height: 100%;
    background-color:rgba(0, 0, 0, 0.5);
    &--mobile {
      background-color:rgba(0, 0, 0, 0.5);
      position: relative;
      height: 50px;
      z-index: 3;
      &--burger {
        position: absolute;
        width: 40px;
        height: 20px;
        top: 10px;
        left: 15px;
        cursor: pointer;
        img {
          width: 100%;
        }
      }
      &--login {
        position: absolute;
        top: 13px;
        right: 15px;
        a {
          color: white;
          text-transform: uppercase;
          font-size: 14px;
          margin: 0px;
          margin-top: -5px;
        }
      }
    }
    &--header {
      padding-bottom: 20px;
      z-index: 2;
    }
    &--menu {
      color: white;
      padding: 30px 20px;
      z-index: 777;
      font-weight: 500;
      width: 100%;
      height: 100%;
      top: 0;
      position: fixed;
      display: none;
      @include media-breakpoint-up(lg) {
        display: block;
        position: relative;
        height: auto;
        background-color: transparent !important;
      }
      .hr-white {
        height: 1px;
      }
      a {
        color: white;
      }
      ul {
        list-style: none;
        padding: 0;
        line-height: 40px;
      }
      &--closed {
        position: absolute;
        top: 10px;
        right: 0;
        display: block;
        @include media-breakpoint-up(lg) {
          display: none;
        }
      }
    }
    .btn-buscar {
      color: white;
      font-size: 16px;
      padding: 0;
      margin: 0;
      text-transform: inherit;
    }
  }
}

.hero-interno {
  height: 550px;
}

//ARTICLES
.articles {
  padding-top: 30px;
  padding-bottom: 20px;
  color: $grey;
  @include media-breakpoint-up(lg) {
    padding-top: 47px;
    padding-bottom: 60px;
  }
  &--title {
    text-transform: uppercase;
    position: relative;
    h3 {
      font-weight: 900;
    }
  }
  .btn {
    background: black;
    margin-top: 15px;
    border-radius: 0;
    button {
      font-size: 12px !important;
    }
  }
  button {
    font-size: 27px;
  }
  &--vermas {
    margin-left: 30px;
    margin-top: 15px;
    margin-bottom: 15px;
  }
}

button {
  background: none;
  border: none;
}
//ADVERTISING

.advertising {
  padding: 30px 0px;
  @include media-breakpoint-up(lg) {
    padding: 60px 0px;
  }
  &--brand {
    margin-bottom: 15px;
    @include media-breakpoint-up(lg) {
      margin-bottom: 0px;
    }
    img {
      width: 100%;
      height: 100%;
    }
  }
  &--info {
    margin: 15px;
    display: flex;
    flex-wrap:wrap;
    flex-direction: column;
    @include media-breakpoint-up(lg) {
      align-items: center;
      flex-direction:row;
      margin: 0px;
    }
    h3 {
      color: $grey;
      font-weight: bold;
      line-height: 32px;
    }
  }
}

.about {
  padding: 30px 0px;
  @include media-breakpoint-up(lg) {
    padding: 60px 0px;
  }
  &--article {
    margin-right: 20px;
    margin-bottom: 10px;
    display: none;
    @include media-breakpoint-up(lg) {
      display: block;
      margin-right: 0;
      margin-bottom: 0;
    }
    &::after {
      content: "";
      display: block;
      background: black;
      position: absolute;
      top: 30px;
      right: 0;
      height: 80%;
      width: 94%;
      z-index: -1;
    }
  }
  &--info {
    right: 0px;
    margin-bottom: 15px;
    .borde {
      @include media-breakpoint-up(lg) {
        margin-right: -40px;
      }
    }
    &--vermas {
      margin-top: 15px;
      margin-left: 25px;
      margin-bottom: 15px;
    }
  }
}

.about--submit {
  position: relative;
}
.about--submit .bk-primary {
  height: 100%;
  position: relative;
}
.about .about--submit--info {
  padding: 20px;
  color: white;
  @include media-breakpoint-up(lg){
    position: absolute;
    top: 15%;
    left: 10%;
    max-width: 400px;
  }
  button {
    background: black;
    color: white;
  }
}

.about--submit--workplaces {
  content: "";
  display: none;
  @include media-breakpoint-up(lg){
    display: block;
  }
  background-image: url('../images/img-article.png');
  position: absolute;
  right:0;
  top: 0;
  height: 100%;
  width: 100px;
  z-index: 999;
  overflow: hidden;
  &::after {
    content: "";
    display: block;
    background-color:rgba(0, 0, 0, 0.5);
    position: absolute;
    left: 0;
    top: 0;
    height: 100%;
    width: 100%;
    z-index: 0;
  }
  &--text {
    position: absolute;
    height: 30px;
    width: 300px;
    top: 48%;
    color: white;
    left: -100%;
    z-index: 1;
    -webkit-transform: rotate(-90deg);
    transform: rotate(-90deg);
    overflow: hidden;
    max-height: 30px;
    display: -webkit-box;
    -webkit-line-clamp: 3;
    -webkit-box-orient: vertical;
  }
}

//CAROUSELS

.item-article {
  // height: 490px;
  position: relative;
  padding-right: 30px;
  color: white;
  margin-top: 20px;
  overflow: hidden;
  @include media-breakpoint-up(lg) {
    margin-top: 0px;
  }
  &--data {
    position: absolute;
    bottom: 49px;
    padding: 10px 15px;
    width: calc(100% - 30px);
    @include gradient;
    z-index: 2;
    overflow: hidden;
    p {
      font-weight: 300;
      font-size: 16px;
      line-height: 21px;
    }
  }
  &--vermas {
    margin-top: 12px;
    margin-left: 40px;
    margin-bottom: 13px;
  }
  img {
    // height: 90%;
    width: 100%;
  }

  .bk-primary {
    content: "";
    display: block;
    position: absolute;
    top: 100px;
    left: 20px;
    height: 100%;
    width: 92%;
    z-index:-1;
  }
  .bg-black {
    content: "";
    display: block;
    position: absolute;
    top: 100px;
    left: 20px;
    height: 100%;
    width: 92%;
    z-index:-1;
  }
  a {
    z-index: 999;
    &:hover {
      color: white;
    }
  }
}

.item-article--data:hover > .detail {
  opacity: 10;
  overflow: hidden;
  height: 70px;
  max-height: 65px;
  display: -webkit-box;
  -webkit-line-clamp: 3;
  -webkit-box-orient: vertical;
}
.detail {
  height: 0;
  opacity: 0;
  bottom: -30px;
  transition: height 0.7s ease-out;
}
.item-numbers {
  // height: 250px;
  position: relative;
  padding-right: 20px;
  color: white;
  @include media-breakpoint-up(lg) {
    padding-right: 5px;
    // height: 430px;
  }
  &--data {
    padding: 20% 7% 30% 7%;
    width: 100%;
    z-index: 2;
    margin-top: 20px;
    @include media-breakpoint-up(lg) {
      margin-top: 0px;
    }
    h3 {
      text-transform: uppercase;
    }
  }
  img {
    width: 100%;
  }
  &--anteriores {
    display: flex;
    align-items: center;
    width: 100%;
    hr {
      margin-top: 5px;
    }
    @include media-breakpoint-up(lg) {
      height: 55%;
      width: 90%;
    }
    a {
      color: white;
    }
    h3 {
      font-weight: bold;
      @include media-breakpoint-up(lg) {
        line-height: 28px;
      }
    }
    &:after {
      content: "";
      display: block;
      background: black;
      position: absolute;
      top: 20px;
      left: 5px;
      height: 30%;
      @include media-breakpoint-up(lg) {
        height: 40%;
        top: 30px;
      }
      width: 94%;
      z-index:-1;
    }
  }
}

.single {
  &--detail {
    ul {
      display: flex;
    }
  }
  .wp-block-table{
    margin: 40px 0px;
    td{
      padding: 10px;
    }
  }

  &--txt {
    // h1,h2,h3 {
    //   margin-bottom: 20px;
    //   font-weight: bold;
    // }
    .has-medium-font-size{
      line-height: 30px;
    }
    h2{
      margin: 20px 0px;
      font-weight: bold;
      font-size: 22px;
    }
    h3{
      margin: 15px 0px;
      font-weight: bold;
      font-size: 20px;
    }
    ul {
      list-style-type:disc;
    }
    p {
      margin: 10px 0px;
      line-height: 26px;
    }
    a {
      text-decoration: underline;
      margin-bottom: 15px;
      &:hover {
        color: black;
      }
    }
    img {
      margin: 15px 0px;
      max-width: 100%;
    }
  }
  .breadcrumb {
    background: none;
    padding: 0;
  }
}
.single-txt {
  padding: 60px 0px;
}
.adv-vertical {
  img {
    width: 100%;
  }
  &--title {
    padding: 35px 20px;
    border: 4px solid $light-grey;
    h3 {
      text-transform: uppercase;
      font-weight: 900;
      line-height: 32px;
      color: $grey;
    }
  }
}
.grilla-ediciones {
  padding: 50px 0px;
  img {
    width: 100%;
  }
  &--tapa {
    margin-bottom: 15px;
    @include media-breakpoint-up(lg) {
      margin-bottom: 25px;
    }
    a:hover {
      color: black;
      text-decoration: underline;
    }
  }
  &--wrap {
    margin-bottom: 30px;
  }
  &--pagination {
    display: flex;
    justify-content: center;
    margin-top: 50px;
  }
}
.submit-form {
  form {
    margin-bottom: 50px;
  }
  label {
    font-weight: 500;
    color: $grey;
  }
  &--aclaracion {
    margin-top: 50px;
    p {
      font-size: 12px;
      line-height: 16px;
    }
  }
  &--txt {
    margin-bottom: 30px;
    p {
      margin-bottom: 30px;
    }
  }
  .btn {
    border-radius: 0;
    button {
      padding: 8px 20px;
      color: white;
    }
  }
  .form-contacto {
    margin-bottom: 120px;
  }
}
.employees {
  label {
    font-weight: normal;
  }
}
.submit-horizontal {
  padding: 30px 40px;
  margin: 30px 0px;
  &--info {
    p {
      color: white;
      font-size: 22px;
      line-height: 28px;
      font-weight: bold;
    }
  }
  button {
    color: white;
    background: black;
    width: 182px;
    height: 40px;
  }
}
// BUTTONS - slick
.slick-arrow {
  position: absolute;
  bottom: -50px;
  right: 0;
  z-index:10;
}
.slick-prev {
  margin-right: 30px;
  border: none;
}
.slick-next {
  border: none;
}

.topnav a, .topnav input[type=text], .topnav .search-container button {
  float: none;
  text-align: left;
  margin: 0;
  padding: 14px;
  background: none;
}
.topnav input[type=text] {
  border: none;
  color: white;
}
.topnav {
  input::-webkit-input-placeholder {
    color: white;
  }
  input:-moz-placeholder {
    color: white;
  }
  input:-ms-input-placeholder {
    color: white;
  }
}

button {
  font-size: 12px;
  text-transform: uppercase;
  padding: 8px 20px;
}

//staff

.staff {
  margin-top: 90px;
  .staff_title {
    margin-bottom: 30px;
  }
  .profile {
    margin-bottom: 30px;
  }
  .staff_box {
    border: 4px solid $light-grey;
    padding: 30px;
    color: $grey;
    &--status {
      margin-bottom: 10px;
      span {
        font-size: 14px;
        font-weight: 500;
        text-transform: uppercase;
      }
    }
    &--name {
      font-weight: bold;
    }
    &--email {
      font-size: 18px;
      a {color: $grey;}
    }
  }
}
//MODAL FOOTER

.modal-footer {
  justify-content:flex-start;
  button {
    text-transform: uppercase;
    color: white;
    padding: 10px;
    font-size: 16px;
  }
}

.login {
  &--acount {
    font-weight: 500;
    a {
      color: $grey;
    }
  }
  .modal-title {
    font-size: 28px;
    color: $grey;
    font-weight: bold;
  }
  .modal-content {
    padding-bottom: 30px;
  }
  .text-underline {
    text-decoration: underline;
  }
  &--btn {
    padding: 0px 15px;
    margin-bottom: 15px;
    button {
      padding: 15px 0px;
      font-size: 16px;
    }
  }
  label {
    font-weight: 500;
    font-size: 20px;
  }
}


//animate nav
.animated {
 -webkit-animation-duration: 0.5s;
 animation-duration: 0.5s;
 -webkit-animation-fill-mode: both;
 animation-fill-mode: both;
}

@-webkit-keyframes fadeInLeft {
  from {
    opacity: 0;
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0);
  }

  to {
    opacity: 1;
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0);
  }
}

@keyframes fadeInLeft {
  from {
    opacity: 0;
    -webkit-transform: translate3d(-100%, 0, 0);
    transform: translate3d(-100%, 0, 0);
  }

  to {
    opacity: 1;
    -webkit-transform: translate3d(0, 0, 0);
    transform: translate3d(0, 0, 0);
  }
}

.fadeInLeft {
  display: block;
  -webkit-animation-name: fadeInLeft;
  animation-name: fadeInLeft;
}
