
<div class="slick-articles">
  {% for post in posts %}
  <div class="item-article">
    <a href="{{ post.link }}">
      {% if post.thumbnail %}
      <img src="{{ post.thumbnail.src }}" alt="{{ post.thumbnail.alt }}">
      {% else %}
      <img src="{{site_options.default_images.magazine_post.featured.url }}" alt="{{site_options.default_images.magazine_post.featured.alt }}">
      {% endif %}
    </a>
      <div class="item-article--data">
        <a href="{{ post.link }}" class='show-txt'><h3 class="heading-3 pt-2">{{post.title}}</h3></a>
        <div class="detail">
          <p>{{post.preview}}</p>
        </div>
      </div>
    <div class="{% if is_current %}bk-primary{% else %}bg-black{% endif %}"></div>
    <div class="item-article--vermas">
      <a href="{{ post.link }}" class="text-white">Ver más<i class="fas fa-long-arrow-alt-right ml-2"></i></a>
    </div>
  </div>
  {% endfor %}
</div>
