{% extends "fm/base.twig" %}

{% block content %}
  {% include 'fm/partials/heroes/hero.twig' with {'post' : last_magazine} %}
  {% include 'fm/partials/sections/last-magazine-articles.twig' with {'post' : last_magazine, 'last_magazine_posts': last_magazine_posts } %}

  {{ function(
      [ 'CDController', 'init_controller'],
      'module',
      {
        template: 'fm_advertising',
        args: {
          direction: 'horizontal',
          location: 'home'
        }
      }
  ) }}

  {{ function(
      [ 'CDController', 'init_controller'],
      'module',
      {
        template: 'fm_other_numbers',
        args: {
          magazine_id: last_magazine.id,
        }
      }
  ) }}

  {% include 'fm/partials/sections/about-subscription.twig' %}
{% endblock %}
