<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'AltoRouter' => $vendorDir . '/altorouter/altorouter/AltoRouter.php',
);
