{% if post.hero_image %}
<div class="hero hero-interno" style="background-image: url('{{ Image(post.hero_image).src }}');">
{% else %}
<div class="hero hero-interno" style="background-image: url('{{ site_options.default_images.magazine_post.hero.url }}');">
{% endif %}

  <div class="hero--navigation col-lg-3 p-0">
    {% include 'fm/header.twig' %}
  </div>

  <div class="hero--text col-xl-4 col-lg-6 text-right">
    <span>{{post.hero_lead}}</span>
    <h1 class="text-uppercase heading-2">{{post.hero_title}}</h1>
    {% if post.hero_subtitle %}
    <p>{{post.hero_subtitle}}</p>
    {% endif %}
    {% if post.hero_login %}
    <a href="#" data-toggle="modal" data-toggle="modal" data-target="#login">Ingresar</a>
    {% endif %}
  </div>
</div>
