{% set form_text =  post.meta('form_text') %}
<div class="submit-form">
  <h3 class="heading-3">{{form_text.title}}</h3>
  <hr class="hr-primary bk-primary">
  <div class="submit-form--txt">
    <p>{{form_text.text}}</p>
  </div>
  <form id="form-confirm-migrated-user">
    <div class="row">
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputCargo">Contraseña*</label>
          <input type="password" name="password" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputTel">Repetir Contraseña*</label>
          <input type="password" name="repeat_password" class="form-control data-field">
        </div>
      </div>

      <div class="col-lg-6">
        <label for="exampleInputnombre">País*</label>
        <select name="country" class="form-control data-field">
          <option value="">Seleccione un país</option>
          {% for key,choice in form.country_choices %}
            <option value="{{key}}">{{choice}}</option>
          {% endfor %}
        </select>
      </div>

      <div class="col-lg-6 employees">
        <label class="form-check-label mb-2 font-medium" for="exampleCheck1">Tipo de actividad de su empresa</label>
        <select name="company_area" class="form-control mb-5 data-field">
          <option value="">Seleccione un tipo de actividad</option>
          {% for key,choice in form.company_area_choices %}
            <option value="{{key}}">{{choice}}</option>
          {% endfor %}
        </select>

        <div class="form-group other-area-container" style="display: none;">
          <label for="localidad">Otra actividad</label>
          <input type="text" name="company_area_other" class="form-control data-field">
        </div>

      </div>

    </div>

    <div class="bk-primary btn btn-lg text-white my-4">
      <button type="submit" name="button">Actualizar datos</button>
    </div>

    <div class="row form-message sending-message" style="display:none;">
      <div class="col alert alert-info" role="alert">
       <p>Enviando información...</p>
      </div>
    </div>
    <div class="row form-message error-message" style="display:none;">
      <div class="col alert alert-danger" role="alert">
       <p></p>
      </div>
    </div>
    <div class="row form-message success-message" style="display:none;">
      <div class="col alert alert-success" role="alert">
       <p>Actualización completa! Redireccionando...</p>
      </div>
    </div>
  </form>

  <div class="submit-form--aclaracion">
    <p>(1) Los datos correspondientes a la información y documentación brindada en razón de la presente, que podrán ser cedidos a terceros, podrán ser utilizados por Facility Magazine & Workplaces a los efectos de planear y realizar acciones de marketing promocional, ofrecer productos o servicios propios o de terceros; respecto de todo lo cual se presta el libre, expreso e informado consentimiento.</p>
    <p>(2) Actualmente, la edición de papel sólo está disponible en la Argentina, Chile, Uruguay, Perú, Paraguay y Colombia. Desde el resto de los paises se puede acceder a los contenidos a través de las versiones online o en formato PDF.</p>
  </div>
</div>
