div.wpcf7 .ajax-loader{
  display: none;
}
.wpcf7-response-output{
  margin: 2em 0em 1em;
  text-align: center;
  padding: 1em;
}
.div.wpcf7-validation-errors{
  background-color: #f7e700;
}
.wpcf7-mail-sent-ok{
  background-color: rgb(122, 242, 123);
  border-color: rgb(62, 207, 63);
}


input.error-field{
  border-color: red;
}
.error-field{
  color: red;
}
