@mixin parallax(){
    background-attachment: fixed;
    background-position: center;
    background-repeat: no-repeat;
    background-size: cover;
}

@mixin colCenter(){
    float: none;
    margin-right: auto;
    margin-left: auto;
}

@mixin transition($transition-time: .25s, $transition-property: all, $method: ease-in-out) {
    transition: $transition-property $transition-time $method;
}

@mixin elemCenter(
	$position: absolute,
	$left: 50%,
	$top: 50%
	){
	position: $position;
    left: $left;
    top: $top;
    transform: translate(-50%,-50%);
}

@mixin placeholder(){
    &:-moz-placeholder {
        @content;
    }
    &::-moz-placeholder {
        @content;
    } // Firefox 19+
    &:-ms-input-placeholder {
        @content;
    } // Internet Explorer 10+
    &::-webkit-input-placeholder {
        @content;
    } // Safari and Chrome
}

@mixin gradient(){
  background: -moz-linear-gradient(top, rgba(0,0,0,0) 0%, rgba(0,0,0,0) 1%, rgba(0,0,0,0.65) 100%); /* FF3.6-15 */
  background: -webkit-linear-gradient(top, rgba(0,0,0,0) 0%,rgba(0,0,0,0) 1%,rgba(0,0,0,0.65) 100%); /* Chrome10-25,Safari5.1-6 */
  background: linear-gradient(to bottom, rgba(0,0,0,0) 0%,rgba(0,0,0,0) 1%,rgba(0,0,0,0.65) 100%); /* W3C, IE10+, FF16+, Chrome26+, Opera12+, Safari7+ */
  filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#00000000', endColorstr='#a6000000',GradientType=0 ); /* IE6-9 */
}
