<div class="row">
  <div class="about--info col-lg-6 align-self-center">
    <div class="borde p-4">
      <div class="articles--title">
        <h3 class="heading-3">{{ module.black_title }}</h3>
        <h3 class="heading-3 txt-primary">{{ module.color_title }}</h3>
      </div>
      <hr class="hr-primary bk-primary">
      <p>{{ module.text }}</p>
    </div>
    <div class="about--info--vermas">
      <a href="{{function('get_permalink',8)}}" class="txt-primary">Más info<i class="fas fa-long-arrow-alt-right txt-primary ml-2"></i></a>
    </div>
  </div>
  <div class="about--article col-lg-6">
    <a href="{{function('get_permalink',8)}}">
      <img src="{{ module.last_magazine.thumbnail.src }}" alt="{{ module.last_magazine.thumbnail.alt }}" class="w-100">
    </a>
  </div>
</div>
