{% extends "fm/base.twig" %}

{% block content %}

  {% include 'fm/partials/heroes/hero-internal.twig' %}
  <div class="container">
    <div class="single-txt row"> 
      <div class="col-lg-9">
        {% include 'fm/partials/elements/post-content.twig' %}
        {% include 'fm/partials/elements/subscription-banner.twig' %}
      </div>
      <div class="col-lg-3">
        {{ function(
            [ 'CDController', 'init_controller'],
            'module',
            {
              template: 'fm_advertising',
              args: {
                direction: 'vertical',
              }
            }
        ) }}
      </div> 
    </div>
  </div>

  {{ function(
      [ 'CDController', 'init_controller'],
      'module',
      {
        template: 'fm_other_numbers',
        args: {
          magazine_id: post.post_parent,
        }
      }
  ) }}

{% endblock %}
