{% extends "fm/base.twig" %}

{% block content %}
  {% include 'fm/partials/heroes/hero-internal.twig' %}
  {% include 'fm/partials/magazines/magazine-grid.twig' with {'posts': posts} %}
  {% include 'fm/partials/sections/about-subscription.twig' %}
  {{ function(
      [ 'CDController', 'init_controller'],
      'module',
      {
        template: 'fm_advertising',
        args: {
          direction: 'horizontal',
        }
      }
  ) }}
{% endblock %}
