<html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta charset="utf-8">
</head>

<table style="width: 640px;font-family:Arial;" border="0" cellspacing="0" cellpadding="0" align="center">
	<tbody>
	   {% include 'fm/notifications/header.twig' %}
    <tr>
      <td style="font-family:Arial;font-size:18px; width: 100%;padding: 0px 20px; font-weight:bolder; color:#31363A;">
      	<h3>Bienvenido {{user_data.first_name}}</h3>
      </td>
    </tr>
		<tr>
			<td style="font-family:Arial;color:#999999;font-size:30px; width: 100%;padding: 0px 20px; font-weight:bolder;color:#31363A; padding-top: 30px;">
				<p>Ya puede disfrutar de nuestro sitio.</p>
			</td>
		</tr>
    <tr>
			<a href="{{ site.url }}" style="color:#FFFFFF;background-color:#000000;padding:10px 15px;text-decoration:none;border-radius:6px;">Ir al sitio</a>
		</tr>
	</tbody>
</table>
</html>
