.header {
 &--options {
   position: absolute;
   right: 27px;
   top: 0;
   &--mobile {
     position: relative;
     right: 0;
     a {
       color: white !important;
     }
   }
  .active {
    color: #BDB9B8 !important;
  }
  a {
    font-size: 11px;
    color: black;
    font-weight: bold;
  }
}
}
.navbar-light {
  background: white;
}
.nav-link {
  font-size: 11px;
  font-weight: bold;
  color: white !important;
  @include media-breakpoint-up(lg) {
    color: black !important;
  }
  text-transform: uppercase;
}
.link-active {
  color: $red !important;
}
.bg-contract-black {
  background: black;
  margin-right: -16px;
  margin-left: -16px;
  margin-bottom: -8px;
  padding: 20px;
  @include media-breakpoint-up(lg) {
    background: white;
    margin-right: 0;
    margin-left: 0;
    margin-bottom: 0;
    padding: 0;
  }
}

.dropdown-menu {
  background-color: black;
  @include media-breakpoint-up(lg) {
    background-color: white;
    max-width: 130px;
    left: -60%;
  }
  a{
    padding: 0.25rem 0.5rem;
    font-size: 11px;
    white-space: normal;
    text-align: center;
  }
}

.dropdown-item {
  text-align: center;
  color: white;
    @include media-breakpoint-up(lg) {
      text-align: left;
      color: black;
    }
}
