// HEADINGS
.heading {
  line-height: 1;
  color: inherit;
  strong, b {
    font-weight: 700;
  }
}

h1,
h2,
h3,
h4,
h5,
h6 {
  @extend .heading;
}
a {
  color: inherit;
}
// HEADINGS
.heading-1{
  font-size: 45px;
}
.heading-2{
  font-size: 50px;
}
.heading-3{
  font-size: 25px;
}
.heading-4{
  font-size: 28px;
}
.heading-5{
  font-size: 14px;
}

// PARAGRAPHS
p {
    font-size: 18px;
    font-weight: normal;
    line-height: 24px;
    color: inherit;
    margin: 0;
}
hr.hr-white {
  background: white;
  width: 40px;
  display: inline-block;
  height: 3px;
}
hr.hr-black {
  background: black;
  width: 40px;
  display: inline-block;
  height: 3px;
}
hr.hr-primary {
    width: 40px;
    display: inline-block;
    height: 3px;
}
.bg-black {
  background: black !important;
}
.btn-black {
  background-color: black !important;
}
.borde {
  border: 4px solid #D8D8D8;
}
.font-normal {
  font-weight: normal !important;
}
.font-medium {
  font-weight: 500 !important;
}
.font-light {
  font-weight: 300 !important;
}
.font-black {
  font-weight: 900;
}

.font-regular {
  font-weight: normal !important;
}
@include media-breakpoint-down(lg){

}

@include media-breakpoint-down(md){
  .heading-1{
    font-size: 38px;
  }
  .heading-2{
    font-size: 40px;
  }
  .heading-3{
    font-size: 20px;
  }
}

@include media-breakpoint-down(sm){
  .heading-1{
    font-size: 30px;
  }
  .heading-2{
    font-size: 40px;
  }
  .heading-3{
    font-size: 18px;
  }
  .heading-4{
    font-size: 21px;
  }
  p{
    font-size: 14px;
  }
}
