/*!
Theme Name: Creativedog | Timber Theme
Author: Creativedog Agency
Author URI: http://creativedog.agency
Description: Creativedog is a very professional yet casual agency. We provide consultancy services for web development and design within a digital marketing context. Innovation, expertise, and personalized advisory services are the key elements that make the difference when providing cutting edge solutions.
Version: 1.0.0
License: GNU General Public License v2 or later
License URI: LICENSE
Text Domain: creativedog_timber_theme
Tags: Custom web development with love by Creativedog.

This theme, like WordPress, is licensed under the GPL.
Use it to make something cool, have fun, and share what you've learned with others.
*/
