<header class="header">
  <nav class="navbar navbar-expand-lg navbar-light">
    <div class="header--options d-none d-lg-block">
      <a href="https://www.contractworkplaces.com/Intranet_2016/intranet-login.php" rel="nofollow" class="active">Login Intranet clientes</a>
      <a href="https://contractworkplaces.com/en">English Site</a>
    </div>
    <a class="navbar-brand" href="https://contractworkplaces.com/web/">
      <img src="{{ theme.path }}/assets/images/logo-contract.png" alt="Contract Workplaces">
    </a>
    <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarNavDropdown" aria-controls="navbarNavDropdown" aria-expanded="false" aria-label="Toggle navigation">
      <span class="navbar-toggler-icon"></span>
    </button>
    <div class="collapse navbar-collapse justify-content-end bg-contract-black" id="navbarNavDropdown">
      <ul class="navbar-nav text-center text-lg-left">
        {# <li class="nav-item">
          <a class="nav-link link-active" href="https://contractworkplaces.com/web/">Inicio</a>
        </li> #}
        <li class="nav-item dropdown">
          <a class="nav-link dropdown-toggle" href="#" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            Nosotros
          </a>
          <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
            <a class="dropdown-item" href="https://contractworkplaces.com/web/introduccion/">INTRODUCCIÓN</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/equipo/">EQUIPO</a>
            {# <a class="dropdown-item" href="https://contractworkplaces.com/web/historia-y-presencia-regional/">HISTORIA Y PRESENCIA REGIONAL</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/noticias-y-novedades/">NOTICIAS Y NOVEDADES</a> #}
          </div>
        </li>
        <li class="nav-item dropdown">
          <a class="nav-link dropdown-toggle" href="#" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            Proyectos
          </a>
          <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
            <a class="dropdown-item" href="https://contractworkplaces.com/web/casos-destacados/">OBRAS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/testimonios/">TESTIMONIOS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/encuesta-de-satisfaccion-al-cliente-2018/">ENCUESTA DE SATISFACCIÓN</a>
          </div>
        </li>
        <li class="nav-item">
          <a class="nav-link" href="https://contractworkplaces.com/web/servicios/">Servicios</a>
        </li>
        <li class="nav-item dropdown">
          <a class="nav-link dropdown-toggle" href="#" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            Tendencias
          </a>
          <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
            <a class="dropdown-item" href="https://contractworkplaces.com/worktech/" rel="nofollow">WORKTECH</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/site/">FM & WORKPLACES</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/blog/">NEWS & TRENDS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/indices-mercados/">ÍNDICE CONTRACT</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/videos/">VIDEOS</a>
            
          </div>
        </li>
        {# <li class="nav-item dropdown">
          <a class="nav-link dropdown-toggle" href="#" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            Conocimiento
          </a>
          <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
            <a class="dropdown-item" href="https://contractworkplaces.com/web/indices-mercados/">ÍNDICE CONTRACT</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/blog/">BLOG</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/videos/">VIDEOS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/site/">FM & WORKPLACES</a>
          </div>
        </li> #}
        {# <li class="nav-item">
          <a class="nav-link" href="https://contractworkplaces.com/worktech/" rel="nofollow">Worktech</a>
        </li>
        <li class="nav-item">
          <a class="nav-link" href="https://contractworkplaces.com/recreo/" rel="nofollow">Recreo</a>
        </li> #}
        <li class="nav-item dropdown">
          <a class="nav-link dropdown-toggle" href="#" id="navbarDropdownMenuLink" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
            Contacto
          </a>
          <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
            <a class="dropdown-item" href="https://contractworkplaces.com/web/contacto-contract-workplaces/#contactanos">ESCRÍBENOS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/contacto-contract-workplaces/">DONDE ESTAMOS</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/envianos-tu-cv-hoja-de-vida/">QUIERO TRABAJAR EN CONTRACT WORKPLACES</a>
            <a class="dropdown-item" href="https://contractworkplaces.com/web/sea-proveedor/">QUIERO SER PROVEEDOR</a>
          </div>
        </li>
        <li>
          <div class="header--options header--options--mobile d-block d-lg-none">
            <a href="https://www.contractworkplaces.com/Intranet_2016/intranet-login.php" rel="nofollow" class="active">Login Intranet clientes</a>
            <a href="https://contractworkplaces.com/en">English Site</a>
          </div>
        </li>
      </ul>
    </div>
  </nav>
</header>
