{% extends "fm/base.twig" %}

{% block content %}

  <div class="container">
    <div class="single-txt row">
      <div class="col-lg-12">
        <div class="single">
          <div class="single--detail">
            <h3 class="heading-3">Migración de usuarios</h3>
            <hr class="hr-primary bk-primary">
          </div>
          <div class="single--txt">
            Totales : {{suscribers_total}}
          </div>
          <div class="single--txt">
            Correctos : {{suscribers_fine}}
          </div>
          <div class="single--txt">
            Con problemas : {{suscribers_problem | length}}
          </div>
          <div class="single--txt">

              <table class="table">
                <thead>
                  <tr>
                    <th scope="col">Id</th>
                    <th scope="col">email</th>
                    <th scope="col">Nombre</th>
                    <th scope="col">Apellido</th>
                  </tr>
                </thead>
                <tbody>
                  {% for subscriber in suscribers_problem %}
                  <tr>
                    <td>{{subscriber.Id}}</td>
                    <td>{{subscriber.Nombre}}</td>
                    <td>{{subscriber.Apellido}}</td>
                    <td>{{subscriber.email}}</td>
                  </tr>
                  {% endfor %}
                </tbody>
            </table>
          </div>
        </div>

      </div>
    </div>
  </div>

{% endblock %}
