
<form role="search" method="get" action="{{ module.action }}">
  <div class="row">
    <div class="col-lg-12">
      <div class="form-group">
        <label for="search">Búscar*</label>
        <input type="text" name="query" class="form-control" id="search">
      </div>
    </div>
    <div class="col-lg-12">
      <label for="search">Categoría</label>
      <select name="category" class="form-control">
        <option value="">Todas las categorías</option>
        {% for category in module.fm_categories %}
        <option value="{{category.id}}">{{category.name}}</option>
        {% endfor %}
      </select>
    </div>
    <div class="col-lg-12 mt-3">
      <div class="bk-primary float-left text-white">
        <button type="submit" class="text-white">Búscar</button>
      </div>
    </div>
  </div>
</form>
