<nav aria-label="Page navigation example">
  <ul class="pagination">
    {% if pagination.prev %}
    <li class="page-item">
      <a class="page-link" href="{{pagination.prev.link}}" aria-label="Previous">
        <span aria-hidden="true">&laquo;</span>
        <span class="sr-only">Previous</span>
      </a>
    </li>
    {% endif %}
    {% for page in pagination.pages %}
    <li class="page-item">
      <a
        class="page-link {% if page.current %}active{% endif %}"
        {% if not page.current %} href="{{page.link}}" {% endif %}
      >{{page.title}}
      </a>
    </li>
    {% endfor %}
    {% if pagination.next %}
    <li class="page-item">
      <a class="page-link" href="{{pagination.next.link}}" aria-label="Next">
        <span aria-hidden="true">&raquo;</span>
        <span class="sr-only">Next</span>
      </a>
    </li>
    {% endif %}
  </ul>
</nav>
