<?php

/*
 * This file is part of Twig.
 *
 * (c) Fabien Potencier
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Represents a body node.
 *
 * @author Fabien Potencier <fabien@symfony.com>
 */
class Twig_Node_Body extends Twig_Node
{
}

class_alias('Twig_Node_Body', 'Twig\Node\BodyNode', false);
