$('.open-menu').on('click', function(){
    $('.hero--navigation--menu').addClass('fadeInLeft');
    $(".close-menu").css("display", "block");
  });

  $('.close-menu').on('click', function(){
    $('.hero--navigation--menu').removeClass('fadeInLeft');
  });

// CAROUSELS

$('.slick-articles').slick({
  slidesToShow: 3,
  slidesToScroll: 3,
  autoplay: true,
  speed: 600,
  autoplaySpeed: 2000,
  prevArrow: '<button class="slick-prev" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-left"></i></button>',
  nextArrow: '<button class="slick-next" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-right"></i></button>',
  responsive: [
    {
      breakpoint: 768,
      settings: {
        slidesToShow: 1,
        slidesToScroll: 1
      }
    }
  ]
});

$('.slick-numbers').slick({
  slidesToShow: 3,
  slidesToScroll: 3,
  autoplay: true,
  speed: 600,
  autoplaySpeed: 2000,
  prevArrow: '<button class="slick-prev" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-left"></i></button>',
  nextArrow: '<button class="slick-next" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-right"></i></button>',
  responsive: [
    {
      breakpoint: 768,
      settings: {
        slidesToShow: 2,
        slidesToScroll: 1,
      }
    }
  ]
});

$('.slick-advertiser').slick({
  slidesToShow: 3,
  slidesToScroll: 3,
  autoplay: true,
  speed: 600,
  autoplaySpeed: 2000,
  prevArrow: '<button class="slick-prev" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-left"></i></button>',
  nextArrow: '<button class="slick-next" aria-label="Previous" type="button" style=""><i class="fas fa-long-arrow-alt-right"></i></button>',
  responsive: [
    {
      breakpoint: 768,
      settings: {
        slidesToShow: 2,
        slidesToScroll: 1,
      }
    }
  ]
});

//open modal anunciante
$("#btn-adv").click(function(){
  $("#anunciante").modal();
});
