
<div class="single">
  <div class="single--detail">
    <nav aria-label="breadcrumb">
      <ol class="breadcrumb list-unstyled">
        <li class="breadcrumb-item"><a href="{{function('get_post_type_archive_link','fm_magazine')}}">Revistas</a></li>
        <li class="breadcrumb-item"><a href="{{function('get_permalink',post.post_parent)}}">{{post.hero_lead}}</a></li>
        <li class="breadcrumb-item active" aria-current="page">{{post.title}}</li>
      </ol>
    </nav>
    {% if post.author_name %}
    <h3 class="heading-3">Por {{post.author_name}}</h3>
    {% endif %}
    <hr class="hr-primary bk-primary">
  </div>
  <div class="single--txt">
    {{post.content}}
  </div>
</div>
