<?php
class Page404 extends CDController{

  public function __construct() {
    $this->view = 'page/404.twig';
  }

  public function get_context(){

    $this->data = CDTimber::get_context();

    $hero_404 = get_field('option_404_hero', 'option');
    $this->data['post'] = (object)array();
    $this->data['post']->hero_image = $hero_404['image'];
    $this->data['post']->hero_lead = $hero_404['lead'];
    $this->data['post']->hero_title = $hero_404['title'];
    $this->data['post']->hero_subtitle = $hero_404['subtitle'];

  }

}
