<div class="articles"> 
  <div class="container-fluid">
    <div class="row">
      <div class="articles--actual col-lg-3 align-self-center">
        <div class="articles--title">
          <h3 class="heading-3">Artículos</h3>

          <h3 class="heading-3">#{{post.magazine_number}} | {{ post.post_date | date("F Y") }}</h3>
          <hr class="hr-primary bk-primary">
        </div>
        <p>{{post.posts_summary }}</p>
        {% if post.pdf_magazine and user %}
        <div class="article--btn btn p-0">
          <a href="{{function('wp_get_attachment_url', post.pdf_magazine)}}" download>
            <button type="button" name="button" class="text-white"><i class="far fa-file-pdf mr-2"></i>Descargar</button>
          </a>
        </div>
        {% endif %}
      </div>
      <div class="col-lg-9">
        {% include 'fm/partials/carousels/carousel-articles.twig' with {'posts':magazine_posts, 'is_current':is_current} %}
      </div>
    </div>
  </div>
</div>
