<div class="bk-primary">
  <div class="about--submit--info">
    <h3 class="heading-3">{{module.title}}</h3>
    <hr class="hr-white">
    <p>{{module.text}}</p>
    <a href="{{function('get_permalink',18)}}">
      <button type="button" name="button" class="mt-4">Suscribirme ahora</button>
    </a>
  </div>
  <div
    class="about--submit--workplaces"
    style="background-image: url({{module.image.url}})"
  >
    <div class="about--submit--workplaces--text">
      <p>{{module.vertical_text}}</p>
    </div>
  </div>
</div>
