<?php

namespace Twig\Node\Expression\Binary;

class_exists('Twig_Node_Expression_Binary_EndsWith');

if (\false) {
    class EndsWithBinary extends \Twig_Node_Expression_Binary_EndsWith
    {
    }
}
