{% if menu %}
	<ul>
	{% for item in menu %}
		<li class="{{ item.classes | join(' ') }}">
			<a target="{{ item.target }}" href="{{ item.link }}">{{ item.title }}</a>
			{% include "menu.twig" with {'menu': item.get_children} %}
		</li>
	{% endfor %}
	</ul>
{% endif %}
