(function($) {
    $('[name="download_from"]').on('change', function(){
        $('[name="download_to"]').attr('min', $(this).val());
    });

    $('[name="download_to"]').on('change', function(){
        $('[name="download_from"]').attr('max', $(this).val());
    });
})( jQuery );