/*
 Theme Name:   Twenty Thirteen Child
 Theme URI:    http://example.com/twenty-fourteen-child/
 Description:  Twenty Thirteen Child Theme
 Author:       John Doe
 Author URI:   http://example.com
 Template:     twentyfifteen
 Version:      1.0.0
 Tags:         light, dark, two-columns, right-sidebar, responsive-layout, accessibility-ready
 Text Domain:  twenty-thirteen-child
*/

/* =Theme customization starts here
-------------------------------------------------------------- */
