<div class="submit-form">
  <h3 class="heading-3 text-uppercase">Contáctate con nosotros</h3>
  <hr class="hr-primary bk-primary">
  {% include 'fm/partials/forms/form-contact.twig' %}
  <h3 class="heading-3 text-uppercase">¿Quieres ser anunciante?</h3>
  <hr class="hr-primary bk-primary">
  {% include 'fm/partials/forms/form-advertiser.twig' %}
</div>
