{% extends "fm/base.twig" %}

{% block content %}
  {% include 'fm/partials/heroes/hero-internal.twig' %}
  <div class="container">
    <div class="single-txt row justify-content-center">
      <div class="col-lg-8">
        {% include 'fm/partials/sections/subscription-form.twig' %}
      </div>
    </div>
  </div>

{% endblock %}
