// SITE FONTS
@font-face {
    font-family: 'Brandon Grotesque';
    src: url('../fonts/BrandonGrotesque-Black.eot');
    src: local('Brandon Grotesque Black'), local('BrandonGrotesque-Black'),
        url('../font/BrandonGrotesque-Black.eot?#iefix') format('embedded-opentype'),
        url('../font/BrandonGrotesque-Black.woff2') format('woff2'),
        url('../font/BrandonGrotesque-Black.woff') format('woff'),
        url('../font/BrandonGrotesque-Black.ttf') format('truetype'),
        url('../font/BrandonGrotesque-Black.svg#BrandonGrotesque-Black') format('svg');
    font-weight: 900;
    font-style: normal;
}

@font-face {
    font-family: 'Brandon Grotesque';
    src: url('../font/BrandonGrotesque-Medium.eot');
    src: local('Brandon Grotesque Medium'), local('BrandonGrotesque-Medium'),
        url('../font/BrandonGrotesque-Medium.eot?#iefix') format('embedded-opentype'),
        url('../font/BrandonGrotesque-Medium.woff2') format('woff2'),
        url('../font/BrandonGrotesque-Medium.woff') format('woff'),
        url('../font/BrandonGrotesque-Medium.ttf') format('truetype'),
        url('../font/BrandonGrotesque-Medium.svg#BrandonGrotesque-Medium') format('svg');
    font-weight: 500;
    font-style: normal;
}

@font-face {
    font-family: 'Brandon Grotesque';
    src: url('../font/BrandonGrotesque-Regular.eot');
    src: local('Brandon Grotesque Regular'), local('BrandonGrotesque-Regular'),
        url('../font/BrandonGrotesque-Regular.eot?#iefix') format('embedded-opentype'),
        url('../font/BrandonGrotesque-Regular.woff2') format('woff2'),
        url('../font/BrandonGrotesque-Regular.woff') format('woff'),
        url('../font/BrandonGrotesque-Regular.ttf') format('truetype'),
        url('../font/BrandonGrotesque-Regular.svg#BrandonGrotesque-Regular') format('svg');
    font-weight: normal;
    font-style: normal;
}

@font-face {
    font-family: 'Brandon Grotesque';
    src: url('../font/BrandonGrotesque-Light.eot');
    src: local('Brandon Grotesque Light'), local('BrandonGrotesque-Light'),
        url('../font/BrandonGrotesque-Light.eot?#iefix') format('embedded-opentype'),
        url('../font/BrandonGrotesque-Light.woff2') format('woff2'),
        url('../font/BrandonGrotesque-Light.woff') format('woff'),
        url('../font/BrandonGrotesque-Light.ttf') format('truetype'),
        url('../font/BrandonGrotesque-Light.svg#BrandonGrotesque-Light') format('svg');
    font-weight: 300;
    font-style: normal;
}

@font-face {
    font-family: 'Brandon Grotesque';
    src: url('../font/BrandonGrotesque-Bold.eot');
    src: local('../font/Brandon Grotesque Bold'), local('BrandonGrotesque-Bold'),
        url('../font/BrandonGrotesque-Bold.eot?#iefix') format('embedded-opentype'),
        url('../font/BrandonGrotesque-Bold.woff2') format('woff2'),
        url('../font/BrandonGrotesque-Bold.woff') format('woff'),
        url('../font/BrandonGrotesque-Bold.ttf') format('truetype'),
        url('../font/BrandonGrotesque-Bold.svg#BrandonGrotesque-Bold') format('svg');
    font-weight: bold;
    font-style: normal;
}


$primary-font: 'Brandon Grotesque', sans-serif;

//SITE COLOR PALLETE
$red: #DF1737;
$grey: #3C3C3B;
$light-grey: #F3F3F3;


//estamos probando el gulp
body{
    font-family: $primary-font;
}
