<div class="row justify-content-center">
  <div class="col-lg-5 borde p-4">
    <div class="modulo-login">

      <div class="modulo-login--form">
    
        <h3 class="heading-3 font-black">{{module.login_title.black}}</h3>
        <h3 class="heading-3 font-black txt-primary">{{module.login_title.color}}</h3>
        <hr class="hr-primary bk-primary">
        {{ function(
            [ 'CDController', 'init_controller'],
            'form',
            { template: 'fm_login'}
        ) }}
        <div class="login--acount text-center mt-3">
          <p>Si no tenés cuenta<a href="{{function('get_permalink',18)}}" class="ml-1 text-underline">suscribite</a></p>
          <a href="#">¿Desea recuperar la contraseña?</a>
        </div>
      </div>
    </div>
  </div>
  <div class="col-lg-5 align-self-center">
    <div class="bk-primary text-white p-4">
      <div class="">
        <h3 class="heading-3 font-black">{{module.register_title}}</h3>
          <hr class="hr-white">
          <p class="font-light">{{module.register_text}}</p>
          <a href="{{function('get_permalink',18)}}">
            <button type="button" name="button" class="mt-4 btn-black text-white">Suscribirme ahora</button>
          </a>
        </div>
      </div>
    </div>
  </div>
</div>
