<div class="footer"> 
  <div class="container">
    <footer>
      <div class="row text-center text-lg-left">
        <div class="col-lg-3"> 
          <h3>Contáctenos</h3>
          <ul>
            <li><a href="mailto:{{site_options.contact_data.email}}">
              {{site_options.contact_data.email}}</a>
            </li>
            <li><a href="tel:{{site_options.contact_data.phone.link}}">
              {{site_options.contact_data.phone.text}}</a>
            </li>
          </ul>
          <div class="footer--share">
            <ul class="list-unstyled">
              {% if site_options.social_media.facebook %}
              <li><a href="{{site_options.social_media.facebook}}" class="bk-primary">
                <i class="fab fa-facebook-f"></i>
              </a></li>
              {% endif %}
              {% if site_options.social_media.twitter %}
              <li><a href="{{site_options.social_media.twitter}}" class="bk-primary">
                <i class="fab fa-twitter"></i>
              </a></li>
              {% endif %}
            </ul>
          </div>
        </div>
        <div class="col-lg-3 mb-4"> 
          <ul>
            {% set last_magazine = function('cw_get_last_magazine') %}
            <li><a href="{{last_magazine.link}}">Edición actual</a></li>
            <li><a href="{{function('get_post_type_archive_link','fm_magazine')}}">Otros números</a></li>
          </ul>        
        </div>
        <div class="col-lg-3 mb-4">
          <ul>
            {% set last_magazine = function('cw_get_last_magazine') %}
            <li><a href="{{function('get_permalink',8)}}">Acerca de FM & Workplaces</a></li>
            <li><a href="{{function('get_permalink',18)}}">Suscribirme al Newsletter</a></li>
          </ul>
        </div>
        <div class="col-lg-3 mb-4">
          <ul>
            <li><a href="#" data-toggle="modal" data-target="#anunciante">Publicidad</a></li>
            <li><a href="{{function('get_permalink',10)}}">Contacto</a></li>
          </ul>
        </div>
      </div>
    </footer>
  </div>
  <div class="footer--copy col-lg-12 text-center text-lg-left">
    <div class="container-fluid">
      <p><span>FM&Workplaces 2019© Todos los derechos reservados | Diseño y desarrollo por <a class="text-primary" href="https://creativedog.agency" rel="nofollow" target="_blank">Creativedog Agency</a></span></p>
    </div>
  </div>
</div>
