<html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta charset="utf-8">
</head>

<table style="width: 640px;font-family:Arial;" border="0" cellspacing="0" cellpadding="0" align="center">
	<tbody>
	   {% include 'fm/notifications/header.twig' %}
     <tr>
 			<td style="font-family:Arial;font-size:18px; width: 100%;padding: 0px 20px; font-weight:bolder; color:#31363A;">
 				<h3>Ud ha solicitado recuperar su contraseña:</h3>
 			</td>
 		</tr>
		<tr>
			<td style="font-family:Arial;font-size:18px; width: 100%;font-weight:bolder;color:#31363A;">
				<p>Su nueva contraseña es: <strong>{{new_password}}</strong></p>
			</td>
		</tr>
	</tbody>
</table>
</html>
