<form id="form-restore-password">
  <div class="form-group">
    <label for="email">Email*</label>
    <input type="email" name="email" class="form-control">
  </div>
  <div class="text-center mb-3">
    <button type="submit" class="btn-block btn-lg bk-primary text-white">Enviar</button>
  </div>

  <div class="form-message sending-message" style="display:none;">
    <div class="col alert alert-info" role="alert">
     <p>Enviando información...</p>
    </div>
  </div>
  <div class="form-message error-message" style="display:none;">
    <div class="col alert alert-danger" role="alert">
     <p></p>
    </div>
  </div>
  <div class="form-message success-message" style="display:none;">
    <div class="col alert alert-success" role="alert">
     <p>Le hemos enviado una nueva contraseña, por favor revise su casilla de correo.</p>
    </div>
  </div>
</form>
