.aligned-row {
    display: flex;
    flex-flow: row wrap;

    &::before {
        display: block;
    }
}

.container-custom {
    width: 100%;
    max-width: 100%;
    margin-left: auto;
    margin-right: auto;
    padding-left: 15px;
    padding-right: 15px;
    position: relative;
    @include media-breakpoint-up(md) {
      padding-left: 15px;
      padding-right: 15px;
    }
    @include media-breakpoint-up(lg) {
      width: 1280px;
    }
    @include media-breakpoint-up(xl) {
      width: 1600px;
    }
}
