<?php
class Post extends CDController{

    public function __construct() {
      if ( is_single() ){
  			$this->view = 'post/single.twig';
  		}
      if ( is_home() ){
  			$this->view = 'post/archive.twig';
  		}

    }

}
