{% set form_text =  post.meta('form_text') %}
<div class="submit-form">
  <h3 class="heading-3">{{form_text.title}}</h3>
  <hr class="hr-primary bk-primary">
  <div class="submit-form--txt">
    <p>{{form_text.first_text}}</p>
  </div>
  <form id="form-subscription">
    <div class="row">
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputnombre">Nombre*</label>
          <input type="text" name="first_name" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputApellido">Apellido*</label>
          <input type="text" name="last_name" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputprof">Profesión*</label>
          <input type="text" name="profession" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputEmpresa">Empresa*</label>
          <input type="text" name="company" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputCargo">Cargo*</label>
          <input type="text" name="position" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputTel">Teléfono*</label>
          <input type="text" name="phone" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-12">
        <div class="form-group">
          <label for="exampleInputemail">Email*</label>
          <input type="email" name="email" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputCargo">Contraseña*</label>
          <input type="password" name="password" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputTel">Repetir Contraseña*</label>
          <input type="password" name="repeat_password" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-12">
        <div class="submit-form--txt my-4">
          <p>{{form_text.second_text}}</p>
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputdomicilio">Domicilio*</label>
          <input type="text" name="address" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-6">
        <div class="form-group">
          <label for="codigoPostal">Código Postal*</label>
          <input type="text" name="postal_code" class="form-control data-field">
        </div>
      </div>

      <div class="col-lg-4">
        <div class="form-group">
          <label for="localidad">Ciudad*</label>
          <input type="text" name="city" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-4">
        <div class="form-group">
          <label for="exampleInputnombre">Región*</label>
          <input type="text" name="region" class="form-control data-field">
        </div>
      </div>
      <div class="col-lg-4">
        <label for="exampleInputnombre">País*</label>
        <select name="country" class="form-control data-field">
          <option value="">Seleccione un país</option>
          {% for key,choice in form.country_choices %}
            <option value="{{key}}">{{choice}}</option>
          {% endfor %}
        </select>
      </div>
      <div class="col-lg-12">
        <div class="form-group form-check mb-4 mt-2">
          <input type="checkbox" name="is_working_address" class="form-check-input true-false data-field">
          <label class="form-check-label font-regular" for="exampleCheck1">Marque esta casilla si este es su domicilio laboral</label>
        </div>
      </div>

      <div class="col-lg-6 employees">
        <label class="form-check-label mb-2 font-medium" for="exampleCheck1">Tipo de actividad de su empresa</label>
        <select name="company_area" class="form-control mb-5 data-field">
          <option value="">Seleccione un tipo de actividad</option>
          {% for key,choice in form.company_area_choices %}
            <option value="{{key}}">{{choice}}</option>
          {% endfor %}
        </select>

        <div class="form-group other-area-container" style="display: none;">
          <label for="localidad">Otra actividad</label>
          <input type="text" name="company_area_other" class="form-control data-field">
        </div>

        <label class="form-check-label font-medium mb-2" for="exampleCheck1">Cantidad de empleados</label>
        {% for key,choice in form.employees_quantity_choices %}
        <div class="form-check">
          <input class="form-check-input data-field" type="radio" name="employees_quantity" value="{{key}}">
          <label class="form-check-label" for="exampleRadios1">
            {{choice}}
          </label>
        </div>
        {% endfor %}
      </div>

      <div class="col-lg-6">
        <div class="form-group">
          <label for="exampleInputnombre">Principal producto o servicio</label>
          <input type="text" name="main_product" class="form-control data-field">
        </div>
        <label class="form-check-label font-medium mt-4 mb-4" for="exampleCheck1"> ¿Participa usted en los procesos de decisión relacionados a los espacios de trabajo de su compañía?</label>
        <div class="form-check">
          <input class="form-check-input data-field is-boolean" type="radio" name="company_participation" value="1">
          <label class="form-check-label font-regular" for="exampleRadios1">
            Si
          </label>
        </div>
        <div class="form-check">
          <input class="form-check-input data-field is-boolean" type="radio" name="company_participation" value="0">
          <label class="form-check-label font-regular" for="exampleRadios1">
            No
          </label>
        </div>
      </div>
    </div>

    <div class="bk-primary btn text-white my-4">
      <button type="submit" name="button">Suscribirme ahora</button>
    </div>

    <div class="row form-message sending-message" style="display:none;">
      <div class="col alert alert-info" role="alert">
       <p>Enviando información...</p>
      </div>
    </div>
    <div class="row form-message error-message" style="display:none;">
      <div class="col alert alert-danger" role="alert">
       <p></p>
      </div>
    </div>
    <div class="row form-message success-message" style="display:none;">
      <div class="col alert alert-success" role="alert">
       <p>Suscripción completa! Redireccionando...</p>
      </div>
    </div>
  </form>

  <div class="submit-form--aclaracion">
    <p>(1) Los datos correspondientes a la información y documentación brindada en razón de la presente, que podrán ser cedidos a terceros, podrán ser utilizados por Facility Magazine & Workplaces a los efectos de planear y realizar acciones de marketing promocional, ofrecer productos o servicios propios o de terceros; respecto de todo lo cual se presta el libre, expreso e informado consentimiento.</p> 
  </div>
</div>
