{% extends "base.twig" %}

{% block content %}

{% endblock %}

{% block footer %}
  {% include 'fm/partials/modals/modal-login.twig' %}
  {% include 'fm/partials/modals/modal-contact.twig' %}
  {% include 'fm/partials/modals/modal-advertiser.twig' %}
  {% include 'fm/partials/modals/modal-search.twig' %}
  {% include 'fm/partials/modals/modal-restore-password.twig' %}
  {% include 'fm/footer.twig' %}
{% endblock %}
