<div class="about">
  <div class="container-fluid">
    <div class="row">
      <div class="col-lg-6">
        {{ function(
            [ 'CDController', 'init_controller'],
            'module',
            { template: 'fm_about'}
        ) }}
      </div>
      <div class="about--submit col-lg-6">
        {{ function(
            [ 'CDController', 'init_controller'],
            'module',
            { template: 'fm_subscribe'}
        ) }}
      </div>
    </div>
  </div>
</div>
