{% extends 'base.twig' %}

{% block content %}
	<h2>{{ foo }}</h2>
	<p>{{ qux }}</p>
	{% for post in posts %}
	
	{% endfor %}

	{% include 'partial/pagination.twig' with { pagination: posts.pagination({show_all: false, mid_size: 3, end_size: 2}) } %}
{% endblock %}
