@import '../../../node_modules/bootstrap/scss/bootstrap';
@import '../../../node_modules/slick-carousel/slick/slick.scss';
@import "basics/_mixins.scss";
@import "basics/_variables.scss";
@import "basics/structure.scss";
@import "basics/typography.scss";
@import "partials/footer.scss";
@import "partials/header.scss";
@import "partials/sections.scss";

