
  <div class="btn-white">
    <button type="button" data-toggle="modal" data-toggle="modal" data-target="#login">Ingresar Hero</button>
  </div>
  <div class="btn-blue bk-primary">
    <a  href="{{function('get_permalink',18)}}">
      <button type="button" class="text-white">Suscribirme</button>
    </a>
  </div>
