
<div class="advertising">
  <div class="container">
    <div class="row">
      <div class="advertising--info col-lg-3 borde p-4">
        <h3 class="heading-3 text-uppercase">{{module.title}}</h3>
        <div class="vermas">
          <button type="button" class="bk-primary text-white" data-toggle="modal" data-target="#anunciante">Quiero ser anunciante</button>
        </div>
      </div>
      <article class="slick-advertiser col-lg-9">
        {% for brand in module.brands %}
          <div class="advertising--brand">
            <a href="{% if brand.link %} {{brand.link}} {% else %} javascript:void(0); {% endif %}" rel="nofollow" target="_blank">
              <img src="{{ brand.image.url }}" alt="{{ brand.image.alt }}">
            </a>
          </div>
        {% endfor %}
      </article>
    </div>
  </div>
</div>
