<html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta charset="utf-8">
</head>

<table style="width: 640px;font-family:Arial;" border="0" cellspacing="0" cellpadding="0" align="center">
	<tbody>
	   {% include 'fm/notifications/header.twig' %}
		<tr>
			<td style="font-family:Arial;font-size:18px; width: 100%;padding: 0px 20px; font-weight:bolder; color:#31363A;">
				<h3>Se ha registrado un nuevo usuario con los siguiente datos:</h3>
			</td>
		</tr>
    <tr>
      <td style="font-family:Arial;font-size:14px; width: 100%;color:#63696e;">
        <p>
          <strong>Nombre: </strong>{{user_data.first_name}}<br/>
          <strong>Apellido: </strong>{{user_data.last_name}}<br/>
          <strong>Email: </strong>{{user_data.email}}
        </p>
      </td>
		</tr>
	</tbody>
</table>
</html>
