<div class="adv-vertical">
  <div class="col-lg-12 mb-4">
    <div class="adv-vertical--title">
      <h3 class="heading-3">{{module.title}}</h3>
      <div class="mt-4 text-center">
        <button type="button" data-toggle="modal-anunciante" data-toggle="modal-anunciante" data-target="#anunciante" class="text-white bk-primary btn-block" id="btn-adv">Quiero ser anunciante</button>
      </div>
    </div>
  </div>
 
  {% for brand in module.brands %}
    <div class="vertical">
      <a href="{% if brand.link %} {{brand.link}} {% else %} javascript:void(0); {% endif %}" rel="nofollow" target="_blank">
        <img src="{{ brand.image.url }}" alt="{{ brand.image.alt }}">
      </a>
    </div>
  {% endfor %}

  <script>
    var elmnt = document.querySelector(".single--txt");
    var elmntH = elmnt.offsetHeight;
    var elmnTarg = document.getElementById("verticales");

    elmnTarg.style.height = (elmntH - 300) + 'px';
 
  </script>


  
</div>
