<div class="articles"> 
  <div class="container-fluid">
    <div class="row">
      <div class="articles--actual col-lg-3 align-self-center">
        <div class="articles--title">
          <h3 class="heading-3">Artículos</h3>
          <h3 class="heading-3 txt-primary">Edición actual</h3>
          <hr class="hr-primary bk-primary">
        </div>
        <p>{{post.posts_summary}}</p>
      </div>
      <div class="col-lg-9">
        {% include 'fm/partials/carousels/carousel-articles.twig' with {'posts':last_magazine_posts, 'is_current':true} %}
      </div>
    </div>
  </div>
</div>
