{% extends "fm/base.twig" %}

{% block content %}
  {% include 'fm/partials/heroes/hero-internal.twig' %} 
  
    {% include 'fm/partials/magazines/current-magazine-articles.twig' %}

  {{ function(
      [ 'CDController', 'init_controller'],
      'module',
      {
        template: 'fm_other_numbers',
        args: {
          magazine_id: post.id,
        }
      }
  ) }}

{% endblock %}
