<div class="submit-horizontal bk-primary">
  <div class="row justify-content-between">
    <div class="submit-horizontal--info">
      <p>Suscribite al Newsletter y mantente actualizado sobre todas las novedades de FM & WORKPLACES, una publicación de Contract Workplaces.</p>
    </div>
    <div class="align-self-center">
<a href="/site/fm-workplaces/suscripcion/">
      <button type="button" name="button" class="mt-4">Suscribirme ahora</button>
    </a>
    </div>
  </div>
</div>
