<?php
class ModuleFmAbout extends CDController{

  public function __construct() {

    $this->view = 'fm/partials/sections/module-about.twig';

  }

  public function get_context(){

    $this->data['module'] = get_field('option_module_fm_about', 'option');

    $this->data['module']['last_magazine'] = cw_get_last_magazine();

  }

}
