<ul>
  <li><a href="{{function('get_permalink',6)}}">Inicio</a></li>
  {% set last_magazine = function('cw_get_last_magazine') %}
  <li><a href="{{last_magazine.link}}">Edición actual</a></li>
  <li><a href="{{function('get_post_type_archive_link','fm_magazine')}}">Otros números</a></li>
  <li><a href="{{function('get_permalink',8)}}">Acerca de FM</a></li>
  <li><a href="{{function('get_permalink',10)}}">Contacto</a></li>
</ul>

<!--hr class="hr-white w-100"-->
<!--button class="btn-buscar btn-search text-white" data-toggle="modal" data-target="#modal-buscar"><i class="fa fa-search text-white mr-2"></i>Búscar artículos</button-->
