<style media="screen">
.bk-primary{
  background-color: {{site_options.current_color}};
}
.txt-primary{
  color: {{site_options.current_color}};
}
</style>
<div class="hero--navigation--header col-lg-12 bk-primary px-0">
  <img src="{{ theme.path }}/assets/images/fmlogo.svg" alt="" class="w-100">
</div>
<div class="hero--navigation--mobile d-block d-lg-none">
  <div class="hero--navigation--mobile--burger">
    <a href="javascript:void(0);" class="open-menu">
      <img src="{{ theme.path }}/assets/images/burger-menu.svg" alt="">
    </a>
  </div>
  <div class="hero--navigation--mobile--login">
    {% if user %}
      {% include 'fm/partials/elements/box-logged.twig' %}
    {% else %}
      {% include 'fm/partials/elements/box-not-logged-header.twig' %}
    {% endif %}
  </div>
</div>
<div class="hero--navigation--menu d-none d-lg-block">
  {% include 'fm/partials/elements/navigation.twig' %}
</div>

<!-- NAV MOBILE -->
<div class="d-block d-lg-none">
  <div class="hero--navigation--menu animated bk-primary">
    <div class="hero--navigation--menu--closed">
      <button type="button" name="button" class="close-menu"><img src="{{ theme.path }}/assets/images/btn-closed.svg" alt=""></button>
    </div>
    {% include 'fm/partials/elements/navigation.twig' %}
  </div>
</div>
