<div class="about">
  <div class="container-fluid">
    <div class="row">
      <div class="col-lg-6">
        <div class="row">
          {% include 'fm/partials/elements/box-revista-pdf.twig' %}
        </div>
      </div>
      <div class="about--submit col-lg-6">
        <div class="bk-primary">
          <div class="about--submit--info">
          <h3 class="heading-3">SUSCRIBITE A<br>FM&WORKPLACES</h3>
          <hr class="hr-white">
          <p>[* Editar]Al contrario del pensamiento popular, el texto de Lorem Ipsum no es simplemente texto aleatorio. Tiene sus raices en una pieza clásica de la literatura del Latin, que data del año 45 antes de Cristo.</p>
          <button type="button" name="button" class="mt-4">Suscribirme ahora</button>
        </div>
        <div class="about--submit--workplaces">
          <div class="about--submit--workplaces--text">
            <p>Lorem ipsum dolor sit amet, consectetur adipisicing elit</p>
          </div>
        </div>
      </div>
      </div>
    </div>
  </div>
</div>
