<div class="about--info col-lg-6 align-self-center">
  <div class="borde p-4">
    <div class="articles--title">
      <h3 class="heading-3">Para ver la</h3>
      <h3 class="heading-3 txt-primary">Revista en pdf</h3>
    </div>
    <hr class="hr-primary bk-primary"><br>
    <div class="article--btn btn p-0">
      <button type="button" name="button" class="text-white btn-black"><i class="far fa-file-pdf mr-2"></i>Descargar</button>
    </div>
  </div>
</div>
<div class="about--article col-lg-6">
  <a href="#">
  <img src="{{ theme.path }}/assets/images/numero-3.png" alt="" class="w-100">
  </a>
</div>
