{% extends "fm/base.twig" %}

{% block content %}
  {% include 'fm/partials/heroes/hero-internal.twig' %}
  <div class="container">
    <div class="single-txt row">
      <div class="col-lg-9">
        <div class="single">
          <div class="single--detail">
            <h3 class="heading-3">FM & WORKPLACES</h3>
            <hr class="hr-primary bk-primary">
          </div>
          <div class="single--txt">
            {{post.content}}
          </div>
        </div>

        {% include 'fm/partials/elements/staff.twig' with {'staff': post.meta('staff') } %}

      </div>
      <div class="col-lg-3">
        {{ function(
            [ 'CDController', 'init_controller'],
            'module',
            {
              template: 'fm_advertising',
              args: {
                direction: 'vertical',
              }
            }
        ) }}
      </div>
    </div>
  </div>

{% endblock %}
