
<!DOCTYPE html>
<html>
<head>
	<meta charset="UTF-8">
	<title>Competency List</title>
</head>
<body>
<table class="w3-table-all notranslate">
    <tr>
    <th style="background-color: #0000FF;color:#FFFFFF">Color Name</th>
    <th style="background-color: #FF0000">HEX</th>
    <th style="background-color: #008000">Color</th>
    </tr>

    <tr>
    <td style="color:#000000">Black</td>
    <td>#000000</td>
    <td style="background-color:#000000">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#0000FF">Blue</td>
    <td>#0000FF</td>
    <td style="background-color:#0000FF">&nbsp;</td>
    </tr>


    <tr>
    <td style="color:#008000">Green</td>
    <td>#008000</td>
    <td style="background-color:#008000">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#00FFFF">Cyan</td>
    <td>#00FFFF</td>
    <td style="background-color:#00FFFF">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#800080">Purple</td>
    <td>#800080</td>
    <td style="background-color:#800080">&nbsp;</td>
    </tr>


    <tr>
    <td style="color:#800080">Grey</td>
    <td>#808080</td>
    <td style="background-color:#808080">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#87CEEB">SkyBlue</td>
    <td>#87CEEB</td>
    <td style="background-color:#87CEEB">&nbsp;</td>
    </tr>


    <tr>
    <td style="color:#A52A2A">Brown</td>
    <td>#A52A2A</td>
    <td style="background-color:#A52A2A">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#C0C0C0;background-color: #000000">Silver</td>
    <td>#C0C0C0</td>
    <td style="background-color:#C0C0C0">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#D2691E">Chocolate</td>
    <td>#D2691E</td>
    <td style="background-color:#D2691E">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#D2B48C;background-color: #000000">Tan</td>
    <td>#D2B48C</td>
    <td style="background-color:#D2B48C">&nbsp;</td>
    </tr>


    <tr>
    <td style="color:#EE82EE;background-color: #000000">Violet</td>
    <td>#EE82EE</td>
    <td style="background-color:#EE82EE">&nbsp;</td>
    </tr>


    <tr>
    <td style="color:#FF0000;background-color: #000000">Red</td>
    <td>#FF0000</td>
    <td style="background-color:#FF0000">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#FFC0CB;background-color: #000000">Pink</td>
    <td>#FFC0CB</td>
    <td style="background-color:#FFC0CB">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#FFD700;background-color: #000000">Gold</td>
    <td>#FFD700</td>
    <td style="background-color:#FFD700">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#FFFF00;background-color: #000000">Yellow</td>
    <td>#FFFF00</td>
    <td style="background-color:#FFFF00">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#FFFFE0;background-color: #000000">LightYellow</td>
    <td>#FFFFE0</td>
    <td style="background-color:#FFFFE0">&nbsp;</td>
    </tr>

    <tr>
    <td style="color:#FFFFFF;background-color: #000000">White</td>
    <td>#FFFFFF</td>
    <td style="background-color:#FFFFFF">&nbsp;</td>
    </tr>
</table>
</body>
</html>
