/* Slider */
@import url("https://fonts.googleapis.com/css?family=Roboto:300,400,500,900");
.slick-slider {
  position: relative;
  display: block;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -ms-touch-action: pan-y;
  touch-action: pan-y;
  -webkit-tap-highlight-color: transparent; }

.slick-list {
  position: relative;
  overflow: hidden;
  display: block;
  margin: 0;
  padding: 0; }
  .slick-list:focus {
    outline: none; }
  .slick-list.dragging {
    cursor: pointer;
    cursor: hand; }

.slick-slider .slick-track,
.slick-slider .slick-list {
  -webkit-transform: translate3d(0, 0, 0);
  transform: translate3d(0, 0, 0); }

.slick-track {
  position: relative;
  left: 0;
  top: 0;
  display: block;
  margin-left: auto;
  margin-right: auto; }
  .slick-track:before, .slick-track:after {
    content: "";
    display: table; }
  .slick-track:after {
    clear: both; }
  .slick-loading .slick-track {
    visibility: hidden; }

.slick-slide {
  float: left;
  height: 100%;
  min-height: 1px;
  display: none; }
  [dir="rtl"] .slick-slide {
    float: right; }
  .slick-slide img {
    display: block; }
  .slick-slide.slick-loading img {
    display: none; }
  .slick-slide.dragging img {
    pointer-events: none; }
  .slick-initialized .slick-slide {
    display: block; }
  .slick-loading .slick-slide {
    visibility: hidden; }
  .slick-vertical .slick-slide {
    display: block;
    height: auto;
    border: 1px solid transparent; }

.slick-arrow.slick-hidden {
  display: none; }

.wrapper-bootstrap {
  /*!
 * Bootstrap v4.3.0 (https://getbootstrap.com/)
 * Copyright 2011-2019 The Bootstrap Authors
 * Copyright 2011-2019 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 */ }
  .wrapper-bootstrap :root {
    --blue: #007bff;
    --indigo: #6610f2;
    --purple: #6f42c1;
    --pink: #e83e8c;
    --red: #dc3545;
    --orange: #fd7e14;
    --yellow: #ffc107;
    --green: #28a745;
    --teal: #20c997;
    --cyan: #17a2b8;
    --white: #fff;
    --gray: #6c757d;
    --gray-dark: #343a40;
    --primary: #007bff;
    --secondary: #6c757d;
    --success: #28a745;
    --info: #17a2b8;
    --warning: #ffc107;
    --danger: #dc3545;
    --light: #f8f9fa;
    --dark: #343a40;
    --breakpoint-xs: 0;
    --breakpoint-sm: 576px;
    --breakpoint-md: 768px;
    --breakpoint-lg: 992px;
    --breakpoint-xl: 1200px;
    --font-family-sans-serif: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    --font-family-monospace: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace; }
  .wrapper-bootstrap *,
  .wrapper-bootstrap *::before,
  .wrapper-bootstrap *::after {
    -webkit-box-sizing: border-box;
    box-sizing: border-box; }
  .wrapper-bootstrap html {
    font-family: sans-serif;
    line-height: 1.15;
    -webkit-text-size-adjust: 100%;
    -webkit-tap-highlight-color: rgba(0, 0, 0, 0); }
  .wrapper-bootstrap article, .wrapper-bootstrap aside, .wrapper-bootstrap figcaption, .wrapper-bootstrap figure, .wrapper-bootstrap footer, .wrapper-bootstrap header, .wrapper-bootstrap hgroup, .wrapper-bootstrap main, .wrapper-bootstrap nav, .wrapper-bootstrap section {
    display: block; }
  .wrapper-bootstrap body {
    margin: 0;
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5;
    color: #212529;
    text-align: left;
    background-color: #fff; }
  .wrapper-bootstrap [tabindex="-1"]:focus {
    outline: 0 !important; }
  .wrapper-bootstrap hr {
    -webkit-box-sizing: content-box;
    box-sizing: content-box;
    height: 0;
    overflow: visible; }
  .wrapper-bootstrap h1, .wrapper-bootstrap h2, .wrapper-bootstrap h3, .wrapper-bootstrap h4, .wrapper-bootstrap h5, .wrapper-bootstrap h6 {
    margin-top: 0;
    margin-bottom: 0.5rem; }
  .wrapper-bootstrap p {
    margin-top: 0;
    margin-bottom: 1rem; }
  .wrapper-bootstrap abbr[title],
  .wrapper-bootstrap abbr[data-original-title] {
    text-decoration: underline;
    -webkit-text-decoration: underline dotted;
    text-decoration: underline dotted;
    cursor: help;
    border-bottom: 0;
    text-decoration-skip-ink: none; }
  .wrapper-bootstrap address {
    margin-bottom: 1rem;
    font-style: normal;
    line-height: inherit; }
  .wrapper-bootstrap ol,
  .wrapper-bootstrap ul,
  .wrapper-bootstrap dl {
    margin-top: 0;
    margin-bottom: 1rem; }
  .wrapper-bootstrap ol ol,
  .wrapper-bootstrap ul ul,
  .wrapper-bootstrap ol ul,
  .wrapper-bootstrap ul ol {
    margin-bottom: 0; }
  .wrapper-bootstrap dt {
    font-weight: 700; }
  .wrapper-bootstrap dd {
    margin-bottom: .5rem;
    margin-left: 0; }
  .wrapper-bootstrap blockquote {
    margin: 0 0 1rem; }
  .wrapper-bootstrap b,
  .wrapper-bootstrap strong {
    font-weight: bolder; }
  .wrapper-bootstrap small {
    font-size: 80%; }
  .wrapper-bootstrap sub,
  .wrapper-bootstrap sup {
    position: relative;
    font-size: 75%;
    line-height: 0;
    vertical-align: baseline; }
  .wrapper-bootstrap sub {
    bottom: -.25em; }
  .wrapper-bootstrap sup {
    top: -.5em; }
  .wrapper-bootstrap a {
    color: #007bff;
    text-decoration: none;
    background-color: transparent; }
    .wrapper-bootstrap a:hover {
      color: #0056b3;
      text-decoration: underline; }
  .wrapper-bootstrap a:not([href]):not([tabindex]) {
    color: inherit;
    text-decoration: none; }
    .wrapper-bootstrap a:not([href]):not([tabindex]):hover, .wrapper-bootstrap a:not([href]):not([tabindex]):focus {
      color: inherit;
      text-decoration: none; }
    .wrapper-bootstrap a:not([href]):not([tabindex]):focus {
      outline: 0; }
  .wrapper-bootstrap pre,
  .wrapper-bootstrap code,
  .wrapper-bootstrap kbd,
  .wrapper-bootstrap samp {
    font-family: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
    font-size: 1em; }
  .wrapper-bootstrap pre {
    margin-top: 0;
    margin-bottom: 1rem;
    overflow: auto; }
  .wrapper-bootstrap figure {
    margin: 0 0 1rem; }
  .wrapper-bootstrap img {
    vertical-align: middle;
    border-style: none; }
  .wrapper-bootstrap svg {
    overflow: hidden;
    vertical-align: middle; }
  .wrapper-bootstrap table {
    border-collapse: collapse; }
  .wrapper-bootstrap caption {
    padding-top: 0.75rem;
    padding-bottom: 0.75rem;
    color: #6c757d;
    text-align: left;
    caption-side: bottom; }
  .wrapper-bootstrap th {
    text-align: inherit; }
  .wrapper-bootstrap label {
    display: inline-block;
    margin-bottom: 0.5rem; }
  .wrapper-bootstrap button {
    border-radius: 0; }
  .wrapper-bootstrap button:focus {
    outline: 1px dotted;
    outline: 5px auto -webkit-focus-ring-color; }
  .wrapper-bootstrap input,
  .wrapper-bootstrap button,
  .wrapper-bootstrap select,
  .wrapper-bootstrap optgroup,
  .wrapper-bootstrap textarea {
    margin: 0;
    font-family: inherit;
    font-size: inherit;
    line-height: inherit; }
  .wrapper-bootstrap button,
  .wrapper-bootstrap input {
    overflow: visible; }
  .wrapper-bootstrap button,
  .wrapper-bootstrap select {
    text-transform: none; }
  .wrapper-bootstrap select {
    word-wrap: normal; }
  .wrapper-bootstrap button,
  .wrapper-bootstrap [type="button"],
  .wrapper-bootstrap [type="reset"],
  .wrapper-bootstrap [type="submit"] {
    -webkit-appearance: button; }
  .wrapper-bootstrap button:not(:disabled),
  .wrapper-bootstrap [type="button"]:not(:disabled),
  .wrapper-bootstrap [type="reset"]:not(:disabled),
  .wrapper-bootstrap [type="submit"]:not(:disabled) {
    cursor: pointer; }
  .wrapper-bootstrap button::-moz-focus-inner,
  .wrapper-bootstrap [type="button"]::-moz-focus-inner,
  .wrapper-bootstrap [type="reset"]::-moz-focus-inner,
  .wrapper-bootstrap [type="submit"]::-moz-focus-inner {
    padding: 0;
    border-style: none; }
  .wrapper-bootstrap input[type="radio"],
  .wrapper-bootstrap input[type="checkbox"] {
    -webkit-box-sizing: border-box;
    box-sizing: border-box;
    padding: 0; }
  .wrapper-bootstrap input[type="date"],
  .wrapper-bootstrap input[type="time"],
  .wrapper-bootstrap input[type="datetime-local"],
  .wrapper-bootstrap input[type="month"] {
    -webkit-appearance: listbox; }
  .wrapper-bootstrap textarea {
    overflow: auto;
    resize: vertical; }
  .wrapper-bootstrap fieldset {
    min-width: 0;
    padding: 0;
    margin: 0;
    border: 0; }
  .wrapper-bootstrap legend {
    display: block;
    width: 100%;
    max-width: 100%;
    padding: 0;
    margin-bottom: .5rem;
    font-size: 1.5rem;
    line-height: inherit;
    color: inherit;
    white-space: normal; }
  .wrapper-bootstrap progress {
    vertical-align: baseline; }
  .wrapper-bootstrap [type="number"]::-webkit-inner-spin-button,
  .wrapper-bootstrap [type="number"]::-webkit-outer-spin-button {
    height: auto; }
  .wrapper-bootstrap [type="search"] {
    outline-offset: -2px;
    -webkit-appearance: none; }
  .wrapper-bootstrap [type="search"]::-webkit-search-decoration {
    -webkit-appearance: none; }
  .wrapper-bootstrap ::-webkit-file-upload-button {
    font: inherit;
    -webkit-appearance: button; }
  .wrapper-bootstrap output {
    display: inline-block; }
  .wrapper-bootstrap summary {
    display: list-item;
    cursor: pointer; }
  .wrapper-bootstrap template {
    display: none; }
  .wrapper-bootstrap [hidden] {
    display: none !important; }
  .wrapper-bootstrap h1, .wrapper-bootstrap h2, .wrapper-bootstrap h3, .wrapper-bootstrap h4, .wrapper-bootstrap h5, .wrapper-bootstrap h6,
  .wrapper-bootstrap .h1, .wrapper-bootstrap .h2, .wrapper-bootstrap .h3, .wrapper-bootstrap .h4, .wrapper-bootstrap .h5, .wrapper-bootstrap .h6 {
    margin-bottom: 0.5rem;
    font-weight: 500;
    line-height: 1.2; }
  .wrapper-bootstrap h1, .wrapper-bootstrap .h1 {
    font-size: 2.5rem; }
  .wrapper-bootstrap h2, .wrapper-bootstrap .h2 {
    font-size: 2rem; }
  .wrapper-bootstrap h3, .wrapper-bootstrap .h3 {
    font-size: 1.75rem; }
  .wrapper-bootstrap h4, .wrapper-bootstrap .h4 {
    font-size: 1.5rem; }
  .wrapper-bootstrap h5, .wrapper-bootstrap .h5 {
    font-size: 1.25rem; }
  .wrapper-bootstrap h6, .wrapper-bootstrap .h6 {
    font-size: 1rem; }
  .wrapper-bootstrap .lead {
    font-size: 1.25rem;
    font-weight: 300; }
  .wrapper-bootstrap .display-1 {
    font-size: 6rem;
    font-weight: 300;
    line-height: 1.2; }
  .wrapper-bootstrap .display-2 {
    font-size: 5.5rem;
    font-weight: 300;
    line-height: 1.2; }
  .wrapper-bootstrap .display-3 {
    font-size: 4.5rem;
    font-weight: 300;
    line-height: 1.2; }
  .wrapper-bootstrap .display-4 {
    font-size: 3.5rem;
    font-weight: 300;
    line-height: 1.2; }
  .wrapper-bootstrap hr {
    margin-top: 1rem;
    margin-bottom: 1rem;
    border: 0;
    border-top: 1px solid rgba(0, 0, 0, 0.1); }
  .wrapper-bootstrap small,
  .wrapper-bootstrap .small {
    font-size: 80%;
    font-weight: 400; }
  .wrapper-bootstrap mark,
  .wrapper-bootstrap .mark {
    padding: 0.2em;
    background-color: #fcf8e3; }
  .wrapper-bootstrap .list-unstyled {
    padding-left: 0;
    list-style: none; }
  .wrapper-bootstrap .list-inline {
    padding-left: 0;
    list-style: none; }
  .wrapper-bootstrap .list-inline-item {
    display: inline-block; }
    .wrapper-bootstrap .list-inline-item:not(:last-child) {
      margin-right: 0.5rem; }
  .wrapper-bootstrap .initialism {
    font-size: 90%;
    text-transform: uppercase; }
  .wrapper-bootstrap .blockquote {
    margin-bottom: 1rem;
    font-size: 1.25rem; }
  .wrapper-bootstrap .blockquote-footer {
    display: block;
    font-size: 80%;
    color: #6c757d; }
    .wrapper-bootstrap .blockquote-footer::before {
      content: "\2014\00A0"; }
  .wrapper-bootstrap .img-fluid {
    max-width: 100%;
    height: auto; }
  .wrapper-bootstrap .img-thumbnail {
    padding: 0.25rem;
    background-color: #fff;
    border: 1px solid #dee2e6;
    border-radius: 0.25rem;
    max-width: 100%;
    height: auto; }
  .wrapper-bootstrap .figure {
    display: inline-block; }
  .wrapper-bootstrap .figure-img {
    margin-bottom: 0.5rem;
    line-height: 1; }
  .wrapper-bootstrap .figure-caption {
    font-size: 90%;
    color: #6c757d; }
  .wrapper-bootstrap code {
    font-size: 87.5%;
    color: #e83e8c;
    word-break: break-word; }
    a > .wrapper-bootstrap code {
      color: inherit; }
  .wrapper-bootstrap kbd {
    padding: 0.2rem 0.4rem;
    font-size: 87.5%;
    color: #fff;
    background-color: #212529;
    border-radius: 0.2rem; }
    .wrapper-bootstrap kbd kbd {
      padding: 0;
      font-size: 100%;
      font-weight: 700; }
  .wrapper-bootstrap pre {
    display: block;
    font-size: 87.5%;
    color: #212529; }
    .wrapper-bootstrap pre code {
      font-size: inherit;
      color: inherit;
      word-break: normal; }
  .wrapper-bootstrap .pre-scrollable {
    max-height: 340px;
    overflow-y: scroll; }
  .wrapper-bootstrap .container {
    width: 100%;
    padding-right: 15px;
    padding-left: 15px;
    margin-right: auto;
    margin-left: auto; }
    @media (min-width: 576px) {
      .wrapper-bootstrap .container {
        max-width: 540px; } }
    @media (min-width: 768px) {
      .wrapper-bootstrap .container {
        max-width: 720px; } }
    @media (min-width: 992px) {
      .wrapper-bootstrap .container {
        max-width: 960px; } }
    @media (min-width: 1200px) {
      .wrapper-bootstrap .container {
        max-width: 1140px; } }
  .wrapper-bootstrap .container-fluid {
    width: 100%;
    padding-right: 15px;
    padding-left: 15px;
    margin-right: auto;
    margin-left: auto; }
  .wrapper-bootstrap .row {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    margin-right: -15px;
    margin-left: -15px; }
  .wrapper-bootstrap .no-gutters {
    margin-right: 0;
    margin-left: 0; }
    .wrapper-bootstrap .no-gutters > .col,
    .wrapper-bootstrap .no-gutters > [class*="col-"] {
      padding-right: 0;
      padding-left: 0; }
  .wrapper-bootstrap .col-1, .wrapper-bootstrap .col-2, .wrapper-bootstrap .col-3, .wrapper-bootstrap .col-4, .wrapper-bootstrap .col-5, .wrapper-bootstrap .col-6, .wrapper-bootstrap .col-7, .wrapper-bootstrap .col-8, .wrapper-bootstrap .col-9, .wrapper-bootstrap .col-10, .wrapper-bootstrap .col-11, .wrapper-bootstrap .col-12, .wrapper-bootstrap .col,
  .wrapper-bootstrap .col-auto, .wrapper-bootstrap .col-sm-1, .wrapper-bootstrap .col-sm-2, .wrapper-bootstrap .col-sm-3, .wrapper-bootstrap .col-sm-4, .wrapper-bootstrap .col-sm-5, .wrapper-bootstrap .col-sm-6, .wrapper-bootstrap .col-sm-7, .wrapper-bootstrap .col-sm-8, .wrapper-bootstrap .col-sm-9, .wrapper-bootstrap .col-sm-10, .wrapper-bootstrap .col-sm-11, .wrapper-bootstrap .col-sm-12, .wrapper-bootstrap .col-sm,
  .wrapper-bootstrap .col-sm-auto, .wrapper-bootstrap .col-md-1, .wrapper-bootstrap .col-md-2, .wrapper-bootstrap .col-md-3, .wrapper-bootstrap .col-md-4, .wrapper-bootstrap .col-md-5, .wrapper-bootstrap .col-md-6, .wrapper-bootstrap .col-md-7, .wrapper-bootstrap .col-md-8, .wrapper-bootstrap .col-md-9, .wrapper-bootstrap .col-md-10, .wrapper-bootstrap .col-md-11, .wrapper-bootstrap .col-md-12, .wrapper-bootstrap .col-md,
  .wrapper-bootstrap .col-md-auto, .wrapper-bootstrap .col-lg-1, .wrapper-bootstrap .col-lg-2, .wrapper-bootstrap .col-lg-3, .wrapper-bootstrap .col-lg-4, .wrapper-bootstrap .col-lg-5, .wrapper-bootstrap .col-lg-6, .wrapper-bootstrap .col-lg-7, .wrapper-bootstrap .col-lg-8, .wrapper-bootstrap .col-lg-9, .wrapper-bootstrap .col-lg-10, .wrapper-bootstrap .col-lg-11, .wrapper-bootstrap .col-lg-12, .wrapper-bootstrap .col-lg,
  .wrapper-bootstrap .col-lg-auto, .wrapper-bootstrap .col-xl-1, .wrapper-bootstrap .col-xl-2, .wrapper-bootstrap .col-xl-3, .wrapper-bootstrap .col-xl-4, .wrapper-bootstrap .col-xl-5, .wrapper-bootstrap .col-xl-6, .wrapper-bootstrap .col-xl-7, .wrapper-bootstrap .col-xl-8, .wrapper-bootstrap .col-xl-9, .wrapper-bootstrap .col-xl-10, .wrapper-bootstrap .col-xl-11, .wrapper-bootstrap .col-xl-12, .wrapper-bootstrap .col-xl,
  .wrapper-bootstrap .col-xl-auto {
    position: relative;
    width: 100%;
    padding-right: 15px;
    padding-left: 15px; }
  .wrapper-bootstrap .col {
    -ms-flex-preferred-size: 0;
    flex-basis: 0;
    -webkit-box-flex: 1;
    -ms-flex-positive: 1;
    flex-grow: 1;
    max-width: 100%; }
  .wrapper-bootstrap .col-auto {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 auto;
    flex: 0 0 auto;
    width: auto;
    max-width: 100%; }
  .wrapper-bootstrap .col-1 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 8.33333%;
    flex: 0 0 8.33333%;
    max-width: 8.33333%; }
  .wrapper-bootstrap .col-2 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 16.66667%;
    flex: 0 0 16.66667%;
    max-width: 16.66667%; }
  .wrapper-bootstrap .col-3 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 25%;
    flex: 0 0 25%;
    max-width: 25%; }
  .wrapper-bootstrap .col-4 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 33.33333%;
    flex: 0 0 33.33333%;
    max-width: 33.33333%; }
  .wrapper-bootstrap .col-5 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 41.66667%;
    flex: 0 0 41.66667%;
    max-width: 41.66667%; }
  .wrapper-bootstrap .col-6 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 50%;
    flex: 0 0 50%;
    max-width: 50%; }
  .wrapper-bootstrap .col-7 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 58.33333%;
    flex: 0 0 58.33333%;
    max-width: 58.33333%; }
  .wrapper-bootstrap .col-8 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 66.66667%;
    flex: 0 0 66.66667%;
    max-width: 66.66667%; }
  .wrapper-bootstrap .col-9 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 75%;
    flex: 0 0 75%;
    max-width: 75%; }
  .wrapper-bootstrap .col-10 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 83.33333%;
    flex: 0 0 83.33333%;
    max-width: 83.33333%; }
  .wrapper-bootstrap .col-11 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 91.66667%;
    flex: 0 0 91.66667%;
    max-width: 91.66667%; }
  .wrapper-bootstrap .col-12 {
    -webkit-box-flex: 0;
    -ms-flex: 0 0 100%;
    flex: 0 0 100%;
    max-width: 100%; }
  .wrapper-bootstrap .order-first {
    -webkit-box-ordinal-group: 0;
    -ms-flex-order: -1;
    order: -1; }
  .wrapper-bootstrap .order-last {
    -webkit-box-ordinal-group: 14;
    -ms-flex-order: 13;
    order: 13; }
  .wrapper-bootstrap .order-0 {
    -webkit-box-ordinal-group: 1;
    -ms-flex-order: 0;
    order: 0; }
  .wrapper-bootstrap .order-1 {
    -webkit-box-ordinal-group: 2;
    -ms-flex-order: 1;
    order: 1; }
  .wrapper-bootstrap .order-2 {
    -webkit-box-ordinal-group: 3;
    -ms-flex-order: 2;
    order: 2; }
  .wrapper-bootstrap .order-3 {
    -webkit-box-ordinal-group: 4;
    -ms-flex-order: 3;
    order: 3; }
  .wrapper-bootstrap .order-4 {
    -webkit-box-ordinal-group: 5;
    -ms-flex-order: 4;
    order: 4; }
  .wrapper-bootstrap .order-5 {
    -webkit-box-ordinal-group: 6;
    -ms-flex-order: 5;
    order: 5; }
  .wrapper-bootstrap .order-6 {
    -webkit-box-ordinal-group: 7;
    -ms-flex-order: 6;
    order: 6; }
  .wrapper-bootstrap .order-7 {
    -webkit-box-ordinal-group: 8;
    -ms-flex-order: 7;
    order: 7; }
  .wrapper-bootstrap .order-8 {
    -webkit-box-ordinal-group: 9;
    -ms-flex-order: 8;
    order: 8; }
  .wrapper-bootstrap .order-9 {
    -webkit-box-ordinal-group: 10;
    -ms-flex-order: 9;
    order: 9; }
  .wrapper-bootstrap .order-10 {
    -webkit-box-ordinal-group: 11;
    -ms-flex-order: 10;
    order: 10; }
  .wrapper-bootstrap .order-11 {
    -webkit-box-ordinal-group: 12;
    -ms-flex-order: 11;
    order: 11; }
  .wrapper-bootstrap .order-12 {
    -webkit-box-ordinal-group: 13;
    -ms-flex-order: 12;
    order: 12; }
  .wrapper-bootstrap .offset-1 {
    margin-left: 8.33333%; }
  .wrapper-bootstrap .offset-2 {
    margin-left: 16.66667%; }
  .wrapper-bootstrap .offset-3 {
    margin-left: 25%; }
  .wrapper-bootstrap .offset-4 {
    margin-left: 33.33333%; }
  .wrapper-bootstrap .offset-5 {
    margin-left: 41.66667%; }
  .wrapper-bootstrap .offset-6 {
    margin-left: 50%; }
  .wrapper-bootstrap .offset-7 {
    margin-left: 58.33333%; }
  .wrapper-bootstrap .offset-8 {
    margin-left: 66.66667%; }
  .wrapper-bootstrap .offset-9 {
    margin-left: 75%; }
  .wrapper-bootstrap .offset-10 {
    margin-left: 83.33333%; }
  .wrapper-bootstrap .offset-11 {
    margin-left: 91.66667%; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .col-sm {
      -ms-flex-preferred-size: 0;
      flex-basis: 0;
      -webkit-box-flex: 1;
      -ms-flex-positive: 1;
      flex-grow: 1;
      max-width: 100%; }
    .wrapper-bootstrap .col-sm-auto {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 auto;
      flex: 0 0 auto;
      width: auto;
      max-width: 100%; }
    .wrapper-bootstrap .col-sm-1 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 8.33333%;
      flex: 0 0 8.33333%;
      max-width: 8.33333%; }
    .wrapper-bootstrap .col-sm-2 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 16.66667%;
      flex: 0 0 16.66667%;
      max-width: 16.66667%; }
    .wrapper-bootstrap .col-sm-3 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 25%;
      flex: 0 0 25%;
      max-width: 25%; }
    .wrapper-bootstrap .col-sm-4 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 33.33333%;
      flex: 0 0 33.33333%;
      max-width: 33.33333%; }
    .wrapper-bootstrap .col-sm-5 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 41.66667%;
      flex: 0 0 41.66667%;
      max-width: 41.66667%; }
    .wrapper-bootstrap .col-sm-6 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 50%;
      flex: 0 0 50%;
      max-width: 50%; }
    .wrapper-bootstrap .col-sm-7 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 58.33333%;
      flex: 0 0 58.33333%;
      max-width: 58.33333%; }
    .wrapper-bootstrap .col-sm-8 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 66.66667%;
      flex: 0 0 66.66667%;
      max-width: 66.66667%; }
    .wrapper-bootstrap .col-sm-9 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 75%;
      flex: 0 0 75%;
      max-width: 75%; }
    .wrapper-bootstrap .col-sm-10 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 83.33333%;
      flex: 0 0 83.33333%;
      max-width: 83.33333%; }
    .wrapper-bootstrap .col-sm-11 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 91.66667%;
      flex: 0 0 91.66667%;
      max-width: 91.66667%; }
    .wrapper-bootstrap .col-sm-12 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 100%;
      flex: 0 0 100%;
      max-width: 100%; }
    .wrapper-bootstrap .order-sm-first {
      -webkit-box-ordinal-group: 0;
      -ms-flex-order: -1;
      order: -1; }
    .wrapper-bootstrap .order-sm-last {
      -webkit-box-ordinal-group: 14;
      -ms-flex-order: 13;
      order: 13; }
    .wrapper-bootstrap .order-sm-0 {
      -webkit-box-ordinal-group: 1;
      -ms-flex-order: 0;
      order: 0; }
    .wrapper-bootstrap .order-sm-1 {
      -webkit-box-ordinal-group: 2;
      -ms-flex-order: 1;
      order: 1; }
    .wrapper-bootstrap .order-sm-2 {
      -webkit-box-ordinal-group: 3;
      -ms-flex-order: 2;
      order: 2; }
    .wrapper-bootstrap .order-sm-3 {
      -webkit-box-ordinal-group: 4;
      -ms-flex-order: 3;
      order: 3; }
    .wrapper-bootstrap .order-sm-4 {
      -webkit-box-ordinal-group: 5;
      -ms-flex-order: 4;
      order: 4; }
    .wrapper-bootstrap .order-sm-5 {
      -webkit-box-ordinal-group: 6;
      -ms-flex-order: 5;
      order: 5; }
    .wrapper-bootstrap .order-sm-6 {
      -webkit-box-ordinal-group: 7;
      -ms-flex-order: 6;
      order: 6; }
    .wrapper-bootstrap .order-sm-7 {
      -webkit-box-ordinal-group: 8;
      -ms-flex-order: 7;
      order: 7; }
    .wrapper-bootstrap .order-sm-8 {
      -webkit-box-ordinal-group: 9;
      -ms-flex-order: 8;
      order: 8; }
    .wrapper-bootstrap .order-sm-9 {
      -webkit-box-ordinal-group: 10;
      -ms-flex-order: 9;
      order: 9; }
    .wrapper-bootstrap .order-sm-10 {
      -webkit-box-ordinal-group: 11;
      -ms-flex-order: 10;
      order: 10; }
    .wrapper-bootstrap .order-sm-11 {
      -webkit-box-ordinal-group: 12;
      -ms-flex-order: 11;
      order: 11; }
    .wrapper-bootstrap .order-sm-12 {
      -webkit-box-ordinal-group: 13;
      -ms-flex-order: 12;
      order: 12; }
    .wrapper-bootstrap .offset-sm-0 {
      margin-left: 0; }
    .wrapper-bootstrap .offset-sm-1 {
      margin-left: 8.33333%; }
    .wrapper-bootstrap .offset-sm-2 {
      margin-left: 16.66667%; }
    .wrapper-bootstrap .offset-sm-3 {
      margin-left: 25%; }
    .wrapper-bootstrap .offset-sm-4 {
      margin-left: 33.33333%; }
    .wrapper-bootstrap .offset-sm-5 {
      margin-left: 41.66667%; }
    .wrapper-bootstrap .offset-sm-6 {
      margin-left: 50%; }
    .wrapper-bootstrap .offset-sm-7 {
      margin-left: 58.33333%; }
    .wrapper-bootstrap .offset-sm-8 {
      margin-left: 66.66667%; }
    .wrapper-bootstrap .offset-sm-9 {
      margin-left: 75%; }
    .wrapper-bootstrap .offset-sm-10 {
      margin-left: 83.33333%; }
    .wrapper-bootstrap .offset-sm-11 {
      margin-left: 91.66667%; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .col-md {
      -ms-flex-preferred-size: 0;
      flex-basis: 0;
      -webkit-box-flex: 1;
      -ms-flex-positive: 1;
      flex-grow: 1;
      max-width: 100%; }
    .wrapper-bootstrap .col-md-auto {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 auto;
      flex: 0 0 auto;
      width: auto;
      max-width: 100%; }
    .wrapper-bootstrap .col-md-1 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 8.33333%;
      flex: 0 0 8.33333%;
      max-width: 8.33333%; }
    .wrapper-bootstrap .col-md-2 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 16.66667%;
      flex: 0 0 16.66667%;
      max-width: 16.66667%; }
    .wrapper-bootstrap .col-md-3 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 25%;
      flex: 0 0 25%;
      max-width: 25%; }
    .wrapper-bootstrap .col-md-4 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 33.33333%;
      flex: 0 0 33.33333%;
      max-width: 33.33333%; }
    .wrapper-bootstrap .col-md-5 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 41.66667%;
      flex: 0 0 41.66667%;
      max-width: 41.66667%; }
    .wrapper-bootstrap .col-md-6 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 50%;
      flex: 0 0 50%;
      max-width: 50%; }
    .wrapper-bootstrap .col-md-7 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 58.33333%;
      flex: 0 0 58.33333%;
      max-width: 58.33333%; }
    .wrapper-bootstrap .col-md-8 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 66.66667%;
      flex: 0 0 66.66667%;
      max-width: 66.66667%; }
    .wrapper-bootstrap .col-md-9 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 75%;
      flex: 0 0 75%;
      max-width: 75%; }
    .wrapper-bootstrap .col-md-10 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 83.33333%;
      flex: 0 0 83.33333%;
      max-width: 83.33333%; }
    .wrapper-bootstrap .col-md-11 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 91.66667%;
      flex: 0 0 91.66667%;
      max-width: 91.66667%; }
    .wrapper-bootstrap .col-md-12 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 100%;
      flex: 0 0 100%;
      max-width: 100%; }
    .wrapper-bootstrap .order-md-first {
      -webkit-box-ordinal-group: 0;
      -ms-flex-order: -1;
      order: -1; }
    .wrapper-bootstrap .order-md-last {
      -webkit-box-ordinal-group: 14;
      -ms-flex-order: 13;
      order: 13; }
    .wrapper-bootstrap .order-md-0 {
      -webkit-box-ordinal-group: 1;
      -ms-flex-order: 0;
      order: 0; }
    .wrapper-bootstrap .order-md-1 {
      -webkit-box-ordinal-group: 2;
      -ms-flex-order: 1;
      order: 1; }
    .wrapper-bootstrap .order-md-2 {
      -webkit-box-ordinal-group: 3;
      -ms-flex-order: 2;
      order: 2; }
    .wrapper-bootstrap .order-md-3 {
      -webkit-box-ordinal-group: 4;
      -ms-flex-order: 3;
      order: 3; }
    .wrapper-bootstrap .order-md-4 {
      -webkit-box-ordinal-group: 5;
      -ms-flex-order: 4;
      order: 4; }
    .wrapper-bootstrap .order-md-5 {
      -webkit-box-ordinal-group: 6;
      -ms-flex-order: 5;
      order: 5; }
    .wrapper-bootstrap .order-md-6 {
      -webkit-box-ordinal-group: 7;
      -ms-flex-order: 6;
      order: 6; }
    .wrapper-bootstrap .order-md-7 {
      -webkit-box-ordinal-group: 8;
      -ms-flex-order: 7;
      order: 7; }
    .wrapper-bootstrap .order-md-8 {
      -webkit-box-ordinal-group: 9;
      -ms-flex-order: 8;
      order: 8; }
    .wrapper-bootstrap .order-md-9 {
      -webkit-box-ordinal-group: 10;
      -ms-flex-order: 9;
      order: 9; }
    .wrapper-bootstrap .order-md-10 {
      -webkit-box-ordinal-group: 11;
      -ms-flex-order: 10;
      order: 10; }
    .wrapper-bootstrap .order-md-11 {
      -webkit-box-ordinal-group: 12;
      -ms-flex-order: 11;
      order: 11; }
    .wrapper-bootstrap .order-md-12 {
      -webkit-box-ordinal-group: 13;
      -ms-flex-order: 12;
      order: 12; }
    .wrapper-bootstrap .offset-md-0 {
      margin-left: 0; }
    .wrapper-bootstrap .offset-md-1 {
      margin-left: 8.33333%; }
    .wrapper-bootstrap .offset-md-2 {
      margin-left: 16.66667%; }
    .wrapper-bootstrap .offset-md-3 {
      margin-left: 25%; }
    .wrapper-bootstrap .offset-md-4 {
      margin-left: 33.33333%; }
    .wrapper-bootstrap .offset-md-5 {
      margin-left: 41.66667%; }
    .wrapper-bootstrap .offset-md-6 {
      margin-left: 50%; }
    .wrapper-bootstrap .offset-md-7 {
      margin-left: 58.33333%; }
    .wrapper-bootstrap .offset-md-8 {
      margin-left: 66.66667%; }
    .wrapper-bootstrap .offset-md-9 {
      margin-left: 75%; }
    .wrapper-bootstrap .offset-md-10 {
      margin-left: 83.33333%; }
    .wrapper-bootstrap .offset-md-11 {
      margin-left: 91.66667%; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .col-lg {
      -ms-flex-preferred-size: 0;
      flex-basis: 0;
      -webkit-box-flex: 1;
      -ms-flex-positive: 1;
      flex-grow: 1;
      max-width: 100%; }
    .wrapper-bootstrap .col-lg-auto {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 auto;
      flex: 0 0 auto;
      width: auto;
      max-width: 100%; }
    .wrapper-bootstrap .col-lg-1 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 8.33333%;
      flex: 0 0 8.33333%;
      max-width: 8.33333%; }
    .wrapper-bootstrap .col-lg-2 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 16.66667%;
      flex: 0 0 16.66667%;
      max-width: 16.66667%; }
    .wrapper-bootstrap .col-lg-3 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 25%;
      flex: 0 0 25%;
      max-width: 25%; }
    .wrapper-bootstrap .col-lg-4 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 33.33333%;
      flex: 0 0 33.33333%;
      max-width: 33.33333%; }
    .wrapper-bootstrap .col-lg-5 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 41.66667%;
      flex: 0 0 41.66667%;
      max-width: 41.66667%; }
    .wrapper-bootstrap .col-lg-6 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 50%;
      flex: 0 0 50%;
      max-width: 50%; }
    .wrapper-bootstrap .col-lg-7 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 58.33333%;
      flex: 0 0 58.33333%;
      max-width: 58.33333%; }
    .wrapper-bootstrap .col-lg-8 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 66.66667%;
      flex: 0 0 66.66667%;
      max-width: 66.66667%; }
    .wrapper-bootstrap .col-lg-9 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 75%;
      flex: 0 0 75%;
      max-width: 75%; }
    .wrapper-bootstrap .col-lg-10 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 83.33333%;
      flex: 0 0 83.33333%;
      max-width: 83.33333%; }
    .wrapper-bootstrap .col-lg-11 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 91.66667%;
      flex: 0 0 91.66667%;
      max-width: 91.66667%; }
    .wrapper-bootstrap .col-lg-12 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 100%;
      flex: 0 0 100%;
      max-width: 100%; }
    .wrapper-bootstrap .order-lg-first {
      -webkit-box-ordinal-group: 0;
      -ms-flex-order: -1;
      order: -1; }
    .wrapper-bootstrap .order-lg-last {
      -webkit-box-ordinal-group: 14;
      -ms-flex-order: 13;
      order: 13; }
    .wrapper-bootstrap .order-lg-0 {
      -webkit-box-ordinal-group: 1;
      -ms-flex-order: 0;
      order: 0; }
    .wrapper-bootstrap .order-lg-1 {
      -webkit-box-ordinal-group: 2;
      -ms-flex-order: 1;
      order: 1; }
    .wrapper-bootstrap .order-lg-2 {
      -webkit-box-ordinal-group: 3;
      -ms-flex-order: 2;
      order: 2; }
    .wrapper-bootstrap .order-lg-3 {
      -webkit-box-ordinal-group: 4;
      -ms-flex-order: 3;
      order: 3; }
    .wrapper-bootstrap .order-lg-4 {
      -webkit-box-ordinal-group: 5;
      -ms-flex-order: 4;
      order: 4; }
    .wrapper-bootstrap .order-lg-5 {
      -webkit-box-ordinal-group: 6;
      -ms-flex-order: 5;
      order: 5; }
    .wrapper-bootstrap .order-lg-6 {
      -webkit-box-ordinal-group: 7;
      -ms-flex-order: 6;
      order: 6; }
    .wrapper-bootstrap .order-lg-7 {
      -webkit-box-ordinal-group: 8;
      -ms-flex-order: 7;
      order: 7; }
    .wrapper-bootstrap .order-lg-8 {
      -webkit-box-ordinal-group: 9;
      -ms-flex-order: 8;
      order: 8; }
    .wrapper-bootstrap .order-lg-9 {
      -webkit-box-ordinal-group: 10;
      -ms-flex-order: 9;
      order: 9; }
    .wrapper-bootstrap .order-lg-10 {
      -webkit-box-ordinal-group: 11;
      -ms-flex-order: 10;
      order: 10; }
    .wrapper-bootstrap .order-lg-11 {
      -webkit-box-ordinal-group: 12;
      -ms-flex-order: 11;
      order: 11; }
    .wrapper-bootstrap .order-lg-12 {
      -webkit-box-ordinal-group: 13;
      -ms-flex-order: 12;
      order: 12; }
    .wrapper-bootstrap .offset-lg-0 {
      margin-left: 0; }
    .wrapper-bootstrap .offset-lg-1 {
      margin-left: 8.33333%; }
    .wrapper-bootstrap .offset-lg-2 {
      margin-left: 16.66667%; }
    .wrapper-bootstrap .offset-lg-3 {
      margin-left: 25%; }
    .wrapper-bootstrap .offset-lg-4 {
      margin-left: 33.33333%; }
    .wrapper-bootstrap .offset-lg-5 {
      margin-left: 41.66667%; }
    .wrapper-bootstrap .offset-lg-6 {
      margin-left: 50%; }
    .wrapper-bootstrap .offset-lg-7 {
      margin-left: 58.33333%; }
    .wrapper-bootstrap .offset-lg-8 {
      margin-left: 66.66667%; }
    .wrapper-bootstrap .offset-lg-9 {
      margin-left: 75%; }
    .wrapper-bootstrap .offset-lg-10 {
      margin-left: 83.33333%; }
    .wrapper-bootstrap .offset-lg-11 {
      margin-left: 91.66667%; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .col-xl {
      -ms-flex-preferred-size: 0;
      flex-basis: 0;
      -webkit-box-flex: 1;
      -ms-flex-positive: 1;
      flex-grow: 1;
      max-width: 100%; }
    .wrapper-bootstrap .col-xl-auto {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 auto;
      flex: 0 0 auto;
      width: auto;
      max-width: 100%; }
    .wrapper-bootstrap .col-xl-1 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 8.33333%;
      flex: 0 0 8.33333%;
      max-width: 8.33333%; }
    .wrapper-bootstrap .col-xl-2 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 16.66667%;
      flex: 0 0 16.66667%;
      max-width: 16.66667%; }
    .wrapper-bootstrap .col-xl-3 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 25%;
      flex: 0 0 25%;
      max-width: 25%; }
    .wrapper-bootstrap .col-xl-4 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 33.33333%;
      flex: 0 0 33.33333%;
      max-width: 33.33333%; }
    .wrapper-bootstrap .col-xl-5 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 41.66667%;
      flex: 0 0 41.66667%;
      max-width: 41.66667%; }
    .wrapper-bootstrap .col-xl-6 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 50%;
      flex: 0 0 50%;
      max-width: 50%; }
    .wrapper-bootstrap .col-xl-7 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 58.33333%;
      flex: 0 0 58.33333%;
      max-width: 58.33333%; }
    .wrapper-bootstrap .col-xl-8 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 66.66667%;
      flex: 0 0 66.66667%;
      max-width: 66.66667%; }
    .wrapper-bootstrap .col-xl-9 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 75%;
      flex: 0 0 75%;
      max-width: 75%; }
    .wrapper-bootstrap .col-xl-10 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 83.33333%;
      flex: 0 0 83.33333%;
      max-width: 83.33333%; }
    .wrapper-bootstrap .col-xl-11 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 91.66667%;
      flex: 0 0 91.66667%;
      max-width: 91.66667%; }
    .wrapper-bootstrap .col-xl-12 {
      -webkit-box-flex: 0;
      -ms-flex: 0 0 100%;
      flex: 0 0 100%;
      max-width: 100%; }
    .wrapper-bootstrap .order-xl-first {
      -webkit-box-ordinal-group: 0;
      -ms-flex-order: -1;
      order: -1; }
    .wrapper-bootstrap .order-xl-last {
      -webkit-box-ordinal-group: 14;
      -ms-flex-order: 13;
      order: 13; }
    .wrapper-bootstrap .order-xl-0 {
      -webkit-box-ordinal-group: 1;
      -ms-flex-order: 0;
      order: 0; }
    .wrapper-bootstrap .order-xl-1 {
      -webkit-box-ordinal-group: 2;
      -ms-flex-order: 1;
      order: 1; }
    .wrapper-bootstrap .order-xl-2 {
      -webkit-box-ordinal-group: 3;
      -ms-flex-order: 2;
      order: 2; }
    .wrapper-bootstrap .order-xl-3 {
      -webkit-box-ordinal-group: 4;
      -ms-flex-order: 3;
      order: 3; }
    .wrapper-bootstrap .order-xl-4 {
      -webkit-box-ordinal-group: 5;
      -ms-flex-order: 4;
      order: 4; }
    .wrapper-bootstrap .order-xl-5 {
      -webkit-box-ordinal-group: 6;
      -ms-flex-order: 5;
      order: 5; }
    .wrapper-bootstrap .order-xl-6 {
      -webkit-box-ordinal-group: 7;
      -ms-flex-order: 6;
      order: 6; }
    .wrapper-bootstrap .order-xl-7 {
      -webkit-box-ordinal-group: 8;
      -ms-flex-order: 7;
      order: 7; }
    .wrapper-bootstrap .order-xl-8 {
      -webkit-box-ordinal-group: 9;
      -ms-flex-order: 8;
      order: 8; }
    .wrapper-bootstrap .order-xl-9 {
      -webkit-box-ordinal-group: 10;
      -ms-flex-order: 9;
      order: 9; }
    .wrapper-bootstrap .order-xl-10 {
      -webkit-box-ordinal-group: 11;
      -ms-flex-order: 10;
      order: 10; }
    .wrapper-bootstrap .order-xl-11 {
      -webkit-box-ordinal-group: 12;
      -ms-flex-order: 11;
      order: 11; }
    .wrapper-bootstrap .order-xl-12 {
      -webkit-box-ordinal-group: 13;
      -ms-flex-order: 12;
      order: 12; }
    .wrapper-bootstrap .offset-xl-0 {
      margin-left: 0; }
    .wrapper-bootstrap .offset-xl-1 {
      margin-left: 8.33333%; }
    .wrapper-bootstrap .offset-xl-2 {
      margin-left: 16.66667%; }
    .wrapper-bootstrap .offset-xl-3 {
      margin-left: 25%; }
    .wrapper-bootstrap .offset-xl-4 {
      margin-left: 33.33333%; }
    .wrapper-bootstrap .offset-xl-5 {
      margin-left: 41.66667%; }
    .wrapper-bootstrap .offset-xl-6 {
      margin-left: 50%; }
    .wrapper-bootstrap .offset-xl-7 {
      margin-left: 58.33333%; }
    .wrapper-bootstrap .offset-xl-8 {
      margin-left: 66.66667%; }
    .wrapper-bootstrap .offset-xl-9 {
      margin-left: 75%; }
    .wrapper-bootstrap .offset-xl-10 {
      margin-left: 83.33333%; }
    .wrapper-bootstrap .offset-xl-11 {
      margin-left: 91.66667%; } }
  .wrapper-bootstrap .table {
    width: 100%;
    margin-bottom: 1rem;
    color: #212529; }
    .wrapper-bootstrap .table th,
    .wrapper-bootstrap .table td {
      padding: 0.75rem;
      vertical-align: top;
      border-top: 1px solid #dee2e6; }
    .wrapper-bootstrap .table thead th {
      vertical-align: bottom;
      border-bottom: 2px solid #dee2e6; }
    .wrapper-bootstrap .table tbody + tbody {
      border-top: 2px solid #dee2e6; }
  .wrapper-bootstrap .table-sm th,
  .wrapper-bootstrap .table-sm td {
    padding: 0.3rem; }
  .wrapper-bootstrap .table-bordered {
    border: 1px solid #dee2e6; }
    .wrapper-bootstrap .table-bordered th,
    .wrapper-bootstrap .table-bordered td {
      border: 1px solid #dee2e6; }
    .wrapper-bootstrap .table-bordered thead th,
    .wrapper-bootstrap .table-bordered thead td {
      border-bottom-width: 2px; }
  .wrapper-bootstrap .table-borderless th,
  .wrapper-bootstrap .table-borderless td,
  .wrapper-bootstrap .table-borderless thead th,
  .wrapper-bootstrap .table-borderless tbody + tbody {
    border: 0; }
  .wrapper-bootstrap .table-striped tbody tr:nth-of-type(odd) {
    background-color: rgba(0, 0, 0, 0.05); }
  .wrapper-bootstrap .table-hover tbody tr:hover {
    color: #212529;
    background-color: rgba(0, 0, 0, 0.075); }
  .wrapper-bootstrap .table-primary,
  .wrapper-bootstrap .table-primary > th,
  .wrapper-bootstrap .table-primary > td {
    background-color: #b8daff; }
  .wrapper-bootstrap .table-primary th,
  .wrapper-bootstrap .table-primary td,
  .wrapper-bootstrap .table-primary thead th,
  .wrapper-bootstrap .table-primary tbody + tbody {
    border-color: #7abaff; }
  .wrapper-bootstrap .table-hover .table-primary:hover {
    background-color: #9fcdff; }
    .wrapper-bootstrap .table-hover .table-primary:hover > td,
    .wrapper-bootstrap .table-hover .table-primary:hover > th {
      background-color: #9fcdff; }
  .wrapper-bootstrap .table-secondary,
  .wrapper-bootstrap .table-secondary > th,
  .wrapper-bootstrap .table-secondary > td {
    background-color: #d6d8db; }
  .wrapper-bootstrap .table-secondary th,
  .wrapper-bootstrap .table-secondary td,
  .wrapper-bootstrap .table-secondary thead th,
  .wrapper-bootstrap .table-secondary tbody + tbody {
    border-color: #b3b7bb; }
  .wrapper-bootstrap .table-hover .table-secondary:hover {
    background-color: #c8cbcf; }
    .wrapper-bootstrap .table-hover .table-secondary:hover > td,
    .wrapper-bootstrap .table-hover .table-secondary:hover > th {
      background-color: #c8cbcf; }
  .wrapper-bootstrap .table-success,
  .wrapper-bootstrap .table-success > th,
  .wrapper-bootstrap .table-success > td {
    background-color: #c3e6cb; }
  .wrapper-bootstrap .table-success th,
  .wrapper-bootstrap .table-success td,
  .wrapper-bootstrap .table-success thead th,
  .wrapper-bootstrap .table-success tbody + tbody {
    border-color: #8fd19e; }
  .wrapper-bootstrap .table-hover .table-success:hover {
    background-color: #b1dfbb; }
    .wrapper-bootstrap .table-hover .table-success:hover > td,
    .wrapper-bootstrap .table-hover .table-success:hover > th {
      background-color: #b1dfbb; }
  .wrapper-bootstrap .table-info,
  .wrapper-bootstrap .table-info > th,
  .wrapper-bootstrap .table-info > td {
    background-color: #bee5eb; }
  .wrapper-bootstrap .table-info th,
  .wrapper-bootstrap .table-info td,
  .wrapper-bootstrap .table-info thead th,
  .wrapper-bootstrap .table-info tbody + tbody {
    border-color: #86cfda; }
  .wrapper-bootstrap .table-hover .table-info:hover {
    background-color: #abdde5; }
    .wrapper-bootstrap .table-hover .table-info:hover > td,
    .wrapper-bootstrap .table-hover .table-info:hover > th {
      background-color: #abdde5; }
  .wrapper-bootstrap .table-warning,
  .wrapper-bootstrap .table-warning > th,
  .wrapper-bootstrap .table-warning > td {
    background-color: #ffeeba; }
  .wrapper-bootstrap .table-warning th,
  .wrapper-bootstrap .table-warning td,
  .wrapper-bootstrap .table-warning thead th,
  .wrapper-bootstrap .table-warning tbody + tbody {
    border-color: #ffdf7e; }
  .wrapper-bootstrap .table-hover .table-warning:hover {
    background-color: #ffe8a1; }
    .wrapper-bootstrap .table-hover .table-warning:hover > td,
    .wrapper-bootstrap .table-hover .table-warning:hover > th {
      background-color: #ffe8a1; }
  .wrapper-bootstrap .table-danger,
  .wrapper-bootstrap .table-danger > th,
  .wrapper-bootstrap .table-danger > td {
    background-color: #f5c6cb; }
  .wrapper-bootstrap .table-danger th,
  .wrapper-bootstrap .table-danger td,
  .wrapper-bootstrap .table-danger thead th,
  .wrapper-bootstrap .table-danger tbody + tbody {
    border-color: #ed969e; }
  .wrapper-bootstrap .table-hover .table-danger:hover {
    background-color: #f1b0b7; }
    .wrapper-bootstrap .table-hover .table-danger:hover > td,
    .wrapper-bootstrap .table-hover .table-danger:hover > th {
      background-color: #f1b0b7; }
  .wrapper-bootstrap .table-light,
  .wrapper-bootstrap .table-light > th,
  .wrapper-bootstrap .table-light > td {
    background-color: #fdfdfe; }
  .wrapper-bootstrap .table-light th,
  .wrapper-bootstrap .table-light td,
  .wrapper-bootstrap .table-light thead th,
  .wrapper-bootstrap .table-light tbody + tbody {
    border-color: #fbfcfc; }
  .wrapper-bootstrap .table-hover .table-light:hover {
    background-color: #ececf6; }
    .wrapper-bootstrap .table-hover .table-light:hover > td,
    .wrapper-bootstrap .table-hover .table-light:hover > th {
      background-color: #ececf6; }
  .wrapper-bootstrap .table-dark,
  .wrapper-bootstrap .table-dark > th,
  .wrapper-bootstrap .table-dark > td {
    background-color: #c6c8ca; }
  .wrapper-bootstrap .table-dark th,
  .wrapper-bootstrap .table-dark td,
  .wrapper-bootstrap .table-dark thead th,
  .wrapper-bootstrap .table-dark tbody + tbody {
    border-color: #95999c; }
  .wrapper-bootstrap .table-hover .table-dark:hover {
    background-color: #b9bbbe; }
    .wrapper-bootstrap .table-hover .table-dark:hover > td,
    .wrapper-bootstrap .table-hover .table-dark:hover > th {
      background-color: #b9bbbe; }
  .wrapper-bootstrap .table-active,
  .wrapper-bootstrap .table-active > th,
  .wrapper-bootstrap .table-active > td {
    background-color: rgba(0, 0, 0, 0.075); }
  .wrapper-bootstrap .table-hover .table-active:hover {
    background-color: rgba(0, 0, 0, 0.075); }
    .wrapper-bootstrap .table-hover .table-active:hover > td,
    .wrapper-bootstrap .table-hover .table-active:hover > th {
      background-color: rgba(0, 0, 0, 0.075); }
  .wrapper-bootstrap .table .thead-dark th {
    color: #fff;
    background-color: #343a40;
    border-color: #454d55; }
  .wrapper-bootstrap .table .thead-light th {
    color: #495057;
    background-color: #e9ecef;
    border-color: #dee2e6; }
  .wrapper-bootstrap .table-dark {
    color: #fff;
    background-color: #343a40; }
    .wrapper-bootstrap .table-dark th,
    .wrapper-bootstrap .table-dark td,
    .wrapper-bootstrap .table-dark thead th {
      border-color: #454d55; }
    .wrapper-bootstrap .table-dark.table-bordered {
      border: 0; }
    .wrapper-bootstrap .table-dark.table-striped tbody tr:nth-of-type(odd) {
      background-color: rgba(255, 255, 255, 0.05); }
    .wrapper-bootstrap .table-dark.table-hover tbody tr:hover {
      color: #fff;
      background-color: rgba(255, 255, 255, 0.075); }
  @media (max-width: 575.98px) {
    .wrapper-bootstrap .table-responsive-sm {
      display: block;
      width: 100%;
      overflow-x: auto;
      -webkit-overflow-scrolling: touch; }
      .wrapper-bootstrap .table-responsive-sm > .table-bordered {
        border: 0; } }
  @media (max-width: 767.98px) {
    .wrapper-bootstrap .table-responsive-md {
      display: block;
      width: 100%;
      overflow-x: auto;
      -webkit-overflow-scrolling: touch; }
      .wrapper-bootstrap .table-responsive-md > .table-bordered {
        border: 0; } }
  @media (max-width: 991.98px) {
    .wrapper-bootstrap .table-responsive-lg {
      display: block;
      width: 100%;
      overflow-x: auto;
      -webkit-overflow-scrolling: touch; }
      .wrapper-bootstrap .table-responsive-lg > .table-bordered {
        border: 0; } }
  @media (max-width: 1199.98px) {
    .wrapper-bootstrap .table-responsive-xl {
      display: block;
      width: 100%;
      overflow-x: auto;
      -webkit-overflow-scrolling: touch; }
      .wrapper-bootstrap .table-responsive-xl > .table-bordered {
        border: 0; } }
  .wrapper-bootstrap .table-responsive {
    display: block;
    width: 100%;
    overflow-x: auto;
    -webkit-overflow-scrolling: touch; }
    .wrapper-bootstrap .table-responsive > .table-bordered {
      border: 0; }
  .wrapper-bootstrap .form-control {
    display: block;
    width: 100%;
    height: calc(1.5em + 0.75rem + 2px);
    padding: 0.375rem 0.75rem;
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5;
    color: #495057;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid #ced4da;
    border-radius: 0.25rem;
    -webkit-transition: border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
    transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .form-control {
        -webkit-transition: none;
        transition: none; } }
    .wrapper-bootstrap .form-control::-ms-expand {
      background-color: transparent;
      border: 0; }
    .wrapper-bootstrap .form-control:focus {
      color: #495057;
      background-color: #fff;
      border-color: #80bdff;
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
    .wrapper-bootstrap .form-control::-webkit-input-placeholder {
      color: #6c757d;
      opacity: 1; }
    .wrapper-bootstrap .form-control::-ms-input-placeholder {
      color: #6c757d;
      opacity: 1; }
    .wrapper-bootstrap .form-control::placeholder {
      color: #6c757d;
      opacity: 1; }
    .wrapper-bootstrap .form-control:disabled, .wrapper-bootstrap .form-control[readonly] {
      background-color: #e9ecef;
      opacity: 1; }
  .wrapper-bootstrap select.form-control:focus::-ms-value {
    color: #495057;
    background-color: #fff; }
  .wrapper-bootstrap .form-control-file,
  .wrapper-bootstrap .form-control-range {
    display: block;
    width: 100%; }
  .wrapper-bootstrap .col-form-label {
    padding-top: calc(0.375rem + 1px);
    padding-bottom: calc(0.375rem + 1px);
    margin-bottom: 0;
    font-size: inherit;
    line-height: 1.5; }
  .wrapper-bootstrap .col-form-label-lg {
    padding-top: calc(0.5rem + 1px);
    padding-bottom: calc(0.5rem + 1px);
    font-size: 1.25rem;
    line-height: 1.5; }
  .wrapper-bootstrap .col-form-label-sm {
    padding-top: calc(0.25rem + 1px);
    padding-bottom: calc(0.25rem + 1px);
    font-size: 0.875rem;
    line-height: 1.5; }
  .wrapper-bootstrap .form-control-plaintext {
    display: block;
    width: 100%;
    padding-top: 0.375rem;
    padding-bottom: 0.375rem;
    margin-bottom: 0;
    line-height: 1.5;
    color: #212529;
    background-color: transparent;
    border: solid transparent;
    border-width: 1px 0; }
    .wrapper-bootstrap .form-control-plaintext.form-control-sm, .wrapper-bootstrap .form-control-plaintext.form-control-lg {
      padding-right: 0;
      padding-left: 0; }
  .wrapper-bootstrap .form-control-sm {
    height: calc(1.5em + 0.5rem + 2px);
    padding: 0.25rem 0.5rem;
    font-size: 0.875rem;
    line-height: 1.5;
    border-radius: 0.2rem; }
  .wrapper-bootstrap .form-control-lg {
    height: calc(1.5em + 1rem + 2px);
    padding: 0.5rem 1rem;
    font-size: 1.25rem;
    line-height: 1.5;
    border-radius: 0.3rem; }
  .wrapper-bootstrap select.form-control[size], .wrapper-bootstrap select.form-control[multiple] {
    height: auto; }
  .wrapper-bootstrap textarea.form-control {
    height: auto; }
  .wrapper-bootstrap .form-group {
    margin-bottom: 1rem; }
  .wrapper-bootstrap .form-text {
    display: block;
    margin-top: 0.25rem; }
  .wrapper-bootstrap .form-row {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    margin-right: -5px;
    margin-left: -5px; }
    .wrapper-bootstrap .form-row > .col,
    .wrapper-bootstrap .form-row > [class*="col-"] {
      padding-right: 5px;
      padding-left: 5px; }
  .wrapper-bootstrap .form-check {
    position: relative;
    display: block;
    padding-left: 1.25rem; }
  .wrapper-bootstrap .form-check-input {
    position: absolute;
    margin-top: 0.3rem;
    margin-left: -1.25rem; }
    .wrapper-bootstrap .form-check-input:disabled ~ .form-check-label {
      color: #6c757d; }
  .wrapper-bootstrap .form-check-label {
    margin-bottom: 0; }
  .wrapper-bootstrap .form-check-inline {
    display: -webkit-inline-box;
    display: -ms-inline-flexbox;
    display: inline-flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    padding-left: 0;
    margin-right: 0.75rem; }
    .wrapper-bootstrap .form-check-inline .form-check-input {
      position: static;
      margin-top: 0;
      margin-right: 0.3125rem;
      margin-left: 0; }
  .wrapper-bootstrap .valid-feedback {
    display: none;
    width: 100%;
    margin-top: 0.25rem;
    font-size: 80%;
    color: #28a745; }
  .wrapper-bootstrap .valid-tooltip {
    position: absolute;
    top: 100%;
    z-index: 5;
    display: none;
    max-width: 100%;
    padding: 0.25rem 0.5rem;
    margin-top: .1rem;
    font-size: 0.875rem;
    line-height: 1.5;
    color: #fff;
    background-color: rgba(40, 167, 69, 0.9);
    border-radius: 0.25rem; }
  .was-validated .wrapper-bootstrap .form-control:valid, .wrapper-bootstrap .form-control.is-valid {
    border-color: #28a745;
    padding-right: calc(1.5em + 0.75rem);
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%2328a745' d='M2.3 6.73L.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e");
    background-repeat: no-repeat;
    background-position: center right calc(0.375em + 0.1875rem);
    background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem); }
    .was-validated .wrapper-bootstrap .form-control:valid:focus, .wrapper-bootstrap .form-control.is-valid:focus {
      border-color: #28a745;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25); }
    .was-validated .wrapper-bootstrap .form-control:valid ~ .valid-feedback,
    .was-validated .wrapper-bootstrap .form-control:valid ~ .valid-tooltip, .wrapper-bootstrap .form-control.is-valid ~ .valid-feedback,
    .wrapper-bootstrap .form-control.is-valid ~ .valid-tooltip {
      display: block; }
  .was-validated .wrapper-bootstrap textarea.form-control:valid, .wrapper-bootstrap textarea.form-control.is-valid {
    padding-right: calc(1.5em + 0.75rem);
    background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem); }
  .was-validated .wrapper-bootstrap .custom-select:valid, .wrapper-bootstrap .custom-select.is-valid {
    border-color: #28a745;
    padding-right: calc((1em + 0.75rem) * 3 / 4 + 1.75rem);
    background: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23343a40' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px, url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%2328a745' d='M2.3 6.73L.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e") #fff no-repeat center right 1.75rem/calc(0.75em + 0.375rem) calc(0.75em + 0.375rem); }
    .was-validated .wrapper-bootstrap .custom-select:valid:focus, .wrapper-bootstrap .custom-select.is-valid:focus {
      border-color: #28a745;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25); }
    .was-validated .wrapper-bootstrap .custom-select:valid ~ .valid-feedback,
    .was-validated .wrapper-bootstrap .custom-select:valid ~ .valid-tooltip, .wrapper-bootstrap .custom-select.is-valid ~ .valid-feedback,
    .wrapper-bootstrap .custom-select.is-valid ~ .valid-tooltip {
      display: block; }
  .was-validated .wrapper-bootstrap .form-control-file:valid ~ .valid-feedback,
  .was-validated .wrapper-bootstrap .form-control-file:valid ~ .valid-tooltip, .wrapper-bootstrap .form-control-file.is-valid ~ .valid-feedback,
  .wrapper-bootstrap .form-control-file.is-valid ~ .valid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .form-check-input:valid ~ .form-check-label, .wrapper-bootstrap .form-check-input.is-valid ~ .form-check-label {
    color: #28a745; }
  .was-validated .wrapper-bootstrap .form-check-input:valid ~ .valid-feedback,
  .was-validated .wrapper-bootstrap .form-check-input:valid ~ .valid-tooltip, .wrapper-bootstrap .form-check-input.is-valid ~ .valid-feedback,
  .wrapper-bootstrap .form-check-input.is-valid ~ .valid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-control-input:valid ~ .custom-control-label, .wrapper-bootstrap .custom-control-input.is-valid ~ .custom-control-label {
    color: #28a745; }
    .was-validated .wrapper-bootstrap .custom-control-input:valid ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-valid ~ .custom-control-label::before {
      border-color: #28a745; }
  .was-validated .wrapper-bootstrap .custom-control-input:valid ~ .valid-feedback,
  .was-validated .wrapper-bootstrap .custom-control-input:valid ~ .valid-tooltip, .wrapper-bootstrap .custom-control-input.is-valid ~ .valid-feedback,
  .wrapper-bootstrap .custom-control-input.is-valid ~ .valid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-control-input:valid:checked ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-valid:checked ~ .custom-control-label::before {
    border-color: #34ce57;
    background-color: #34ce57; }
  .was-validated .wrapper-bootstrap .custom-control-input:valid:focus ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-valid:focus ~ .custom-control-label::before {
    -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25);
    box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25); }
  .was-validated .wrapper-bootstrap .custom-control-input:valid:focus:not(:checked) ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-valid:focus:not(:checked) ~ .custom-control-label::before {
    border-color: #28a745; }
  .was-validated .wrapper-bootstrap .custom-file-input:valid ~ .custom-file-label, .wrapper-bootstrap .custom-file-input.is-valid ~ .custom-file-label {
    border-color: #28a745; }
  .was-validated .wrapper-bootstrap .custom-file-input:valid ~ .valid-feedback,
  .was-validated .wrapper-bootstrap .custom-file-input:valid ~ .valid-tooltip, .wrapper-bootstrap .custom-file-input.is-valid ~ .valid-feedback,
  .wrapper-bootstrap .custom-file-input.is-valid ~ .valid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-file-input:valid:focus ~ .custom-file-label, .wrapper-bootstrap .custom-file-input.is-valid:focus ~ .custom-file-label {
    border-color: #28a745;
    -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25);
    box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.25); }
  .wrapper-bootstrap .invalid-feedback {
    display: none;
    width: 100%;
    margin-top: 0.25rem;
    font-size: 80%;
    color: #dc3545; }
  .wrapper-bootstrap .invalid-tooltip {
    position: absolute;
    top: 100%;
    z-index: 5;
    display: none;
    max-width: 100%;
    padding: 0.25rem 0.5rem;
    margin-top: .1rem;
    font-size: 0.875rem;
    line-height: 1.5;
    color: #fff;
    background-color: rgba(220, 53, 69, 0.9);
    border-radius: 0.25rem; }
  .was-validated .wrapper-bootstrap .form-control:invalid, .wrapper-bootstrap .form-control.is-invalid {
    border-color: #dc3545;
    padding-right: calc(1.5em + 0.75rem);
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23dc3545' viewBox='-2 -2 7 7'%3e%3cpath stroke='%23dc3545' d='M0 0l3 3m0-3L0 3'/%3e%3ccircle r='.5'/%3e%3ccircle cx='3' r='.5'/%3e%3ccircle cy='3' r='.5'/%3e%3ccircle cx='3' cy='3' r='.5'/%3e%3c/svg%3E");
    background-repeat: no-repeat;
    background-position: center right calc(0.375em + 0.1875rem);
    background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem); }
    .was-validated .wrapper-bootstrap .form-control:invalid:focus, .wrapper-bootstrap .form-control.is-invalid:focus {
      border-color: #dc3545;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25); }
    .was-validated .wrapper-bootstrap .form-control:invalid ~ .invalid-feedback,
    .was-validated .wrapper-bootstrap .form-control:invalid ~ .invalid-tooltip, .wrapper-bootstrap .form-control.is-invalid ~ .invalid-feedback,
    .wrapper-bootstrap .form-control.is-invalid ~ .invalid-tooltip {
      display: block; }
  .was-validated .wrapper-bootstrap textarea.form-control:invalid, .wrapper-bootstrap textarea.form-control.is-invalid {
    padding-right: calc(1.5em + 0.75rem);
    background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem); }
  .was-validated .wrapper-bootstrap .custom-select:invalid, .wrapper-bootstrap .custom-select.is-invalid {
    border-color: #dc3545;
    padding-right: calc((1em + 0.75rem) * 3 / 4 + 1.75rem);
    background: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23343a40' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px, url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23dc3545' viewBox='-2 -2 7 7'%3e%3cpath stroke='%23dc3545' d='M0 0l3 3m0-3L0 3'/%3e%3ccircle r='.5'/%3e%3ccircle cx='3' r='.5'/%3e%3ccircle cy='3' r='.5'/%3e%3ccircle cx='3' cy='3' r='.5'/%3e%3c/svg%3E") #fff no-repeat center right 1.75rem/calc(0.75em + 0.375rem) calc(0.75em + 0.375rem); }
    .was-validated .wrapper-bootstrap .custom-select:invalid:focus, .wrapper-bootstrap .custom-select.is-invalid:focus {
      border-color: #dc3545;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25); }
    .was-validated .wrapper-bootstrap .custom-select:invalid ~ .invalid-feedback,
    .was-validated .wrapper-bootstrap .custom-select:invalid ~ .invalid-tooltip, .wrapper-bootstrap .custom-select.is-invalid ~ .invalid-feedback,
    .wrapper-bootstrap .custom-select.is-invalid ~ .invalid-tooltip {
      display: block; }
  .was-validated .wrapper-bootstrap .form-control-file:invalid ~ .invalid-feedback,
  .was-validated .wrapper-bootstrap .form-control-file:invalid ~ .invalid-tooltip, .wrapper-bootstrap .form-control-file.is-invalid ~ .invalid-feedback,
  .wrapper-bootstrap .form-control-file.is-invalid ~ .invalid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .form-check-input:invalid ~ .form-check-label, .wrapper-bootstrap .form-check-input.is-invalid ~ .form-check-label {
    color: #dc3545; }
  .was-validated .wrapper-bootstrap .form-check-input:invalid ~ .invalid-feedback,
  .was-validated .wrapper-bootstrap .form-check-input:invalid ~ .invalid-tooltip, .wrapper-bootstrap .form-check-input.is-invalid ~ .invalid-feedback,
  .wrapper-bootstrap .form-check-input.is-invalid ~ .invalid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-control-input:invalid ~ .custom-control-label, .wrapper-bootstrap .custom-control-input.is-invalid ~ .custom-control-label {
    color: #dc3545; }
    .was-validated .wrapper-bootstrap .custom-control-input:invalid ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-invalid ~ .custom-control-label::before {
      border-color: #dc3545; }
  .was-validated .wrapper-bootstrap .custom-control-input:invalid ~ .invalid-feedback,
  .was-validated .wrapper-bootstrap .custom-control-input:invalid ~ .invalid-tooltip, .wrapper-bootstrap .custom-control-input.is-invalid ~ .invalid-feedback,
  .wrapper-bootstrap .custom-control-input.is-invalid ~ .invalid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-control-input:invalid:checked ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-invalid:checked ~ .custom-control-label::before {
    border-color: #e4606d;
    background-color: #e4606d; }
  .was-validated .wrapper-bootstrap .custom-control-input:invalid:focus ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-invalid:focus ~ .custom-control-label::before {
    -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25);
    box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25); }
  .was-validated .wrapper-bootstrap .custom-control-input:invalid:focus:not(:checked) ~ .custom-control-label::before, .wrapper-bootstrap .custom-control-input.is-invalid:focus:not(:checked) ~ .custom-control-label::before {
    border-color: #dc3545; }
  .was-validated .wrapper-bootstrap .custom-file-input:invalid ~ .custom-file-label, .wrapper-bootstrap .custom-file-input.is-invalid ~ .custom-file-label {
    border-color: #dc3545; }
  .was-validated .wrapper-bootstrap .custom-file-input:invalid ~ .invalid-feedback,
  .was-validated .wrapper-bootstrap .custom-file-input:invalid ~ .invalid-tooltip, .wrapper-bootstrap .custom-file-input.is-invalid ~ .invalid-feedback,
  .wrapper-bootstrap .custom-file-input.is-invalid ~ .invalid-tooltip {
    display: block; }
  .was-validated .wrapper-bootstrap .custom-file-input:invalid:focus ~ .custom-file-label, .wrapper-bootstrap .custom-file-input.is-invalid:focus ~ .custom-file-label {
    border-color: #dc3545;
    -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25);
    box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.25); }
  .wrapper-bootstrap .form-inline {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: horizontal;
    -webkit-box-direction: normal;
    -ms-flex-flow: row wrap;
    flex-flow: row wrap;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center; }
    .wrapper-bootstrap .form-inline .form-check {
      width: 100%; }
    @media (min-width: 576px) {
      .wrapper-bootstrap .form-inline label {
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-align: center;
        -ms-flex-align: center;
        align-items: center;
        -webkit-box-pack: center;
        -ms-flex-pack: center;
        justify-content: center;
        margin-bottom: 0; }
      .wrapper-bootstrap .form-inline .form-group {
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-flex: 0;
        -ms-flex: 0 0 auto;
        flex: 0 0 auto;
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-flow: row wrap;
        flex-flow: row wrap;
        -webkit-box-align: center;
        -ms-flex-align: center;
        align-items: center;
        margin-bottom: 0; }
      .wrapper-bootstrap .form-inline .form-control {
        display: inline-block;
        width: auto;
        vertical-align: middle; }
      .wrapper-bootstrap .form-inline .form-control-plaintext {
        display: inline-block; }
      .wrapper-bootstrap .form-inline .input-group,
      .wrapper-bootstrap .form-inline .custom-select {
        width: auto; }
      .wrapper-bootstrap .form-inline .form-check {
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        -webkit-box-align: center;
        -ms-flex-align: center;
        align-items: center;
        -webkit-box-pack: center;
        -ms-flex-pack: center;
        justify-content: center;
        width: auto;
        padding-left: 0; }
      .wrapper-bootstrap .form-inline .form-check-input {
        position: relative;
        -ms-flex-negative: 0;
        flex-shrink: 0;
        margin-top: 0;
        margin-right: 0.25rem;
        margin-left: 0; }
      .wrapper-bootstrap .form-inline .custom-control {
        -webkit-box-align: center;
        -ms-flex-align: center;
        align-items: center;
        -webkit-box-pack: center;
        -ms-flex-pack: center;
        justify-content: center; }
      .wrapper-bootstrap .form-inline .custom-control-label {
        margin-bottom: 0; } }
  .wrapper-bootstrap .btn {
    display: inline-block;
    font-weight: 400;
    color: #212529;
    text-align: center;
    vertical-align: middle;
    -webkit-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    background-color: transparent;
    border: 1px solid transparent;
    padding: 0.375rem 0.75rem;
    font-size: 1rem;
    line-height: 1.5;
    border-radius: 0.25rem;
    -webkit-transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .btn {
        -webkit-transition: none;
        transition: none; } }
    .wrapper-bootstrap .btn:hover {
      color: #212529;
      text-decoration: none; }
    .wrapper-bootstrap .btn:focus, .wrapper-bootstrap .btn.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
    .wrapper-bootstrap .btn.disabled, .wrapper-bootstrap .btn:disabled {
      opacity: 0.65; }
  .wrapper-bootstrap a.btn.disabled,
  .wrapper-bootstrap fieldset:disabled a.btn {
    pointer-events: none; }
  .wrapper-bootstrap .btn-primary {
    color: #fff;
    background-color: #007bff;
    border-color: #007bff; }
    .wrapper-bootstrap .btn-primary:hover {
      color: #fff;
      background-color: #0069d9;
      border-color: #0062cc; }
    .wrapper-bootstrap .btn-primary:focus, .wrapper-bootstrap .btn-primary.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(38, 143, 255, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(38, 143, 255, 0.5); }
    .wrapper-bootstrap .btn-primary.disabled, .wrapper-bootstrap .btn-primary:disabled {
      color: #fff;
      background-color: #007bff;
      border-color: #007bff; }
    .wrapper-bootstrap .btn-primary:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-primary:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-primary.dropdown-toggle {
      color: #fff;
      background-color: #0062cc;
      border-color: #005cbf; }
      .wrapper-bootstrap .btn-primary:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-primary:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-primary.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(38, 143, 255, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(38, 143, 255, 0.5); }
  .wrapper-bootstrap .btn-secondary {
    color: #fff;
    background-color: #6c757d;
    border-color: #6c757d; }
    .wrapper-bootstrap .btn-secondary:hover {
      color: #fff;
      background-color: #5a6268;
      border-color: #545b62; }
    .wrapper-bootstrap .btn-secondary:focus, .wrapper-bootstrap .btn-secondary.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(130, 138, 145, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(130, 138, 145, 0.5); }
    .wrapper-bootstrap .btn-secondary.disabled, .wrapper-bootstrap .btn-secondary:disabled {
      color: #fff;
      background-color: #6c757d;
      border-color: #6c757d; }
    .wrapper-bootstrap .btn-secondary:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-secondary:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-secondary.dropdown-toggle {
      color: #fff;
      background-color: #545b62;
      border-color: #4e555b; }
      .wrapper-bootstrap .btn-secondary:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-secondary:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-secondary.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(130, 138, 145, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(130, 138, 145, 0.5); }
  .wrapper-bootstrap .btn-success {
    color: #fff;
    background-color: #28a745;
    border-color: #28a745; }
    .wrapper-bootstrap .btn-success:hover {
      color: #fff;
      background-color: #218838;
      border-color: #1e7e34; }
    .wrapper-bootstrap .btn-success:focus, .wrapper-bootstrap .btn-success.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(72, 180, 97, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(72, 180, 97, 0.5); }
    .wrapper-bootstrap .btn-success.disabled, .wrapper-bootstrap .btn-success:disabled {
      color: #fff;
      background-color: #28a745;
      border-color: #28a745; }
    .wrapper-bootstrap .btn-success:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-success:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-success.dropdown-toggle {
      color: #fff;
      background-color: #1e7e34;
      border-color: #1c7430; }
      .wrapper-bootstrap .btn-success:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-success:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-success.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(72, 180, 97, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(72, 180, 97, 0.5); }
  .wrapper-bootstrap .btn-info {
    color: #fff;
    background-color: #17a2b8;
    border-color: #17a2b8; }
    .wrapper-bootstrap .btn-info:hover {
      color: #fff;
      background-color: #138496;
      border-color: #117a8b; }
    .wrapper-bootstrap .btn-info:focus, .wrapper-bootstrap .btn-info.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(58, 176, 195, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(58, 176, 195, 0.5); }
    .wrapper-bootstrap .btn-info.disabled, .wrapper-bootstrap .btn-info:disabled {
      color: #fff;
      background-color: #17a2b8;
      border-color: #17a2b8; }
    .wrapper-bootstrap .btn-info:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-info:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-info.dropdown-toggle {
      color: #fff;
      background-color: #117a8b;
      border-color: #10707f; }
      .wrapper-bootstrap .btn-info:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-info:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-info.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(58, 176, 195, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(58, 176, 195, 0.5); }
  .wrapper-bootstrap .btn-warning {
    color: #212529;
    background-color: #ffc107;
    border-color: #ffc107; }
    .wrapper-bootstrap .btn-warning:hover {
      color: #212529;
      background-color: #e0a800;
      border-color: #d39e00; }
    .wrapper-bootstrap .btn-warning:focus, .wrapper-bootstrap .btn-warning.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(222, 170, 12, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(222, 170, 12, 0.5); }
    .wrapper-bootstrap .btn-warning.disabled, .wrapper-bootstrap .btn-warning:disabled {
      color: #212529;
      background-color: #ffc107;
      border-color: #ffc107; }
    .wrapper-bootstrap .btn-warning:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-warning:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-warning.dropdown-toggle {
      color: #212529;
      background-color: #d39e00;
      border-color: #c69500; }
      .wrapper-bootstrap .btn-warning:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-warning:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-warning.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(222, 170, 12, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(222, 170, 12, 0.5); }
  .wrapper-bootstrap .btn-danger {
    color: #fff;
    background-color: #dc3545;
    border-color: #dc3545; }
    .wrapper-bootstrap .btn-danger:hover {
      color: #fff;
      background-color: #c82333;
      border-color: #bd2130; }
    .wrapper-bootstrap .btn-danger:focus, .wrapper-bootstrap .btn-danger.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(225, 83, 97, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(225, 83, 97, 0.5); }
    .wrapper-bootstrap .btn-danger.disabled, .wrapper-bootstrap .btn-danger:disabled {
      color: #fff;
      background-color: #dc3545;
      border-color: #dc3545; }
    .wrapper-bootstrap .btn-danger:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-danger:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-danger.dropdown-toggle {
      color: #fff;
      background-color: #bd2130;
      border-color: #b21f2d; }
      .wrapper-bootstrap .btn-danger:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-danger:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-danger.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(225, 83, 97, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(225, 83, 97, 0.5); }
  .wrapper-bootstrap .btn-light {
    color: #212529;
    background-color: #f8f9fa;
    border-color: #f8f9fa; }
    .wrapper-bootstrap .btn-light:hover {
      color: #212529;
      background-color: #e2e6ea;
      border-color: #dae0e5; }
    .wrapper-bootstrap .btn-light:focus, .wrapper-bootstrap .btn-light.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(216, 217, 219, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(216, 217, 219, 0.5); }
    .wrapper-bootstrap .btn-light.disabled, .wrapper-bootstrap .btn-light:disabled {
      color: #212529;
      background-color: #f8f9fa;
      border-color: #f8f9fa; }
    .wrapper-bootstrap .btn-light:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-light:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-light.dropdown-toggle {
      color: #212529;
      background-color: #dae0e5;
      border-color: #d3d9df; }
      .wrapper-bootstrap .btn-light:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-light:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-light.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(216, 217, 219, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(216, 217, 219, 0.5); }
  .wrapper-bootstrap .btn-dark {
    color: #fff;
    background-color: #343a40;
    border-color: #343a40; }
    .wrapper-bootstrap .btn-dark:hover {
      color: #fff;
      background-color: #23272b;
      border-color: #1d2124; }
    .wrapper-bootstrap .btn-dark:focus, .wrapper-bootstrap .btn-dark.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(82, 88, 93, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(82, 88, 93, 0.5); }
    .wrapper-bootstrap .btn-dark.disabled, .wrapper-bootstrap .btn-dark:disabled {
      color: #fff;
      background-color: #343a40;
      border-color: #343a40; }
    .wrapper-bootstrap .btn-dark:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-dark:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-dark.dropdown-toggle {
      color: #fff;
      background-color: #1d2124;
      border-color: #171a1d; }
      .wrapper-bootstrap .btn-dark:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-dark:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-dark.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(82, 88, 93, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(82, 88, 93, 0.5); }
  .wrapper-bootstrap .btn-outline-primary {
    color: #007bff;
    border-color: #007bff; }
    .wrapper-bootstrap .btn-outline-primary:hover {
      color: #fff;
      background-color: #007bff;
      border-color: #007bff; }
    .wrapper-bootstrap .btn-outline-primary:focus, .wrapper-bootstrap .btn-outline-primary.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5); }
    .wrapper-bootstrap .btn-outline-primary.disabled, .wrapper-bootstrap .btn-outline-primary:disabled {
      color: #007bff;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-primary:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-primary:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-primary.dropdown-toggle {
      color: #fff;
      background-color: #007bff;
      border-color: #007bff; }
      .wrapper-bootstrap .btn-outline-primary:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-primary:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-primary.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .btn-outline-secondary {
    color: #6c757d;
    border-color: #6c757d; }
    .wrapper-bootstrap .btn-outline-secondary:hover {
      color: #fff;
      background-color: #6c757d;
      border-color: #6c757d; }
    .wrapper-bootstrap .btn-outline-secondary:focus, .wrapper-bootstrap .btn-outline-secondary.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5); }
    .wrapper-bootstrap .btn-outline-secondary.disabled, .wrapper-bootstrap .btn-outline-secondary:disabled {
      color: #6c757d;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-secondary:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-secondary:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-secondary.dropdown-toggle {
      color: #fff;
      background-color: #6c757d;
      border-color: #6c757d; }
      .wrapper-bootstrap .btn-outline-secondary:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-secondary:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-secondary.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5); }
  .wrapper-bootstrap .btn-outline-success {
    color: #28a745;
    border-color: #28a745; }
    .wrapper-bootstrap .btn-outline-success:hover {
      color: #fff;
      background-color: #28a745;
      border-color: #28a745; }
    .wrapper-bootstrap .btn-outline-success:focus, .wrapper-bootstrap .btn-outline-success.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5); }
    .wrapper-bootstrap .btn-outline-success.disabled, .wrapper-bootstrap .btn-outline-success:disabled {
      color: #28a745;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-success:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-success:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-success.dropdown-toggle {
      color: #fff;
      background-color: #28a745;
      border-color: #28a745; }
      .wrapper-bootstrap .btn-outline-success:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-success:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-success.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5); }
  .wrapper-bootstrap .btn-outline-info {
    color: #17a2b8;
    border-color: #17a2b8; }
    .wrapper-bootstrap .btn-outline-info:hover {
      color: #fff;
      background-color: #17a2b8;
      border-color: #17a2b8; }
    .wrapper-bootstrap .btn-outline-info:focus, .wrapper-bootstrap .btn-outline-info.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5); }
    .wrapper-bootstrap .btn-outline-info.disabled, .wrapper-bootstrap .btn-outline-info:disabled {
      color: #17a2b8;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-info:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-info:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-info.dropdown-toggle {
      color: #fff;
      background-color: #17a2b8;
      border-color: #17a2b8; }
      .wrapper-bootstrap .btn-outline-info:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-info:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-info.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5); }
  .wrapper-bootstrap .btn-outline-warning {
    color: #ffc107;
    border-color: #ffc107; }
    .wrapper-bootstrap .btn-outline-warning:hover {
      color: #212529;
      background-color: #ffc107;
      border-color: #ffc107; }
    .wrapper-bootstrap .btn-outline-warning:focus, .wrapper-bootstrap .btn-outline-warning.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5); }
    .wrapper-bootstrap .btn-outline-warning.disabled, .wrapper-bootstrap .btn-outline-warning:disabled {
      color: #ffc107;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-warning:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-warning:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-warning.dropdown-toggle {
      color: #212529;
      background-color: #ffc107;
      border-color: #ffc107; }
      .wrapper-bootstrap .btn-outline-warning:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-warning:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-warning.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5); }
  .wrapper-bootstrap .btn-outline-danger {
    color: #dc3545;
    border-color: #dc3545; }
    .wrapper-bootstrap .btn-outline-danger:hover {
      color: #fff;
      background-color: #dc3545;
      border-color: #dc3545; }
    .wrapper-bootstrap .btn-outline-danger:focus, .wrapper-bootstrap .btn-outline-danger.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5); }
    .wrapper-bootstrap .btn-outline-danger.disabled, .wrapper-bootstrap .btn-outline-danger:disabled {
      color: #dc3545;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-danger:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-danger:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-danger.dropdown-toggle {
      color: #fff;
      background-color: #dc3545;
      border-color: #dc3545; }
      .wrapper-bootstrap .btn-outline-danger:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-danger:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-danger.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5); }
  .wrapper-bootstrap .btn-outline-light {
    color: #f8f9fa;
    border-color: #f8f9fa; }
    .wrapper-bootstrap .btn-outline-light:hover {
      color: #212529;
      background-color: #f8f9fa;
      border-color: #f8f9fa; }
    .wrapper-bootstrap .btn-outline-light:focus, .wrapper-bootstrap .btn-outline-light.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5); }
    .wrapper-bootstrap .btn-outline-light.disabled, .wrapper-bootstrap .btn-outline-light:disabled {
      color: #f8f9fa;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-light:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-light:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-light.dropdown-toggle {
      color: #212529;
      background-color: #f8f9fa;
      border-color: #f8f9fa; }
      .wrapper-bootstrap .btn-outline-light:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-light:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-light.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5); }
  .wrapper-bootstrap .btn-outline-dark {
    color: #343a40;
    border-color: #343a40; }
    .wrapper-bootstrap .btn-outline-dark:hover {
      color: #fff;
      background-color: #343a40;
      border-color: #343a40; }
    .wrapper-bootstrap .btn-outline-dark:focus, .wrapper-bootstrap .btn-outline-dark.focus {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5); }
    .wrapper-bootstrap .btn-outline-dark.disabled, .wrapper-bootstrap .btn-outline-dark:disabled {
      color: #343a40;
      background-color: transparent; }
    .wrapper-bootstrap .btn-outline-dark:not(:disabled):not(.disabled):active, .wrapper-bootstrap .btn-outline-dark:not(:disabled):not(.disabled).active,
    .show > .wrapper-bootstrap .btn-outline-dark.dropdown-toggle {
      color: #fff;
      background-color: #343a40;
      border-color: #343a40; }
      .wrapper-bootstrap .btn-outline-dark:not(:disabled):not(.disabled):active:focus, .wrapper-bootstrap .btn-outline-dark:not(:disabled):not(.disabled).active:focus,
      .show > .wrapper-bootstrap .btn-outline-dark.dropdown-toggle:focus {
        -webkit-box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5);
        box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5); }
  .wrapper-bootstrap .btn-link {
    font-weight: 400;
    color: #007bff;
    text-decoration: none; }
    .wrapper-bootstrap .btn-link:hover {
      color: #0056b3;
      text-decoration: underline; }
    .wrapper-bootstrap .btn-link:focus, .wrapper-bootstrap .btn-link.focus {
      text-decoration: underline;
      -webkit-box-shadow: none;
      box-shadow: none; }
    .wrapper-bootstrap .btn-link:disabled, .wrapper-bootstrap .btn-link.disabled {
      color: #6c757d;
      pointer-events: none; }
  .wrapper-bootstrap .btn-lg, .wrapper-bootstrap .btn-group-lg > .btn {
    padding: 0.5rem 1rem;
    font-size: 1.25rem;
    line-height: 1.5;
    border-radius: 0.3rem; }
  .wrapper-bootstrap .btn-sm, .wrapper-bootstrap .btn-group-sm > .btn {
    padding: 0.25rem 0.5rem;
    font-size: 0.875rem;
    line-height: 1.5;
    border-radius: 0.2rem; }
  .wrapper-bootstrap .btn-block {
    display: block;
    width: 100%; }
    .wrapper-bootstrap .btn-block + .btn-block {
      margin-top: 0.5rem; }
  .wrapper-bootstrap input[type="submit"].btn-block,
  .wrapper-bootstrap input[type="reset"].btn-block,
  .wrapper-bootstrap input[type="button"].btn-block {
    width: 100%; }
  .wrapper-bootstrap .fade {
    -webkit-transition: opacity 0.15s linear;
    transition: opacity 0.15s linear; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .fade {
        -webkit-transition: none;
        transition: none; } }
    .wrapper-bootstrap .fade:not(.show) {
      opacity: 0; }
  .wrapper-bootstrap .collapse:not(.show) {
    display: none; }
  .wrapper-bootstrap .collapsing {
    position: relative;
    height: 0;
    overflow: hidden;
    -webkit-transition: height 0.35s ease;
    transition: height 0.35s ease; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .collapsing {
        -webkit-transition: none;
        transition: none; } }
  .wrapper-bootstrap .dropup,
  .wrapper-bootstrap .dropright,
  .wrapper-bootstrap .dropdown,
  .wrapper-bootstrap .dropleft {
    position: relative; }
  .wrapper-bootstrap .dropdown-toggle {
    white-space: nowrap; }
    .wrapper-bootstrap .dropdown-toggle::after {
      display: inline-block;
      margin-left: 0.255em;
      vertical-align: 0.255em;
      content: "";
      border-top: 0.3em solid;
      border-right: 0.3em solid transparent;
      border-bottom: 0;
      border-left: 0.3em solid transparent; }
    .wrapper-bootstrap .dropdown-toggle:empty::after {
      margin-left: 0; }
  .wrapper-bootstrap .dropdown-menu {
    position: absolute;
    top: 100%;
    left: 0;
    z-index: 1000;
    display: none;
    float: left;
    min-width: 10rem;
    padding: 0.5rem 0;
    margin: 0.125rem 0 0;
    font-size: 1rem;
    color: #212529;
    text-align: left;
    list-style: none;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid rgba(0, 0, 0, 0.15);
    border-radius: 0.25rem; }
  .wrapper-bootstrap .dropdown-menu-left {
    right: auto;
    left: 0; }
  .wrapper-bootstrap .dropdown-menu-right {
    right: 0;
    left: auto; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .dropdown-menu-sm-left {
      right: auto;
      left: 0; }
    .wrapper-bootstrap .dropdown-menu-sm-right {
      right: 0;
      left: auto; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .dropdown-menu-md-left {
      right: auto;
      left: 0; }
    .wrapper-bootstrap .dropdown-menu-md-right {
      right: 0;
      left: auto; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .dropdown-menu-lg-left {
      right: auto;
      left: 0; }
    .wrapper-bootstrap .dropdown-menu-lg-right {
      right: 0;
      left: auto; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .dropdown-menu-xl-left {
      right: auto;
      left: 0; }
    .wrapper-bootstrap .dropdown-menu-xl-right {
      right: 0;
      left: auto; } }
  .wrapper-bootstrap .dropup .dropdown-menu {
    top: auto;
    bottom: 100%;
    margin-top: 0;
    margin-bottom: 0.125rem; }
  .wrapper-bootstrap .dropup .dropdown-toggle::after {
    display: inline-block;
    margin-left: 0.255em;
    vertical-align: 0.255em;
    content: "";
    border-top: 0;
    border-right: 0.3em solid transparent;
    border-bottom: 0.3em solid;
    border-left: 0.3em solid transparent; }
  .wrapper-bootstrap .dropup .dropdown-toggle:empty::after {
    margin-left: 0; }
  .wrapper-bootstrap .dropright .dropdown-menu {
    top: 0;
    right: auto;
    left: 100%;
    margin-top: 0;
    margin-left: 0.125rem; }
  .wrapper-bootstrap .dropright .dropdown-toggle::after {
    display: inline-block;
    margin-left: 0.255em;
    vertical-align: 0.255em;
    content: "";
    border-top: 0.3em solid transparent;
    border-right: 0;
    border-bottom: 0.3em solid transparent;
    border-left: 0.3em solid; }
  .wrapper-bootstrap .dropright .dropdown-toggle:empty::after {
    margin-left: 0; }
  .wrapper-bootstrap .dropright .dropdown-toggle::after {
    vertical-align: 0; }
  .wrapper-bootstrap .dropleft .dropdown-menu {
    top: 0;
    right: 100%;
    left: auto;
    margin-top: 0;
    margin-right: 0.125rem; }
  .wrapper-bootstrap .dropleft .dropdown-toggle::after {
    display: inline-block;
    margin-left: 0.255em;
    vertical-align: 0.255em;
    content: ""; }
  .wrapper-bootstrap .dropleft .dropdown-toggle::after {
    display: none; }
  .wrapper-bootstrap .dropleft .dropdown-toggle::before {
    display: inline-block;
    margin-right: 0.255em;
    vertical-align: 0.255em;
    content: "";
    border-top: 0.3em solid transparent;
    border-right: 0.3em solid;
    border-bottom: 0.3em solid transparent; }
  .wrapper-bootstrap .dropleft .dropdown-toggle:empty::after {
    margin-left: 0; }
  .wrapper-bootstrap .dropleft .dropdown-toggle::before {
    vertical-align: 0; }
  .wrapper-bootstrap .dropdown-menu[x-placement^="top"], .wrapper-bootstrap .dropdown-menu[x-placement^="right"], .wrapper-bootstrap .dropdown-menu[x-placement^="bottom"], .wrapper-bootstrap .dropdown-menu[x-placement^="left"] {
    right: auto;
    bottom: auto; }
  .wrapper-bootstrap .dropdown-divider {
    height: 0;
    margin: 0.5rem 0;
    overflow: hidden;
    border-top: 1px solid #e9ecef; }
  .wrapper-bootstrap .dropdown-item {
    display: block;
    width: 100%;
    padding: 0.25rem 1.5rem;
    clear: both;
    font-weight: 400;
    color: #212529;
    text-align: inherit;
    white-space: nowrap;
    background-color: transparent;
    border: 0; }
    .wrapper-bootstrap .dropdown-item:hover, .wrapper-bootstrap .dropdown-item:focus {
      color: #16181b;
      text-decoration: none;
      background-color: #f8f9fa; }
    .wrapper-bootstrap .dropdown-item.active, .wrapper-bootstrap .dropdown-item:active {
      color: #fff;
      text-decoration: none;
      background-color: #007bff; }
    .wrapper-bootstrap .dropdown-item.disabled, .wrapper-bootstrap .dropdown-item:disabled {
      color: #6c757d;
      pointer-events: none;
      background-color: transparent; }
  .wrapper-bootstrap .dropdown-menu.show {
    display: block; }
  .wrapper-bootstrap .dropdown-header {
    display: block;
    padding: 0.5rem 1.5rem;
    margin-bottom: 0;
    font-size: 0.875rem;
    color: #6c757d;
    white-space: nowrap; }
  .wrapper-bootstrap .dropdown-item-text {
    display: block;
    padding: 0.25rem 1.5rem;
    color: #212529; }
  .wrapper-bootstrap .btn-group,
  .wrapper-bootstrap .btn-group-vertical {
    position: relative;
    display: -webkit-inline-box;
    display: -ms-inline-flexbox;
    display: inline-flex;
    vertical-align: middle; }
    .wrapper-bootstrap .btn-group > .btn,
    .wrapper-bootstrap .btn-group-vertical > .btn {
      position: relative;
      -webkit-box-flex: 1;
      -ms-flex: 1 1 auto;
      flex: 1 1 auto; }
      .wrapper-bootstrap .btn-group > .btn:hover,
      .wrapper-bootstrap .btn-group-vertical > .btn:hover {
        z-index: 1; }
      .wrapper-bootstrap .btn-group > .btn:focus, .wrapper-bootstrap .btn-group > .btn:active, .wrapper-bootstrap .btn-group > .btn.active,
      .wrapper-bootstrap .btn-group-vertical > .btn:focus,
      .wrapper-bootstrap .btn-group-vertical > .btn:active,
      .wrapper-bootstrap .btn-group-vertical > .btn.active {
        z-index: 1; }
  .wrapper-bootstrap .btn-toolbar {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    -webkit-box-pack: start;
    -ms-flex-pack: start;
    justify-content: flex-start; }
    .wrapper-bootstrap .btn-toolbar .input-group {
      width: auto; }
  .wrapper-bootstrap .btn-group > .btn:not(:first-child),
  .wrapper-bootstrap .btn-group > .btn-group:not(:first-child) {
    margin-left: -1px; }
  .wrapper-bootstrap .btn-group > .btn:not(:last-child):not(.dropdown-toggle),
  .wrapper-bootstrap .btn-group > .btn-group:not(:last-child) > .btn {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0; }
  .wrapper-bootstrap .btn-group > .btn:not(:first-child),
  .wrapper-bootstrap .btn-group > .btn-group:not(:first-child) > .btn {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0; }
  .wrapper-bootstrap .dropdown-toggle-split {
    padding-right: 0.5625rem;
    padding-left: 0.5625rem; }
    .wrapper-bootstrap .dropdown-toggle-split::after,
    .dropup .wrapper-bootstrap .dropdown-toggle-split::after,
    .dropright .wrapper-bootstrap .dropdown-toggle-split::after {
      margin-left: 0; }
    .dropleft .wrapper-bootstrap .dropdown-toggle-split::before {
      margin-right: 0; }
  .wrapper-bootstrap .btn-sm + .dropdown-toggle-split, .wrapper-bootstrap .btn-group-sm > .btn + .dropdown-toggle-split {
    padding-right: 0.375rem;
    padding-left: 0.375rem; }
  .wrapper-bootstrap .btn-lg + .dropdown-toggle-split, .wrapper-bootstrap .btn-group-lg > .btn + .dropdown-toggle-split {
    padding-right: 0.75rem;
    padding-left: 0.75rem; }
  .wrapper-bootstrap .btn-group-vertical {
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    -webkit-box-align: start;
    -ms-flex-align: start;
    align-items: flex-start;
    -webkit-box-pack: center;
    -ms-flex-pack: center;
    justify-content: center; }
    .wrapper-bootstrap .btn-group-vertical > .btn,
    .wrapper-bootstrap .btn-group-vertical > .btn-group {
      width: 100%; }
    .wrapper-bootstrap .btn-group-vertical > .btn:not(:first-child),
    .wrapper-bootstrap .btn-group-vertical > .btn-group:not(:first-child) {
      margin-top: -1px; }
    .wrapper-bootstrap .btn-group-vertical > .btn:not(:last-child):not(.dropdown-toggle),
    .wrapper-bootstrap .btn-group-vertical > .btn-group:not(:last-child) > .btn {
      border-bottom-right-radius: 0;
      border-bottom-left-radius: 0; }
    .wrapper-bootstrap .btn-group-vertical > .btn:not(:first-child),
    .wrapper-bootstrap .btn-group-vertical > .btn-group:not(:first-child) > .btn {
      border-top-left-radius: 0;
      border-top-right-radius: 0; }
  .wrapper-bootstrap .btn-group-toggle > .btn,
  .wrapper-bootstrap .btn-group-toggle > .btn-group > .btn {
    margin-bottom: 0; }
    .wrapper-bootstrap .btn-group-toggle > .btn input[type="radio"],
    .wrapper-bootstrap .btn-group-toggle > .btn input[type="checkbox"],
    .wrapper-bootstrap .btn-group-toggle > .btn-group > .btn input[type="radio"],
    .wrapper-bootstrap .btn-group-toggle > .btn-group > .btn input[type="checkbox"] {
      position: absolute;
      clip: rect(0, 0, 0, 0);
      pointer-events: none; }
  .wrapper-bootstrap .input-group {
    position: relative;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    -webkit-box-align: stretch;
    -ms-flex-align: stretch;
    align-items: stretch;
    width: 100%; }
    .wrapper-bootstrap .input-group > .form-control,
    .wrapper-bootstrap .input-group > .form-control-plaintext,
    .wrapper-bootstrap .input-group > .custom-select,
    .wrapper-bootstrap .input-group > .custom-file {
      position: relative;
      -webkit-box-flex: 1;
      -ms-flex: 1 1 auto;
      flex: 1 1 auto;
      width: 1%;
      margin-bottom: 0; }
      .wrapper-bootstrap .input-group > .form-control + .form-control,
      .wrapper-bootstrap .input-group > .form-control + .custom-select,
      .wrapper-bootstrap .input-group > .form-control + .custom-file,
      .wrapper-bootstrap .input-group > .form-control-plaintext + .form-control,
      .wrapper-bootstrap .input-group > .form-control-plaintext + .custom-select,
      .wrapper-bootstrap .input-group > .form-control-plaintext + .custom-file,
      .wrapper-bootstrap .input-group > .custom-select + .form-control,
      .wrapper-bootstrap .input-group > .custom-select + .custom-select,
      .wrapper-bootstrap .input-group > .custom-select + .custom-file,
      .wrapper-bootstrap .input-group > .custom-file + .form-control,
      .wrapper-bootstrap .input-group > .custom-file + .custom-select,
      .wrapper-bootstrap .input-group > .custom-file + .custom-file {
        margin-left: -1px; }
    .wrapper-bootstrap .input-group > .form-control:focus,
    .wrapper-bootstrap .input-group > .custom-select:focus,
    .wrapper-bootstrap .input-group > .custom-file .custom-file-input:focus ~ .custom-file-label {
      z-index: 3; }
    .wrapper-bootstrap .input-group > .custom-file .custom-file-input:focus {
      z-index: 4; }
    .wrapper-bootstrap .input-group > .form-control:not(:last-child),
    .wrapper-bootstrap .input-group > .custom-select:not(:last-child) {
      border-top-right-radius: 0;
      border-bottom-right-radius: 0; }
    .wrapper-bootstrap .input-group > .form-control:not(:first-child),
    .wrapper-bootstrap .input-group > .custom-select:not(:first-child) {
      border-top-left-radius: 0;
      border-bottom-left-radius: 0; }
    .wrapper-bootstrap .input-group > .custom-file {
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -webkit-box-align: center;
      -ms-flex-align: center;
      align-items: center; }
      .wrapper-bootstrap .input-group > .custom-file:not(:last-child) .custom-file-label,
      .wrapper-bootstrap .input-group > .custom-file:not(:last-child) .custom-file-label::after {
        border-top-right-radius: 0;
        border-bottom-right-radius: 0; }
      .wrapper-bootstrap .input-group > .custom-file:not(:first-child) .custom-file-label {
        border-top-left-radius: 0;
        border-bottom-left-radius: 0; }
  .wrapper-bootstrap .input-group-prepend,
  .wrapper-bootstrap .input-group-append {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex; }
    .wrapper-bootstrap .input-group-prepend .btn,
    .wrapper-bootstrap .input-group-append .btn {
      position: relative;
      z-index: 2; }
      .wrapper-bootstrap .input-group-prepend .btn:focus,
      .wrapper-bootstrap .input-group-append .btn:focus {
        z-index: 3; }
    .wrapper-bootstrap .input-group-prepend .btn + .btn,
    .wrapper-bootstrap .input-group-prepend .btn + .input-group-text,
    .wrapper-bootstrap .input-group-prepend .input-group-text + .input-group-text,
    .wrapper-bootstrap .input-group-prepend .input-group-text + .btn,
    .wrapper-bootstrap .input-group-append .btn + .btn,
    .wrapper-bootstrap .input-group-append .btn + .input-group-text,
    .wrapper-bootstrap .input-group-append .input-group-text + .input-group-text,
    .wrapper-bootstrap .input-group-append .input-group-text + .btn {
      margin-left: -1px; }
  .wrapper-bootstrap .input-group-prepend {
    margin-right: -1px; }
  .wrapper-bootstrap .input-group-append {
    margin-left: -1px; }
  .wrapper-bootstrap .input-group-text {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    padding: 0.375rem 0.75rem;
    margin-bottom: 0;
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5;
    color: #495057;
    text-align: center;
    white-space: nowrap;
    background-color: #e9ecef;
    border: 1px solid #ced4da;
    border-radius: 0.25rem; }
    .wrapper-bootstrap .input-group-text input[type="radio"],
    .wrapper-bootstrap .input-group-text input[type="checkbox"] {
      margin-top: 0; }
  .wrapper-bootstrap .input-group-lg > .form-control:not(textarea),
  .wrapper-bootstrap .input-group-lg > .custom-select {
    height: calc(1.5em + 1rem + 2px); }
  .wrapper-bootstrap .input-group-lg > .form-control,
  .wrapper-bootstrap .input-group-lg > .custom-select,
  .wrapper-bootstrap .input-group-lg > .input-group-prepend > .input-group-text,
  .wrapper-bootstrap .input-group-lg > .input-group-append > .input-group-text,
  .wrapper-bootstrap .input-group-lg > .input-group-prepend > .btn,
  .wrapper-bootstrap .input-group-lg > .input-group-append > .btn {
    padding: 0.5rem 1rem;
    font-size: 1.25rem;
    line-height: 1.5;
    border-radius: 0.3rem; }
  .wrapper-bootstrap .input-group-sm > .form-control:not(textarea),
  .wrapper-bootstrap .input-group-sm > .custom-select {
    height: calc(1.5em + 0.5rem + 2px); }
  .wrapper-bootstrap .input-group-sm > .form-control,
  .wrapper-bootstrap .input-group-sm > .custom-select,
  .wrapper-bootstrap .input-group-sm > .input-group-prepend > .input-group-text,
  .wrapper-bootstrap .input-group-sm > .input-group-append > .input-group-text,
  .wrapper-bootstrap .input-group-sm > .input-group-prepend > .btn,
  .wrapper-bootstrap .input-group-sm > .input-group-append > .btn {
    padding: 0.25rem 0.5rem;
    font-size: 0.875rem;
    line-height: 1.5;
    border-radius: 0.2rem; }
  .wrapper-bootstrap .input-group-lg > .custom-select,
  .wrapper-bootstrap .input-group-sm > .custom-select {
    padding-right: 1.75rem; }
  .wrapper-bootstrap .input-group > .input-group-prepend > .btn,
  .wrapper-bootstrap .input-group > .input-group-prepend > .input-group-text,
  .wrapper-bootstrap .input-group > .input-group-append:not(:last-child) > .btn,
  .wrapper-bootstrap .input-group > .input-group-append:not(:last-child) > .input-group-text,
  .wrapper-bootstrap .input-group > .input-group-append:last-child > .btn:not(:last-child):not(.dropdown-toggle),
  .wrapper-bootstrap .input-group > .input-group-append:last-child > .input-group-text:not(:last-child) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0; }
  .wrapper-bootstrap .input-group > .input-group-append > .btn,
  .wrapper-bootstrap .input-group > .input-group-append > .input-group-text,
  .wrapper-bootstrap .input-group > .input-group-prepend:not(:first-child) > .btn,
  .wrapper-bootstrap .input-group > .input-group-prepend:not(:first-child) > .input-group-text,
  .wrapper-bootstrap .input-group > .input-group-prepend:first-child > .btn:not(:first-child),
  .wrapper-bootstrap .input-group > .input-group-prepend:first-child > .input-group-text:not(:first-child) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0; }
  .wrapper-bootstrap .custom-control {
    position: relative;
    display: block;
    min-height: 1.5rem;
    padding-left: 1.5rem; }
  .wrapper-bootstrap .custom-control-inline {
    display: -webkit-inline-box;
    display: -ms-inline-flexbox;
    display: inline-flex;
    margin-right: 1rem; }
  .wrapper-bootstrap .custom-control-input {
    position: absolute;
    z-index: -1;
    opacity: 0; }
    .wrapper-bootstrap .custom-control-input:checked ~ .custom-control-label::before {
      color: #fff;
      border-color: #007bff;
      background-color: #007bff; }
    .wrapper-bootstrap .custom-control-input:focus ~ .custom-control-label::before {
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
    .wrapper-bootstrap .custom-control-input:focus:not(:checked) ~ .custom-control-label::before {
      border-color: #80bdff; }
    .wrapper-bootstrap .custom-control-input:not(:disabled):active ~ .custom-control-label::before {
      color: #fff;
      background-color: #b3d7ff;
      border-color: #b3d7ff; }
    .wrapper-bootstrap .custom-control-input:disabled ~ .custom-control-label {
      color: #6c757d; }
      .wrapper-bootstrap .custom-control-input:disabled ~ .custom-control-label::before {
        background-color: #e9ecef; }
  .wrapper-bootstrap .custom-control-label {
    position: relative;
    margin-bottom: 0;
    vertical-align: top; }
    .wrapper-bootstrap .custom-control-label::before {
      position: absolute;
      top: 0.25rem;
      left: -1.5rem;
      display: block;
      width: 1rem;
      height: 1rem;
      pointer-events: none;
      content: "";
      background-color: #fff;
      border: #adb5bd solid 1px; }
    .wrapper-bootstrap .custom-control-label::after {
      position: absolute;
      top: 0.25rem;
      left: -1.5rem;
      display: block;
      width: 1rem;
      height: 1rem;
      content: "";
      background: no-repeat 50% / 50% 50%; }
  .wrapper-bootstrap .custom-checkbox .custom-control-label::before {
    border-radius: 0.25rem; }
  .wrapper-bootstrap .custom-checkbox .custom-control-input:checked ~ .custom-control-label::after {
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23fff' d='M6.564.75l-3.59 3.612-1.538-1.55L0 4.26 2.974 7.25 8 2.193z'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::before {
    border-color: #007bff;
    background-color: #007bff; }
  .wrapper-bootstrap .custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::after {
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 4'%3e%3cpath stroke='%23fff' d='M0 2h4'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .custom-checkbox .custom-control-input:disabled:checked ~ .custom-control-label::before {
    background-color: rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .custom-checkbox .custom-control-input:disabled:indeterminate ~ .custom-control-label::before {
    background-color: rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .custom-radio .custom-control-label::before {
    border-radius: 50%; }
  .wrapper-bootstrap .custom-radio .custom-control-input:checked ~ .custom-control-label::after {
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%23fff'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .custom-radio .custom-control-input:disabled:checked ~ .custom-control-label::before {
    background-color: rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .custom-switch {
    padding-left: 2.25rem; }
    .wrapper-bootstrap .custom-switch .custom-control-label::before {
      left: -2.25rem;
      width: 1.75rem;
      pointer-events: all;
      border-radius: 0.5rem; }
    .wrapper-bootstrap .custom-switch .custom-control-label::after {
      top: calc(0.25rem + 2px);
      left: calc(-2.25rem + 2px);
      width: calc(1rem - 4px);
      height: calc(1rem - 4px);
      background-color: #adb5bd;
      border-radius: 0.5rem;
      -webkit-transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-transform 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-transform 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
      transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-transform 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out; }
      @media (prefers-reduced-motion: reduce) {
        .wrapper-bootstrap .custom-switch .custom-control-label::after {
          -webkit-transition: none;
          transition: none; } }
    .wrapper-bootstrap .custom-switch .custom-control-input:checked ~ .custom-control-label::after {
      background-color: #fff;
      -webkit-transform: translateX(0.75rem);
      transform: translateX(0.75rem); }
    .wrapper-bootstrap .custom-switch .custom-control-input:disabled:checked ~ .custom-control-label::before {
      background-color: rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .custom-select {
    display: inline-block;
    width: 100%;
    height: calc(1.5em + 0.75rem + 2px);
    padding: 0.375rem 1.75rem 0.375rem 0.75rem;
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5;
    color: #495057;
    vertical-align: middle;
    background: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3e%3cpath fill='%23343a40' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") no-repeat right 0.75rem center/8px 10px;
    background-color: #fff;
    border: 1px solid #ced4da;
    border-radius: 0.25rem;
    -webkit-appearance: none;
    -moz-appearance: none;
    appearance: none; }
    .wrapper-bootstrap .custom-select:focus {
      border-color: #80bdff;
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
      .wrapper-bootstrap .custom-select:focus::-ms-value {
        color: #495057;
        background-color: #fff; }
    .wrapper-bootstrap .custom-select[multiple], .wrapper-bootstrap .custom-select[size]:not([size="1"]) {
      height: auto;
      padding-right: 0.75rem;
      background-image: none; }
    .wrapper-bootstrap .custom-select:disabled {
      color: #6c757d;
      background-color: #e9ecef; }
    .wrapper-bootstrap .custom-select::-ms-expand {
      display: none; }
  .wrapper-bootstrap .custom-select-sm {
    height: calc(1.5em + 0.5rem + 2px);
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
    padding-left: 0.5rem;
    font-size: 0.875rem; }
  .wrapper-bootstrap .custom-select-lg {
    height: calc(1.5em + 1rem + 2px);
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    padding-left: 1rem;
    font-size: 1.25rem; }
  .wrapper-bootstrap .custom-file {
    position: relative;
    display: inline-block;
    width: 100%;
    height: calc(1.5em + 0.75rem + 2px);
    margin-bottom: 0; }
  .wrapper-bootstrap .custom-file-input {
    position: relative;
    z-index: 2;
    width: 100%;
    height: calc(1.5em + 0.75rem + 2px);
    margin: 0;
    opacity: 0; }
    .wrapper-bootstrap .custom-file-input:focus ~ .custom-file-label {
      border-color: #80bdff;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
    .wrapper-bootstrap .custom-file-input:disabled ~ .custom-file-label {
      background-color: #e9ecef; }
    .wrapper-bootstrap .custom-file-input:lang(en) ~ .custom-file-label::after {
      content: "Browse"; }
    .wrapper-bootstrap .custom-file-input ~ .custom-file-label[data-browse]::after {
      content: attr(data-browse); }
  .wrapper-bootstrap .custom-file-label {
    position: absolute;
    top: 0;
    right: 0;
    left: 0;
    z-index: 1;
    height: calc(1.5em + 0.75rem + 2px);
    padding: 0.375rem 0.75rem;
    font-weight: 400;
    line-height: 1.5;
    color: #495057;
    background-color: #fff;
    border: 1px solid #ced4da;
    border-radius: 0.25rem; }
    .wrapper-bootstrap .custom-file-label::after {
      position: absolute;
      top: 0;
      right: 0;
      bottom: 0;
      z-index: 3;
      display: block;
      height: calc(1.5em + 0.75rem);
      padding: 0.375rem 0.75rem;
      line-height: 1.5;
      color: #495057;
      content: "Browse";
      background-color: #e9ecef;
      border-left: inherit;
      border-radius: 0 0.25rem 0.25rem 0; }
  .wrapper-bootstrap .custom-range {
    width: 100%;
    height: calc(1rem + 0.4rem);
    padding: 0;
    background-color: transparent;
    -webkit-appearance: none;
    -moz-appearance: none;
    appearance: none; }
    .wrapper-bootstrap .custom-range:focus {
      outline: none; }
      .wrapper-bootstrap .custom-range:focus::-webkit-slider-thumb {
        -webkit-box-shadow: 0 0 0 1px #fff, 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
        box-shadow: 0 0 0 1px #fff, 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
      .wrapper-bootstrap .custom-range:focus::-moz-range-thumb {
        box-shadow: 0 0 0 1px #fff, 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
      .wrapper-bootstrap .custom-range:focus::-ms-thumb {
        box-shadow: 0 0 0 1px #fff, 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
    .wrapper-bootstrap .custom-range::-moz-focus-outer {
      border: 0; }
    .wrapper-bootstrap .custom-range::-webkit-slider-thumb {
      width: 1rem;
      height: 1rem;
      margin-top: -0.25rem;
      background-color: #007bff;
      border: 0;
      border-radius: 1rem;
      -webkit-transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      -webkit-appearance: none;
      appearance: none; }
      @media (prefers-reduced-motion: reduce) {
        .wrapper-bootstrap .custom-range::-webkit-slider-thumb {
          -webkit-transition: none;
          transition: none; } }
      .wrapper-bootstrap .custom-range::-webkit-slider-thumb:active {
        background-color: #b3d7ff; }
    .wrapper-bootstrap .custom-range::-webkit-slider-runnable-track {
      width: 100%;
      height: 0.5rem;
      color: transparent;
      cursor: pointer;
      background-color: #dee2e6;
      border-color: transparent;
      border-radius: 1rem; }
    .wrapper-bootstrap .custom-range::-moz-range-thumb {
      width: 1rem;
      height: 1rem;
      background-color: #007bff;
      border: 0;
      border-radius: 1rem;
      -webkit-transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      -moz-appearance: none;
      appearance: none; }
      @media (prefers-reduced-motion: reduce) {
        .wrapper-bootstrap .custom-range::-moz-range-thumb {
          -webkit-transition: none;
          transition: none; } }
      .wrapper-bootstrap .custom-range::-moz-range-thumb:active {
        background-color: #b3d7ff; }
    .wrapper-bootstrap .custom-range::-moz-range-track {
      width: 100%;
      height: 0.5rem;
      color: transparent;
      cursor: pointer;
      background-color: #dee2e6;
      border-color: transparent;
      border-radius: 1rem; }
    .wrapper-bootstrap .custom-range::-ms-thumb {
      width: 1rem;
      height: 1rem;
      margin-top: 0;
      margin-right: 0.2rem;
      margin-left: 0.2rem;
      background-color: #007bff;
      border: 0;
      border-radius: 1rem;
      -webkit-transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
      transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
      appearance: none; }
      @media (prefers-reduced-motion: reduce) {
        .wrapper-bootstrap .custom-range::-ms-thumb {
          -webkit-transition: none;
          transition: none; } }
      .wrapper-bootstrap .custom-range::-ms-thumb:active {
        background-color: #b3d7ff; }
    .wrapper-bootstrap .custom-range::-ms-track {
      width: 100%;
      height: 0.5rem;
      color: transparent;
      cursor: pointer;
      background-color: transparent;
      border-color: transparent;
      border-width: 0.5rem; }
    .wrapper-bootstrap .custom-range::-ms-fill-lower {
      background-color: #dee2e6;
      border-radius: 1rem; }
    .wrapper-bootstrap .custom-range::-ms-fill-upper {
      margin-right: 15px;
      background-color: #dee2e6;
      border-radius: 1rem; }
    .wrapper-bootstrap .custom-range:disabled::-webkit-slider-thumb {
      background-color: #adb5bd; }
    .wrapper-bootstrap .custom-range:disabled::-webkit-slider-runnable-track {
      cursor: default; }
    .wrapper-bootstrap .custom-range:disabled::-moz-range-thumb {
      background-color: #adb5bd; }
    .wrapper-bootstrap .custom-range:disabled::-moz-range-track {
      cursor: default; }
    .wrapper-bootstrap .custom-range:disabled::-ms-thumb {
      background-color: #adb5bd; }
  .wrapper-bootstrap .custom-control-label::before,
  .wrapper-bootstrap .custom-file-label,
  .wrapper-bootstrap .custom-select {
    -webkit-transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
    transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .custom-control-label::before,
      .wrapper-bootstrap .custom-file-label,
      .wrapper-bootstrap .custom-select {
        -webkit-transition: none;
        transition: none; } }
  .wrapper-bootstrap .nav {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    padding-left: 0;
    margin-bottom: 0;
    list-style: none; }
  .wrapper-bootstrap .nav-link {
    display: block;
    padding: 0.5rem 1rem; }
    .wrapper-bootstrap .nav-link:hover, .wrapper-bootstrap .nav-link:focus {
      text-decoration: none; }
    .wrapper-bootstrap .nav-link.disabled {
      color: #6c757d;
      pointer-events: none;
      cursor: default; }
  .wrapper-bootstrap .nav-tabs {
    border-bottom: 1px solid #dee2e6; }
    .wrapper-bootstrap .nav-tabs .nav-item {
      margin-bottom: -1px; }
    .wrapper-bootstrap .nav-tabs .nav-link {
      border: 1px solid transparent;
      border-top-left-radius: 0.25rem;
      border-top-right-radius: 0.25rem; }
      .wrapper-bootstrap .nav-tabs .nav-link:hover, .wrapper-bootstrap .nav-tabs .nav-link:focus {
        border-color: #e9ecef #e9ecef #dee2e6; }
      .wrapper-bootstrap .nav-tabs .nav-link.disabled {
        color: #6c757d;
        background-color: transparent;
        border-color: transparent; }
    .wrapper-bootstrap .nav-tabs .nav-link.active,
    .wrapper-bootstrap .nav-tabs .nav-item.show .nav-link {
      color: #495057;
      background-color: #fff;
      border-color: #dee2e6 #dee2e6 #fff; }
    .wrapper-bootstrap .nav-tabs .dropdown-menu {
      margin-top: -1px;
      border-top-left-radius: 0;
      border-top-right-radius: 0; }
  .wrapper-bootstrap .nav-pills .nav-link {
    border-radius: 0.25rem; }
  .wrapper-bootstrap .nav-pills .nav-link.active,
  .wrapper-bootstrap .nav-pills .show > .nav-link {
    color: #fff;
    background-color: #007bff; }
  .wrapper-bootstrap .nav-fill .nav-item {
    -webkit-box-flex: 1;
    -ms-flex: 1 1 auto;
    flex: 1 1 auto;
    text-align: center; }
  .wrapper-bootstrap .nav-justified .nav-item {
    -ms-flex-preferred-size: 0;
    flex-basis: 0;
    -webkit-box-flex: 1;
    -ms-flex-positive: 1;
    flex-grow: 1;
    text-align: center; }
  .wrapper-bootstrap .tab-content > .tab-pane {
    display: none; }
  .wrapper-bootstrap .tab-content > .active {
    display: block; }
  .wrapper-bootstrap .navbar {
    position: relative;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    -webkit-box-pack: justify;
    -ms-flex-pack: justify;
    justify-content: space-between;
    padding: 0.5rem 1rem; }
    .wrapper-bootstrap .navbar > .container,
    .wrapper-bootstrap .navbar > .container-fluid {
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      -ms-flex-wrap: wrap;
      flex-wrap: wrap;
      -webkit-box-align: center;
      -ms-flex-align: center;
      align-items: center;
      -webkit-box-pack: justify;
      -ms-flex-pack: justify;
      justify-content: space-between; }
  .wrapper-bootstrap .navbar-brand {
    display: inline-block;
    padding-top: 0.3125rem;
    padding-bottom: 0.3125rem;
    margin-right: 1rem;
    font-size: 1.25rem;
    line-height: inherit;
    white-space: nowrap; }
    .wrapper-bootstrap .navbar-brand:hover, .wrapper-bootstrap .navbar-brand:focus {
      text-decoration: none; }
  .wrapper-bootstrap .navbar-nav {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    padding-left: 0;
    margin-bottom: 0;
    list-style: none; }
    .wrapper-bootstrap .navbar-nav .nav-link {
      padding-right: 0;
      padding-left: 0; }
    .wrapper-bootstrap .navbar-nav .dropdown-menu {
      position: static;
      float: none; }
  .wrapper-bootstrap .navbar-text {
    display: inline-block;
    padding-top: 0.5rem;
    padding-bottom: 0.5rem; }
  .wrapper-bootstrap .navbar-collapse {
    -ms-flex-preferred-size: 100%;
    flex-basis: 100%;
    -webkit-box-flex: 1;
    -ms-flex-positive: 1;
    flex-grow: 1;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center; }
  .wrapper-bootstrap .navbar-toggler {
    padding: 0.25rem 0.75rem;
    font-size: 1.25rem;
    line-height: 1;
    background-color: transparent;
    border: 1px solid transparent;
    border-radius: 0.25rem; }
    .wrapper-bootstrap .navbar-toggler:hover, .wrapper-bootstrap .navbar-toggler:focus {
      text-decoration: none; }
  .wrapper-bootstrap .navbar-toggler-icon {
    display: inline-block;
    width: 1.5em;
    height: 1.5em;
    vertical-align: middle;
    content: "";
    background: no-repeat center center;
    background-size: 100% 100%; }
  @media (max-width: 575.98px) {
    .wrapper-bootstrap .navbar-expand-sm > .container,
    .wrapper-bootstrap .navbar-expand-sm > .container-fluid {
      padding-right: 0;
      padding-left: 0; } }
  @media (min-width: 576px) {
    .wrapper-bootstrap .navbar-expand-sm {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-flow: row nowrap;
      flex-flow: row nowrap;
      -webkit-box-pack: start;
      -ms-flex-pack: start;
      justify-content: flex-start; }
      .wrapper-bootstrap .navbar-expand-sm .navbar-nav {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-direction: row;
        flex-direction: row; }
        .wrapper-bootstrap .navbar-expand-sm .navbar-nav .dropdown-menu {
          position: absolute; }
        .wrapper-bootstrap .navbar-expand-sm .navbar-nav .nav-link {
          padding-right: 0.5rem;
          padding-left: 0.5rem; }
      .wrapper-bootstrap .navbar-expand-sm > .container,
      .wrapper-bootstrap .navbar-expand-sm > .container-fluid {
        -ms-flex-wrap: nowrap;
        flex-wrap: nowrap; }
      .wrapper-bootstrap .navbar-expand-sm .navbar-collapse {
        display: -webkit-box !important;
        display: -ms-flexbox !important;
        display: flex !important;
        -ms-flex-preferred-size: auto;
        flex-basis: auto; }
      .wrapper-bootstrap .navbar-expand-sm .navbar-toggler {
        display: none; } }
  @media (max-width: 767.98px) {
    .wrapper-bootstrap .navbar-expand-md > .container,
    .wrapper-bootstrap .navbar-expand-md > .container-fluid {
      padding-right: 0;
      padding-left: 0; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .navbar-expand-md {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-flow: row nowrap;
      flex-flow: row nowrap;
      -webkit-box-pack: start;
      -ms-flex-pack: start;
      justify-content: flex-start; }
      .wrapper-bootstrap .navbar-expand-md .navbar-nav {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-direction: row;
        flex-direction: row; }
        .wrapper-bootstrap .navbar-expand-md .navbar-nav .dropdown-menu {
          position: absolute; }
        .wrapper-bootstrap .navbar-expand-md .navbar-nav .nav-link {
          padding-right: 0.5rem;
          padding-left: 0.5rem; }
      .wrapper-bootstrap .navbar-expand-md > .container,
      .wrapper-bootstrap .navbar-expand-md > .container-fluid {
        -ms-flex-wrap: nowrap;
        flex-wrap: nowrap; }
      .wrapper-bootstrap .navbar-expand-md .navbar-collapse {
        display: -webkit-box !important;
        display: -ms-flexbox !important;
        display: flex !important;
        -ms-flex-preferred-size: auto;
        flex-basis: auto; }
      .wrapper-bootstrap .navbar-expand-md .navbar-toggler {
        display: none; } }
  @media (max-width: 991.98px) {
    .wrapper-bootstrap .navbar-expand-lg > .container,
    .wrapper-bootstrap .navbar-expand-lg > .container-fluid {
      padding-right: 0;
      padding-left: 0; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .navbar-expand-lg {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-flow: row nowrap;
      flex-flow: row nowrap;
      -webkit-box-pack: start;
      -ms-flex-pack: start;
      justify-content: flex-start; }
      .wrapper-bootstrap .navbar-expand-lg .navbar-nav {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-direction: row;
        flex-direction: row; }
        .wrapper-bootstrap .navbar-expand-lg .navbar-nav .dropdown-menu {
          position: absolute; }
        .wrapper-bootstrap .navbar-expand-lg .navbar-nav .nav-link {
          padding-right: 0.5rem;
          padding-left: 0.5rem; }
      .wrapper-bootstrap .navbar-expand-lg > .container,
      .wrapper-bootstrap .navbar-expand-lg > .container-fluid {
        -ms-flex-wrap: nowrap;
        flex-wrap: nowrap; }
      .wrapper-bootstrap .navbar-expand-lg .navbar-collapse {
        display: -webkit-box !important;
        display: -ms-flexbox !important;
        display: flex !important;
        -ms-flex-preferred-size: auto;
        flex-basis: auto; }
      .wrapper-bootstrap .navbar-expand-lg .navbar-toggler {
        display: none; } }
  @media (max-width: 1199.98px) {
    .wrapper-bootstrap .navbar-expand-xl > .container,
    .wrapper-bootstrap .navbar-expand-xl > .container-fluid {
      padding-right: 0;
      padding-left: 0; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .navbar-expand-xl {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-flow: row nowrap;
      flex-flow: row nowrap;
      -webkit-box-pack: start;
      -ms-flex-pack: start;
      justify-content: flex-start; }
      .wrapper-bootstrap .navbar-expand-xl .navbar-nav {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-direction: row;
        flex-direction: row; }
        .wrapper-bootstrap .navbar-expand-xl .navbar-nav .dropdown-menu {
          position: absolute; }
        .wrapper-bootstrap .navbar-expand-xl .navbar-nav .nav-link {
          padding-right: 0.5rem;
          padding-left: 0.5rem; }
      .wrapper-bootstrap .navbar-expand-xl > .container,
      .wrapper-bootstrap .navbar-expand-xl > .container-fluid {
        -ms-flex-wrap: nowrap;
        flex-wrap: nowrap; }
      .wrapper-bootstrap .navbar-expand-xl .navbar-collapse {
        display: -webkit-box !important;
        display: -ms-flexbox !important;
        display: flex !important;
        -ms-flex-preferred-size: auto;
        flex-basis: auto; }
      .wrapper-bootstrap .navbar-expand-xl .navbar-toggler {
        display: none; } }
  .wrapper-bootstrap .navbar-expand {
    -webkit-box-orient: horizontal;
    -webkit-box-direction: normal;
    -ms-flex-flow: row nowrap;
    flex-flow: row nowrap;
    -webkit-box-pack: start;
    -ms-flex-pack: start;
    justify-content: flex-start; }
    .wrapper-bootstrap .navbar-expand > .container,
    .wrapper-bootstrap .navbar-expand > .container-fluid {
      padding-right: 0;
      padding-left: 0; }
    .wrapper-bootstrap .navbar-expand .navbar-nav {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-direction: row;
      flex-direction: row; }
      .wrapper-bootstrap .navbar-expand .navbar-nav .dropdown-menu {
        position: absolute; }
      .wrapper-bootstrap .navbar-expand .navbar-nav .nav-link {
        padding-right: 0.5rem;
        padding-left: 0.5rem; }
    .wrapper-bootstrap .navbar-expand > .container,
    .wrapper-bootstrap .navbar-expand > .container-fluid {
      -ms-flex-wrap: nowrap;
      flex-wrap: nowrap; }
    .wrapper-bootstrap .navbar-expand .navbar-collapse {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important;
      -ms-flex-preferred-size: auto;
      flex-basis: auto; }
    .wrapper-bootstrap .navbar-expand .navbar-toggler {
      display: none; }
  .wrapper-bootstrap .navbar-light .navbar-brand {
    color: rgba(0, 0, 0, 0.9); }
    .wrapper-bootstrap .navbar-light .navbar-brand:hover, .wrapper-bootstrap .navbar-light .navbar-brand:focus {
      color: rgba(0, 0, 0, 0.9); }
  .wrapper-bootstrap .navbar-light .navbar-nav .nav-link {
    color: rgba(0, 0, 0, 0.5); }
    .wrapper-bootstrap .navbar-light .navbar-nav .nav-link:hover, .wrapper-bootstrap .navbar-light .navbar-nav .nav-link:focus {
      color: rgba(0, 0, 0, 0.7); }
    .wrapper-bootstrap .navbar-light .navbar-nav .nav-link.disabled {
      color: rgba(0, 0, 0, 0.3); }
  .wrapper-bootstrap .navbar-light .navbar-nav .show > .nav-link,
  .wrapper-bootstrap .navbar-light .navbar-nav .active > .nav-link,
  .wrapper-bootstrap .navbar-light .navbar-nav .nav-link.show,
  .wrapper-bootstrap .navbar-light .navbar-nav .nav-link.active {
    color: rgba(0, 0, 0, 0.9); }
  .wrapper-bootstrap .navbar-light .navbar-toggler {
    color: rgba(0, 0, 0, 0.5);
    border-color: rgba(0, 0, 0, 0.1); }
  .wrapper-bootstrap .navbar-light .navbar-toggler-icon {
    background-image: url("data:image/svg+xml,%3csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3e%3cpath stroke='rgba(0, 0, 0, 0.5)' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .navbar-light .navbar-text {
    color: rgba(0, 0, 0, 0.5); }
    .wrapper-bootstrap .navbar-light .navbar-text a {
      color: rgba(0, 0, 0, 0.9); }
      .wrapper-bootstrap .navbar-light .navbar-text a:hover, .wrapper-bootstrap .navbar-light .navbar-text a:focus {
        color: rgba(0, 0, 0, 0.9); }
  .wrapper-bootstrap .navbar-dark .navbar-brand {
    color: #fff; }
    .wrapper-bootstrap .navbar-dark .navbar-brand:hover, .wrapper-bootstrap .navbar-dark .navbar-brand:focus {
      color: #fff; }
  .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link {
    color: rgba(255, 255, 255, 0.5); }
    .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link:hover, .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link:focus {
      color: rgba(255, 255, 255, 0.75); }
    .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link.disabled {
      color: rgba(255, 255, 255, 0.25); }
  .wrapper-bootstrap .navbar-dark .navbar-nav .show > .nav-link,
  .wrapper-bootstrap .navbar-dark .navbar-nav .active > .nav-link,
  .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link.show,
  .wrapper-bootstrap .navbar-dark .navbar-nav .nav-link.active {
    color: #fff; }
  .wrapper-bootstrap .navbar-dark .navbar-toggler {
    color: rgba(255, 255, 255, 0.5);
    border-color: rgba(255, 255, 255, 0.1); }
  .wrapper-bootstrap .navbar-dark .navbar-toggler-icon {
    background-image: url("data:image/svg+xml,%3csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3e%3cpath stroke='rgba(255, 255, 255, 0.5)' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .navbar-dark .navbar-text {
    color: rgba(255, 255, 255, 0.5); }
    .wrapper-bootstrap .navbar-dark .navbar-text a {
      color: #fff; }
      .wrapper-bootstrap .navbar-dark .navbar-text a:hover, .wrapper-bootstrap .navbar-dark .navbar-text a:focus {
        color: #fff; }
  .wrapper-bootstrap .card {
    position: relative;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    min-width: 0;
    word-wrap: break-word;
    background-color: #fff;
    background-clip: border-box;
    border: 1px solid rgba(0, 0, 0, 0.125);
    border-radius: 0.25rem; }
    .wrapper-bootstrap .card > hr {
      margin-right: 0;
      margin-left: 0; }
    .wrapper-bootstrap .card > .list-group:first-child .list-group-item:first-child {
      border-top-left-radius: 0.25rem;
      border-top-right-radius: 0.25rem; }
    .wrapper-bootstrap .card > .list-group:last-child .list-group-item:last-child {
      border-bottom-right-radius: 0.25rem;
      border-bottom-left-radius: 0.25rem; }
  .wrapper-bootstrap .card-body {
    -webkit-box-flex: 1;
    -ms-flex: 1 1 auto;
    flex: 1 1 auto;
    padding: 1.25rem; }
  .wrapper-bootstrap .card-title {
    margin-bottom: 0.75rem; }
  .wrapper-bootstrap .card-subtitle {
    margin-top: -0.375rem;
    margin-bottom: 0; }
  .wrapper-bootstrap .card-text:last-child {
    margin-bottom: 0; }
  .wrapper-bootstrap .card-link:hover {
    text-decoration: none; }
  .wrapper-bootstrap .card-link + .card-link {
    margin-left: 1.25rem; }
  .wrapper-bootstrap .card-header {
    padding: 0.75rem 1.25rem;
    margin-bottom: 0;
    background-color: rgba(0, 0, 0, 0.03);
    border-bottom: 1px solid rgba(0, 0, 0, 0.125); }
    .wrapper-bootstrap .card-header:first-child {
      border-radius: calc(0.25rem - 1px) calc(0.25rem - 1px) 0 0; }
    .wrapper-bootstrap .card-header + .list-group .list-group-item:first-child {
      border-top: 0; }
  .wrapper-bootstrap .card-footer {
    padding: 0.75rem 1.25rem;
    background-color: rgba(0, 0, 0, 0.03);
    border-top: 1px solid rgba(0, 0, 0, 0.125); }
    .wrapper-bootstrap .card-footer:last-child {
      border-radius: 0 0 calc(0.25rem - 1px) calc(0.25rem - 1px); }
  .wrapper-bootstrap .card-header-tabs {
    margin-right: -0.625rem;
    margin-bottom: -0.75rem;
    margin-left: -0.625rem;
    border-bottom: 0; }
  .wrapper-bootstrap .card-header-pills {
    margin-right: -0.625rem;
    margin-left: -0.625rem; }
  .wrapper-bootstrap .card-img-overlay {
    position: absolute;
    top: 0;
    right: 0;
    bottom: 0;
    left: 0;
    padding: 1.25rem; }
  .wrapper-bootstrap .card-img {
    width: 100%;
    border-radius: calc(0.25rem - 1px); }
  .wrapper-bootstrap .card-img-top {
    width: 100%;
    border-top-left-radius: calc(0.25rem - 1px);
    border-top-right-radius: calc(0.25rem - 1px); }
  .wrapper-bootstrap .card-img-bottom {
    width: 100%;
    border-bottom-right-radius: calc(0.25rem - 1px);
    border-bottom-left-radius: calc(0.25rem - 1px); }
  .wrapper-bootstrap .card-deck {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column; }
    .wrapper-bootstrap .card-deck .card {
      margin-bottom: 15px; }
    @media (min-width: 576px) {
      .wrapper-bootstrap .card-deck {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-flow: row wrap;
        flex-flow: row wrap;
        margin-right: -15px;
        margin-left: -15px; }
        .wrapper-bootstrap .card-deck .card {
          display: -webkit-box;
          display: -ms-flexbox;
          display: flex;
          -webkit-box-flex: 1;
          -ms-flex: 1 0 0%;
          flex: 1 0 0%;
          -webkit-box-orient: vertical;
          -webkit-box-direction: normal;
          -ms-flex-direction: column;
          flex-direction: column;
          margin-right: 15px;
          margin-bottom: 0;
          margin-left: 15px; } }
  .wrapper-bootstrap .card-group {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column; }
    .wrapper-bootstrap .card-group > .card {
      margin-bottom: 15px; }
    @media (min-width: 576px) {
      .wrapper-bootstrap .card-group {
        -webkit-box-orient: horizontal;
        -webkit-box-direction: normal;
        -ms-flex-flow: row wrap;
        flex-flow: row wrap; }
        .wrapper-bootstrap .card-group > .card {
          -webkit-box-flex: 1;
          -ms-flex: 1 0 0%;
          flex: 1 0 0%;
          margin-bottom: 0; }
          .wrapper-bootstrap .card-group > .card + .card {
            margin-left: 0;
            border-left: 0; }
          .wrapper-bootstrap .card-group > .card:not(:last-child) {
            border-top-right-radius: 0;
            border-bottom-right-radius: 0; }
            .wrapper-bootstrap .card-group > .card:not(:last-child) .card-img-top,
            .wrapper-bootstrap .card-group > .card:not(:last-child) .card-header {
              border-top-right-radius: 0; }
            .wrapper-bootstrap .card-group > .card:not(:last-child) .card-img-bottom,
            .wrapper-bootstrap .card-group > .card:not(:last-child) .card-footer {
              border-bottom-right-radius: 0; }
          .wrapper-bootstrap .card-group > .card:not(:first-child) {
            border-top-left-radius: 0;
            border-bottom-left-radius: 0; }
            .wrapper-bootstrap .card-group > .card:not(:first-child) .card-img-top,
            .wrapper-bootstrap .card-group > .card:not(:first-child) .card-header {
              border-top-left-radius: 0; }
            .wrapper-bootstrap .card-group > .card:not(:first-child) .card-img-bottom,
            .wrapper-bootstrap .card-group > .card:not(:first-child) .card-footer {
              border-bottom-left-radius: 0; } }
  .wrapper-bootstrap .card-columns .card {
    margin-bottom: 0.75rem; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .card-columns {
      -webkit-column-count: 3;
      column-count: 3;
      -webkit-column-gap: 1.25rem;
      column-gap: 1.25rem;
      orphans: 1;
      widows: 1; }
      .wrapper-bootstrap .card-columns .card {
        display: inline-block;
        width: 100%; } }
  .wrapper-bootstrap .accordion > .card {
    overflow: hidden; }
    .wrapper-bootstrap .accordion > .card:not(:first-of-type) .card-header:first-child {
      border-radius: 0; }
    .wrapper-bootstrap .accordion > .card:not(:first-of-type):not(:last-of-type) {
      border-bottom: 0;
      border-radius: 0; }
    .wrapper-bootstrap .accordion > .card:first-of-type {
      border-bottom: 0;
      border-bottom-right-radius: 0;
      border-bottom-left-radius: 0; }
    .wrapper-bootstrap .accordion > .card:last-of-type {
      border-top-left-radius: 0;
      border-top-right-radius: 0; }
    .wrapper-bootstrap .accordion > .card .card-header {
      margin-bottom: -1px; }
  .wrapper-bootstrap .breadcrumb {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -ms-flex-wrap: wrap;
    flex-wrap: wrap;
    padding: 0.75rem 1rem;
    margin-bottom: 1rem;
    list-style: none;
    background-color: #e9ecef;
    border-radius: 0.25rem; }
  .wrapper-bootstrap .breadcrumb-item + .breadcrumb-item {
    padding-left: 0.5rem; }
    .wrapper-bootstrap .breadcrumb-item + .breadcrumb-item::before {
      display: inline-block;
      padding-right: 0.5rem;
      color: #6c757d;
      content: "/"; }
  .wrapper-bootstrap .breadcrumb-item + .breadcrumb-item:hover::before {
    text-decoration: underline; }
  .wrapper-bootstrap .breadcrumb-item + .breadcrumb-item:hover::before {
    text-decoration: none; }
  .wrapper-bootstrap .breadcrumb-item.active {
    color: #6c757d; }
  .wrapper-bootstrap .pagination {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    padding-left: 0;
    list-style: none;
    border-radius: 0.25rem; }
  .wrapper-bootstrap .page-link {
    position: relative;
    display: block;
    padding: 0.5rem 0.75rem;
    margin-left: -1px;
    line-height: 1.25;
    color: #007bff;
    background-color: #fff;
    border: 1px solid #dee2e6; }
    .wrapper-bootstrap .page-link:hover {
      z-index: 2;
      color: #0056b3;
      text-decoration: none;
      background-color: #e9ecef;
      border-color: #dee2e6; }
    .wrapper-bootstrap .page-link:focus {
      z-index: 2;
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.25); }
  .wrapper-bootstrap .page-item:first-child .page-link {
    margin-left: 0;
    border-top-left-radius: 0.25rem;
    border-bottom-left-radius: 0.25rem; }
  .wrapper-bootstrap .page-item:last-child .page-link {
    border-top-right-radius: 0.25rem;
    border-bottom-right-radius: 0.25rem; }
  .wrapper-bootstrap .page-item.active .page-link {
    z-index: 1;
    color: #fff;
    background-color: #007bff;
    border-color: #007bff; }
  .wrapper-bootstrap .page-item.disabled .page-link {
    color: #6c757d;
    pointer-events: none;
    cursor: auto;
    background-color: #fff;
    border-color: #dee2e6; }
  .wrapper-bootstrap .pagination-lg .page-link {
    padding: 0.75rem 1.5rem;
    font-size: 1.25rem;
    line-height: 1.5; }
  .wrapper-bootstrap .pagination-lg .page-item:first-child .page-link {
    border-top-left-radius: 0.3rem;
    border-bottom-left-radius: 0.3rem; }
  .wrapper-bootstrap .pagination-lg .page-item:last-child .page-link {
    border-top-right-radius: 0.3rem;
    border-bottom-right-radius: 0.3rem; }
  .wrapper-bootstrap .pagination-sm .page-link {
    padding: 0.25rem 0.5rem;
    font-size: 0.875rem;
    line-height: 1.5; }
  .wrapper-bootstrap .pagination-sm .page-item:first-child .page-link {
    border-top-left-radius: 0.2rem;
    border-bottom-left-radius: 0.2rem; }
  .wrapper-bootstrap .pagination-sm .page-item:last-child .page-link {
    border-top-right-radius: 0.2rem;
    border-bottom-right-radius: 0.2rem; }
  .wrapper-bootstrap .badge {
    display: inline-block;
    padding: 0.25em 0.4em;
    font-size: 75%;
    font-weight: 700;
    line-height: 1;
    text-align: center;
    white-space: nowrap;
    vertical-align: baseline;
    border-radius: 0.25rem;
    -webkit-transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
    transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, -webkit-box-shadow 0.15s ease-in-out; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .badge {
        -webkit-transition: none;
        transition: none; } }
    a.wrapper-bootstrap .badge:hover, a.wrapper-bootstrap .badge:focus {
      text-decoration: none; }
    .wrapper-bootstrap .badge:empty {
      display: none; }
  .wrapper-bootstrap .btn .badge {
    position: relative;
    top: -1px; }
  .wrapper-bootstrap .badge-pill {
    padding-right: 0.6em;
    padding-left: 0.6em;
    border-radius: 10rem; }
  .wrapper-bootstrap .badge-primary {
    color: #fff;
    background-color: #007bff; }
    a.wrapper-bootstrap .badge-primary:hover, a.wrapper-bootstrap .badge-primary:focus {
      color: #fff;
      background-color: #0062cc; }
    a.wrapper-bootstrap .badge-primary:focus, a.wrapper-bootstrap .badge-primary.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(0, 123, 255, 0.5); }
  .wrapper-bootstrap .badge-secondary {
    color: #fff;
    background-color: #6c757d; }
    a.wrapper-bootstrap .badge-secondary:hover, a.wrapper-bootstrap .badge-secondary:focus {
      color: #fff;
      background-color: #545b62; }
    a.wrapper-bootstrap .badge-secondary:focus, a.wrapper-bootstrap .badge-secondary.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(108, 117, 125, 0.5); }
  .wrapper-bootstrap .badge-success {
    color: #fff;
    background-color: #28a745; }
    a.wrapper-bootstrap .badge-success:hover, a.wrapper-bootstrap .badge-success:focus {
      color: #fff;
      background-color: #1e7e34; }
    a.wrapper-bootstrap .badge-success:focus, a.wrapper-bootstrap .badge-success.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(40, 167, 69, 0.5); }
  .wrapper-bootstrap .badge-info {
    color: #fff;
    background-color: #17a2b8; }
    a.wrapper-bootstrap .badge-info:hover, a.wrapper-bootstrap .badge-info:focus {
      color: #fff;
      background-color: #117a8b; }
    a.wrapper-bootstrap .badge-info:focus, a.wrapper-bootstrap .badge-info.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(23, 162, 184, 0.5); }
  .wrapper-bootstrap .badge-warning {
    color: #212529;
    background-color: #ffc107; }
    a.wrapper-bootstrap .badge-warning:hover, a.wrapper-bootstrap .badge-warning:focus {
      color: #212529;
      background-color: #d39e00; }
    a.wrapper-bootstrap .badge-warning:focus, a.wrapper-bootstrap .badge-warning.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(255, 193, 7, 0.5); }
  .wrapper-bootstrap .badge-danger {
    color: #fff;
    background-color: #dc3545; }
    a.wrapper-bootstrap .badge-danger:hover, a.wrapper-bootstrap .badge-danger:focus {
      color: #fff;
      background-color: #bd2130; }
    a.wrapper-bootstrap .badge-danger:focus, a.wrapper-bootstrap .badge-danger.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(220, 53, 69, 0.5); }
  .wrapper-bootstrap .badge-light {
    color: #212529;
    background-color: #f8f9fa; }
    a.wrapper-bootstrap .badge-light:hover, a.wrapper-bootstrap .badge-light:focus {
      color: #212529;
      background-color: #dae0e5; }
    a.wrapper-bootstrap .badge-light:focus, a.wrapper-bootstrap .badge-light.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(248, 249, 250, 0.5); }
  .wrapper-bootstrap .badge-dark {
    color: #fff;
    background-color: #343a40; }
    a.wrapper-bootstrap .badge-dark:hover, a.wrapper-bootstrap .badge-dark:focus {
      color: #fff;
      background-color: #1d2124; }
    a.wrapper-bootstrap .badge-dark:focus, a.wrapper-bootstrap .badge-dark.focus {
      outline: 0;
      -webkit-box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5);
      box-shadow: 0 0 0 0.2rem rgba(52, 58, 64, 0.5); }
  .wrapper-bootstrap .jumbotron {
    padding: 2rem 1rem;
    margin-bottom: 2rem;
    background-color: #e9ecef;
    border-radius: 0.3rem; }
    @media (min-width: 576px) {
      .wrapper-bootstrap .jumbotron {
        padding: 4rem 2rem; } }
  .wrapper-bootstrap .jumbotron-fluid {
    padding-right: 0;
    padding-left: 0;
    border-radius: 0; }
  .wrapper-bootstrap .alert {
    position: relative;
    padding: 0.75rem 1.25rem;
    margin-bottom: 1rem;
    border: 1px solid transparent;
    border-radius: 0.25rem; }
  .wrapper-bootstrap .alert-heading {
    color: inherit; }
  .wrapper-bootstrap .alert-link {
    font-weight: 700; }
  .wrapper-bootstrap .alert-dismissible {
    padding-right: 4rem; }
    .wrapper-bootstrap .alert-dismissible .close {
      position: absolute;
      top: 0;
      right: 0;
      padding: 0.75rem 1.25rem;
      color: inherit; }
  .wrapper-bootstrap .alert-primary {
    color: #004085;
    background-color: #cce5ff;
    border-color: #b8daff; }
    .wrapper-bootstrap .alert-primary hr {
      border-top-color: #9fcdff; }
    .wrapper-bootstrap .alert-primary .alert-link {
      color: #002752; }
  .wrapper-bootstrap .alert-secondary {
    color: #383d41;
    background-color: #e2e3e5;
    border-color: #d6d8db; }
    .wrapper-bootstrap .alert-secondary hr {
      border-top-color: #c8cbcf; }
    .wrapper-bootstrap .alert-secondary .alert-link {
      color: #202326; }
  .wrapper-bootstrap .alert-success {
    color: #155724;
    background-color: #d4edda;
    border-color: #c3e6cb; }
    .wrapper-bootstrap .alert-success hr {
      border-top-color: #b1dfbb; }
    .wrapper-bootstrap .alert-success .alert-link {
      color: #0b2e13; }
  .wrapper-bootstrap .alert-info {
    color: #0c5460;
    background-color: #d1ecf1;
    border-color: #bee5eb; }
    .wrapper-bootstrap .alert-info hr {
      border-top-color: #abdde5; }
    .wrapper-bootstrap .alert-info .alert-link {
      color: #062c33; }
  .wrapper-bootstrap .alert-warning {
    color: #856404;
    background-color: #fff3cd;
    border-color: #ffeeba; }
    .wrapper-bootstrap .alert-warning hr {
      border-top-color: #ffe8a1; }
    .wrapper-bootstrap .alert-warning .alert-link {
      color: #533f03; }
  .wrapper-bootstrap .alert-danger {
    color: #721c24;
    background-color: #f8d7da;
    border-color: #f5c6cb; }
    .wrapper-bootstrap .alert-danger hr {
      border-top-color: #f1b0b7; }
    .wrapper-bootstrap .alert-danger .alert-link {
      color: #491217; }
  .wrapper-bootstrap .alert-light {
    color: #818182;
    background-color: #fefefe;
    border-color: #fdfdfe; }
    .wrapper-bootstrap .alert-light hr {
      border-top-color: #ececf6; }
    .wrapper-bootstrap .alert-light .alert-link {
      color: #686868; }
  .wrapper-bootstrap .alert-dark {
    color: #1b1e21;
    background-color: #d6d8d9;
    border-color: #c6c8ca; }
    .wrapper-bootstrap .alert-dark hr {
      border-top-color: #b9bbbe; }
    .wrapper-bootstrap .alert-dark .alert-link {
      color: #040505; }

@-webkit-keyframes progress-bar-stripes {
  from {
    background-position: 1rem 0; }
  to {
    background-position: 0 0; } }

@keyframes progress-bar-stripes {
  from {
    background-position: 1rem 0; }
  to {
    background-position: 0 0; } }
  .wrapper-bootstrap .progress {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    height: 1rem;
    overflow: hidden;
    font-size: 0.75rem;
    background-color: #e9ecef;
    border-radius: 0.25rem; }
  .wrapper-bootstrap .progress-bar {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    -webkit-box-pack: center;
    -ms-flex-pack: center;
    justify-content: center;
    color: #fff;
    text-align: center;
    white-space: nowrap;
    background-color: #007bff;
    -webkit-transition: width 0.6s ease;
    transition: width 0.6s ease; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .progress-bar {
        -webkit-transition: none;
        transition: none; } }
  .wrapper-bootstrap .progress-bar-striped {
    background-image: linear-gradient(45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
    background-size: 1rem 1rem; }
  .wrapper-bootstrap .progress-bar-animated {
    -webkit-animation: progress-bar-stripes 1s linear infinite;
    animation: progress-bar-stripes 1s linear infinite; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .progress-bar-animated {
        -webkit-animation: none;
        animation: none; } }
  .wrapper-bootstrap .media {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: start;
    -ms-flex-align: start;
    align-items: flex-start; }
  .wrapper-bootstrap .media-body {
    -webkit-box-flex: 1;
    -ms-flex: 1;
    flex: 1; }
  .wrapper-bootstrap .list-group {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    padding-left: 0;
    margin-bottom: 0; }
  .wrapper-bootstrap .list-group-item-action {
    width: 100%;
    color: #495057;
    text-align: inherit; }
    .wrapper-bootstrap .list-group-item-action:hover, .wrapper-bootstrap .list-group-item-action:focus {
      z-index: 1;
      color: #495057;
      text-decoration: none;
      background-color: #f8f9fa; }
    .wrapper-bootstrap .list-group-item-action:active {
      color: #212529;
      background-color: #e9ecef; }
  .wrapper-bootstrap .list-group-item {
    position: relative;
    display: block;
    padding: 0.75rem 1.25rem;
    margin-bottom: -1px;
    background-color: #fff;
    border: 1px solid rgba(0, 0, 0, 0.125); }
    .wrapper-bootstrap .list-group-item:first-child {
      border-top-left-radius: 0.25rem;
      border-top-right-radius: 0.25rem; }
    .wrapper-bootstrap .list-group-item:last-child {
      margin-bottom: 0;
      border-bottom-right-radius: 0.25rem;
      border-bottom-left-radius: 0.25rem; }
    .wrapper-bootstrap .list-group-item.disabled, .wrapper-bootstrap .list-group-item:disabled {
      color: #6c757d;
      pointer-events: none;
      background-color: #fff; }
    .wrapper-bootstrap .list-group-item.active {
      z-index: 2;
      color: #fff;
      background-color: #007bff;
      border-color: #007bff; }
  .wrapper-bootstrap .list-group-horizontal {
    -webkit-box-orient: horizontal;
    -webkit-box-direction: normal;
    -ms-flex-direction: row;
    flex-direction: row; }
    .wrapper-bootstrap .list-group-horizontal .list-group-item {
      margin-right: -1px;
      margin-bottom: 0; }
      .wrapper-bootstrap .list-group-horizontal .list-group-item:first-child {
        border-top-left-radius: 0.25rem;
        border-bottom-left-radius: 0.25rem;
        border-top-right-radius: 0; }
      .wrapper-bootstrap .list-group-horizontal .list-group-item:last-child {
        margin-right: 0;
        border-top-right-radius: 0.25rem;
        border-bottom-right-radius: 0.25rem;
        border-bottom-left-radius: 0; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .list-group-horizontal-sm {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-direction: row;
      flex-direction: row; }
      .wrapper-bootstrap .list-group-horizontal-sm .list-group-item {
        margin-right: -1px;
        margin-bottom: 0; }
        .wrapper-bootstrap .list-group-horizontal-sm .list-group-item:first-child {
          border-top-left-radius: 0.25rem;
          border-bottom-left-radius: 0.25rem;
          border-top-right-radius: 0; }
        .wrapper-bootstrap .list-group-horizontal-sm .list-group-item:last-child {
          margin-right: 0;
          border-top-right-radius: 0.25rem;
          border-bottom-right-radius: 0.25rem;
          border-bottom-left-radius: 0; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .list-group-horizontal-md {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-direction: row;
      flex-direction: row; }
      .wrapper-bootstrap .list-group-horizontal-md .list-group-item {
        margin-right: -1px;
        margin-bottom: 0; }
        .wrapper-bootstrap .list-group-horizontal-md .list-group-item:first-child {
          border-top-left-radius: 0.25rem;
          border-bottom-left-radius: 0.25rem;
          border-top-right-radius: 0; }
        .wrapper-bootstrap .list-group-horizontal-md .list-group-item:last-child {
          margin-right: 0;
          border-top-right-radius: 0.25rem;
          border-bottom-right-radius: 0.25rem;
          border-bottom-left-radius: 0; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .list-group-horizontal-lg {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-direction: row;
      flex-direction: row; }
      .wrapper-bootstrap .list-group-horizontal-lg .list-group-item {
        margin-right: -1px;
        margin-bottom: 0; }
        .wrapper-bootstrap .list-group-horizontal-lg .list-group-item:first-child {
          border-top-left-radius: 0.25rem;
          border-bottom-left-radius: 0.25rem;
          border-top-right-radius: 0; }
        .wrapper-bootstrap .list-group-horizontal-lg .list-group-item:last-child {
          margin-right: 0;
          border-top-right-radius: 0.25rem;
          border-bottom-right-radius: 0.25rem;
          border-bottom-left-radius: 0; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .list-group-horizontal-xl {
      -webkit-box-orient: horizontal;
      -webkit-box-direction: normal;
      -ms-flex-direction: row;
      flex-direction: row; }
      .wrapper-bootstrap .list-group-horizontal-xl .list-group-item {
        margin-right: -1px;
        margin-bottom: 0; }
        .wrapper-bootstrap .list-group-horizontal-xl .list-group-item:first-child {
          border-top-left-radius: 0.25rem;
          border-bottom-left-radius: 0.25rem;
          border-top-right-radius: 0; }
        .wrapper-bootstrap .list-group-horizontal-xl .list-group-item:last-child {
          margin-right: 0;
          border-top-right-radius: 0.25rem;
          border-bottom-right-radius: 0.25rem;
          border-bottom-left-radius: 0; } }
  .wrapper-bootstrap .list-group-flush .list-group-item {
    border-right: 0;
    border-left: 0;
    border-radius: 0; }
    .wrapper-bootstrap .list-group-flush .list-group-item:last-child {
      margin-bottom: -1px; }
  .wrapper-bootstrap .list-group-flush:first-child .list-group-item:first-child {
    border-top: 0; }
  .wrapper-bootstrap .list-group-flush:last-child .list-group-item:last-child {
    margin-bottom: 0;
    border-bottom: 0; }
  .wrapper-bootstrap .list-group-item-primary {
    color: #004085;
    background-color: #b8daff; }
    .wrapper-bootstrap .list-group-item-primary.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-primary.list-group-item-action:focus {
      color: #004085;
      background-color: #9fcdff; }
    .wrapper-bootstrap .list-group-item-primary.list-group-item-action.active {
      color: #fff;
      background-color: #004085;
      border-color: #004085; }
  .wrapper-bootstrap .list-group-item-secondary {
    color: #383d41;
    background-color: #d6d8db; }
    .wrapper-bootstrap .list-group-item-secondary.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-secondary.list-group-item-action:focus {
      color: #383d41;
      background-color: #c8cbcf; }
    .wrapper-bootstrap .list-group-item-secondary.list-group-item-action.active {
      color: #fff;
      background-color: #383d41;
      border-color: #383d41; }
  .wrapper-bootstrap .list-group-item-success {
    color: #155724;
    background-color: #c3e6cb; }
    .wrapper-bootstrap .list-group-item-success.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-success.list-group-item-action:focus {
      color: #155724;
      background-color: #b1dfbb; }
    .wrapper-bootstrap .list-group-item-success.list-group-item-action.active {
      color: #fff;
      background-color: #155724;
      border-color: #155724; }
  .wrapper-bootstrap .list-group-item-info {
    color: #0c5460;
    background-color: #bee5eb; }
    .wrapper-bootstrap .list-group-item-info.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-info.list-group-item-action:focus {
      color: #0c5460;
      background-color: #abdde5; }
    .wrapper-bootstrap .list-group-item-info.list-group-item-action.active {
      color: #fff;
      background-color: #0c5460;
      border-color: #0c5460; }
  .wrapper-bootstrap .list-group-item-warning {
    color: #856404;
    background-color: #ffeeba; }
    .wrapper-bootstrap .list-group-item-warning.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-warning.list-group-item-action:focus {
      color: #856404;
      background-color: #ffe8a1; }
    .wrapper-bootstrap .list-group-item-warning.list-group-item-action.active {
      color: #fff;
      background-color: #856404;
      border-color: #856404; }
  .wrapper-bootstrap .list-group-item-danger {
    color: #721c24;
    background-color: #f5c6cb; }
    .wrapper-bootstrap .list-group-item-danger.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-danger.list-group-item-action:focus {
      color: #721c24;
      background-color: #f1b0b7; }
    .wrapper-bootstrap .list-group-item-danger.list-group-item-action.active {
      color: #fff;
      background-color: #721c24;
      border-color: #721c24; }
  .wrapper-bootstrap .list-group-item-light {
    color: #818182;
    background-color: #fdfdfe; }
    .wrapper-bootstrap .list-group-item-light.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-light.list-group-item-action:focus {
      color: #818182;
      background-color: #ececf6; }
    .wrapper-bootstrap .list-group-item-light.list-group-item-action.active {
      color: #fff;
      background-color: #818182;
      border-color: #818182; }
  .wrapper-bootstrap .list-group-item-dark {
    color: #1b1e21;
    background-color: #c6c8ca; }
    .wrapper-bootstrap .list-group-item-dark.list-group-item-action:hover, .wrapper-bootstrap .list-group-item-dark.list-group-item-action:focus {
      color: #1b1e21;
      background-color: #b9bbbe; }
    .wrapper-bootstrap .list-group-item-dark.list-group-item-action.active {
      color: #fff;
      background-color: #1b1e21;
      border-color: #1b1e21; }
  .wrapper-bootstrap .close {
    float: right;
    font-size: 1.5rem;
    font-weight: 700;
    line-height: 1;
    color: #000;
    text-shadow: 0 1px 0 #fff;
    opacity: .5; }
    .wrapper-bootstrap .close:hover {
      color: #000;
      text-decoration: none; }
    .wrapper-bootstrap .close:not(:disabled):not(.disabled):hover, .wrapper-bootstrap .close:not(:disabled):not(.disabled):focus {
      opacity: .75; }
  .wrapper-bootstrap button.close {
    padding: 0;
    background-color: transparent;
    border: 0;
    -webkit-appearance: none;
    -moz-appearance: none;
    appearance: none; }
  .wrapper-bootstrap a.close.disabled {
    pointer-events: none; }
  .wrapper-bootstrap .toast {
    max-width: 350px;
    overflow: hidden;
    font-size: 0.875rem;
    background-color: rgba(255, 255, 255, 0.85);
    background-clip: padding-box;
    border: 1px solid rgba(0, 0, 0, 0.1);
    -webkit-box-shadow: 0 0.25rem 0.75rem rgba(0, 0, 0, 0.1);
    box-shadow: 0 0.25rem 0.75rem rgba(0, 0, 0, 0.1);
    -webkit-backdrop-filter: blur(10px);
    backdrop-filter: blur(10px);
    opacity: 0;
    border-radius: 0.25rem; }
    .wrapper-bootstrap .toast:not(:last-child) {
      margin-bottom: 0.75rem; }
    .wrapper-bootstrap .toast.showing {
      opacity: 1; }
    .wrapper-bootstrap .toast.show {
      display: block;
      opacity: 1; }
    .wrapper-bootstrap .toast.hide {
      display: none; }
  .wrapper-bootstrap .toast-header {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    padding: 0.25rem 0.75rem;
    color: #6c757d;
    background-color: rgba(255, 255, 255, 0.85);
    background-clip: padding-box;
    border-bottom: 1px solid rgba(0, 0, 0, 0.05); }
  .wrapper-bootstrap .toast-body {
    padding: 0.75rem; }
  .wrapper-bootstrap .modal-open {
    overflow: hidden; }
    .wrapper-bootstrap .modal-open .modal {
      overflow-x: hidden;
      overflow-y: auto; }
  .wrapper-bootstrap .modal {
    position: fixed;
    top: 0;
    left: 0;
    z-index: 1050;
    display: none;
    width: 100%;
    height: 100%;
    overflow: hidden;
    outline: 0; }
  .wrapper-bootstrap .modal-dialog {
    position: relative;
    width: auto;
    margin: 0.5rem;
    pointer-events: none; }
    .modal.fade .wrapper-bootstrap .modal-dialog {
      -webkit-transition: -webkit-transform 0.3s ease-out;
      transition: -webkit-transform 0.3s ease-out;
      transition: transform 0.3s ease-out;
      transition: transform 0.3s ease-out, -webkit-transform 0.3s ease-out;
      -webkit-transform: translate(0, -50px);
      transform: translate(0, -50px); }
      @media (prefers-reduced-motion: reduce) {
        .modal.fade .wrapper-bootstrap .modal-dialog {
          -webkit-transition: none;
          transition: none; } }
    .modal.show .wrapper-bootstrap .modal-dialog {
      -webkit-transform: none;
      transform: none; }
  .wrapper-bootstrap .modal-dialog-scrollable {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    max-height: calc(100% - 1rem); }
    .wrapper-bootstrap .modal-dialog-scrollable .modal-content {
      max-height: calc(100vh - 1rem);
      overflow: hidden; }
    .wrapper-bootstrap .modal-dialog-scrollable .modal-header,
    .wrapper-bootstrap .modal-dialog-scrollable .modal-footer {
      -ms-flex-negative: 0;
      flex-shrink: 0; }
    .wrapper-bootstrap .modal-dialog-scrollable .modal-body {
      overflow-y: auto; }
  .wrapper-bootstrap .modal-dialog-centered {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    min-height: calc(100% - 1rem); }
    .wrapper-bootstrap .modal-dialog-centered::before {
      display: block;
      height: calc(100vh - 1rem);
      content: ""; }
    .wrapper-bootstrap .modal-dialog-centered.modal-dialog-scrollable {
      -webkit-box-orient: vertical;
      -webkit-box-direction: normal;
      -ms-flex-direction: column;
      flex-direction: column;
      -webkit-box-pack: center;
      -ms-flex-pack: center;
      justify-content: center;
      height: 100%; }
      .wrapper-bootstrap .modal-dialog-centered.modal-dialog-scrollable .modal-content {
        max-height: none; }
      .wrapper-bootstrap .modal-dialog-centered.modal-dialog-scrollable::before {
        content: none; }
  .wrapper-bootstrap .modal-content {
    position: relative;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-orient: vertical;
    -webkit-box-direction: normal;
    -ms-flex-direction: column;
    flex-direction: column;
    width: 100%;
    pointer-events: auto;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid rgba(0, 0, 0, 0.2);
    border-radius: 0.3rem;
    outline: 0; }
  .wrapper-bootstrap .modal-backdrop {
    position: fixed;
    top: 0;
    left: 0;
    z-index: 1040;
    width: 100vw;
    height: 100vh;
    background-color: #000; }
    .wrapper-bootstrap .modal-backdrop.fade {
      opacity: 0; }
    .wrapper-bootstrap .modal-backdrop.show {
      opacity: 0.5; }
  .wrapper-bootstrap .modal-header {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: start;
    -ms-flex-align: start;
    align-items: flex-start;
    -webkit-box-pack: justify;
    -ms-flex-pack: justify;
    justify-content: space-between;
    padding: 1rem 1rem;
    border-bottom: 1px solid #dee2e6;
    border-top-left-radius: 0.3rem;
    border-top-right-radius: 0.3rem; }
    .wrapper-bootstrap .modal-header .close {
      padding: 1rem 1rem;
      margin: -1rem -1rem -1rem auto; }
  .wrapper-bootstrap .modal-title {
    margin-bottom: 0;
    line-height: 1.5; }
  .wrapper-bootstrap .modal-body {
    position: relative;
    -webkit-box-flex: 1;
    -ms-flex: 1 1 auto;
    flex: 1 1 auto;
    padding: 1rem; }
  .wrapper-bootstrap .modal-footer {
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    -webkit-box-pack: end;
    -ms-flex-pack: end;
    justify-content: flex-end;
    padding: 1rem;
    border-top: 1px solid #dee2e6;
    border-bottom-right-radius: 0.3rem;
    border-bottom-left-radius: 0.3rem; }
    .wrapper-bootstrap .modal-footer > :not(:first-child) {
      margin-left: .25rem; }
    .wrapper-bootstrap .modal-footer > :not(:last-child) {
      margin-right: .25rem; }
  .wrapper-bootstrap .modal-scrollbar-measure {
    position: absolute;
    top: -9999px;
    width: 50px;
    height: 50px;
    overflow: scroll; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .modal-dialog {
      max-width: 500px;
      margin: 1.75rem auto; }
    .wrapper-bootstrap .modal-dialog-scrollable {
      max-height: calc(100% - 3.5rem); }
      .wrapper-bootstrap .modal-dialog-scrollable .modal-content {
        max-height: calc(100vh - 3.5rem); }
    .wrapper-bootstrap .modal-dialog-centered {
      min-height: calc(100% - 3.5rem); }
      .wrapper-bootstrap .modal-dialog-centered::before {
        height: calc(100vh - 3.5rem); }
    .wrapper-bootstrap .modal-sm {
      max-width: 300px; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .modal-lg,
    .wrapper-bootstrap .modal-xl {
      max-width: 800px; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .modal-xl {
      max-width: 1140px; } }
  .wrapper-bootstrap .tooltip {
    position: absolute;
    z-index: 1070;
    display: block;
    margin: 0;
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    font-style: normal;
    font-weight: 400;
    line-height: 1.5;
    text-align: left;
    text-align: start;
    text-decoration: none;
    text-shadow: none;
    text-transform: none;
    letter-spacing: normal;
    word-break: normal;
    word-spacing: normal;
    white-space: normal;
    line-break: auto;
    font-size: 0.875rem;
    word-wrap: break-word;
    opacity: 0; }
    .wrapper-bootstrap .tooltip.show {
      opacity: 0.9; }
    .wrapper-bootstrap .tooltip .arrow {
      position: absolute;
      display: block;
      width: 0.8rem;
      height: 0.4rem; }
      .wrapper-bootstrap .tooltip .arrow::before {
        position: absolute;
        content: "";
        border-color: transparent;
        border-style: solid; }
  .wrapper-bootstrap .bs-tooltip-top, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="top"] {
    padding: 0.4rem 0; }
    .wrapper-bootstrap .bs-tooltip-top .arrow, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="top"] .arrow {
      bottom: 0; }
      .wrapper-bootstrap .bs-tooltip-top .arrow::before, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="top"] .arrow::before {
        top: 0;
        border-width: 0.4rem 0.4rem 0;
        border-top-color: #000; }
  .wrapper-bootstrap .bs-tooltip-right, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="right"] {
    padding: 0 0.4rem; }
    .wrapper-bootstrap .bs-tooltip-right .arrow, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="right"] .arrow {
      left: 0;
      width: 0.4rem;
      height: 0.8rem; }
      .wrapper-bootstrap .bs-tooltip-right .arrow::before, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="right"] .arrow::before {
        right: 0;
        border-width: 0.4rem 0.4rem 0.4rem 0;
        border-right-color: #000; }
  .wrapper-bootstrap .bs-tooltip-bottom, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="bottom"] {
    padding: 0.4rem 0; }
    .wrapper-bootstrap .bs-tooltip-bottom .arrow, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="bottom"] .arrow {
      top: 0; }
      .wrapper-bootstrap .bs-tooltip-bottom .arrow::before, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="bottom"] .arrow::before {
        bottom: 0;
        border-width: 0 0.4rem 0.4rem;
        border-bottom-color: #000; }
  .wrapper-bootstrap .bs-tooltip-left, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="left"] {
    padding: 0 0.4rem; }
    .wrapper-bootstrap .bs-tooltip-left .arrow, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="left"] .arrow {
      right: 0;
      width: 0.4rem;
      height: 0.8rem; }
      .wrapper-bootstrap .bs-tooltip-left .arrow::before, .wrapper-bootstrap .bs-tooltip-auto[x-placement^="left"] .arrow::before {
        left: 0;
        border-width: 0.4rem 0 0.4rem 0.4rem;
        border-left-color: #000; }
  .wrapper-bootstrap .tooltip-inner {
    max-width: 200px;
    padding: 0.25rem 0.5rem;
    color: #fff;
    text-align: center;
    background-color: #000;
    border-radius: 0.25rem; }
  .wrapper-bootstrap .popover {
    position: absolute;
    top: 0;
    left: 0;
    z-index: 1060;
    display: block;
    max-width: 276px;
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    font-style: normal;
    font-weight: 400;
    line-height: 1.5;
    text-align: left;
    text-align: start;
    text-decoration: none;
    text-shadow: none;
    text-transform: none;
    letter-spacing: normal;
    word-break: normal;
    word-spacing: normal;
    white-space: normal;
    line-break: auto;
    font-size: 0.875rem;
    word-wrap: break-word;
    background-color: #fff;
    background-clip: padding-box;
    border: 1px solid rgba(0, 0, 0, 0.2);
    border-radius: 0.3rem; }
    .wrapper-bootstrap .popover .arrow {
      position: absolute;
      display: block;
      width: 1rem;
      height: 0.5rem;
      margin: 0 0.3rem; }
      .wrapper-bootstrap .popover .arrow::before, .wrapper-bootstrap .popover .arrow::after {
        position: absolute;
        display: block;
        content: "";
        border-color: transparent;
        border-style: solid; }
  .wrapper-bootstrap .bs-popover-top, .wrapper-bootstrap .bs-popover-auto[x-placement^="top"] {
    margin-bottom: 0.5rem; }
    .wrapper-bootstrap .bs-popover-top > .arrow, .wrapper-bootstrap .bs-popover-auto[x-placement^="top"] > .arrow {
      bottom: calc((0.5rem + 1px) * -1); }
      .wrapper-bootstrap .bs-popover-top > .arrow::before, .wrapper-bootstrap .bs-popover-auto[x-placement^="top"] > .arrow::before {
        bottom: 0;
        border-width: 0.5rem 0.5rem 0;
        border-top-color: rgba(0, 0, 0, 0.25); }
      .wrapper-bootstrap .bs-popover-top > .arrow::after, .wrapper-bootstrap .bs-popover-auto[x-placement^="top"] > .arrow::after {
        bottom: 1px;
        border-width: 0.5rem 0.5rem 0;
        border-top-color: #fff; }
  .wrapper-bootstrap .bs-popover-right, .wrapper-bootstrap .bs-popover-auto[x-placement^="right"] {
    margin-left: 0.5rem; }
    .wrapper-bootstrap .bs-popover-right > .arrow, .wrapper-bootstrap .bs-popover-auto[x-placement^="right"] > .arrow {
      left: calc((0.5rem + 1px) * -1);
      width: 0.5rem;
      height: 1rem;
      margin: 0.3rem 0; }
      .wrapper-bootstrap .bs-popover-right > .arrow::before, .wrapper-bootstrap .bs-popover-auto[x-placement^="right"] > .arrow::before {
        left: 0;
        border-width: 0.5rem 0.5rem 0.5rem 0;
        border-right-color: rgba(0, 0, 0, 0.25); }
      .wrapper-bootstrap .bs-popover-right > .arrow::after, .wrapper-bootstrap .bs-popover-auto[x-placement^="right"] > .arrow::after {
        left: 1px;
        border-width: 0.5rem 0.5rem 0.5rem 0;
        border-right-color: #fff; }
  .wrapper-bootstrap .bs-popover-bottom, .wrapper-bootstrap .bs-popover-auto[x-placement^="bottom"] {
    margin-top: 0.5rem; }
    .wrapper-bootstrap .bs-popover-bottom > .arrow, .wrapper-bootstrap .bs-popover-auto[x-placement^="bottom"] > .arrow {
      top: calc((0.5rem + 1px) * -1); }
      .wrapper-bootstrap .bs-popover-bottom > .arrow::before, .wrapper-bootstrap .bs-popover-auto[x-placement^="bottom"] > .arrow::before {
        top: 0;
        border-width: 0 0.5rem 0.5rem 0.5rem;
        border-bottom-color: rgba(0, 0, 0, 0.25); }
      .wrapper-bootstrap .bs-popover-bottom > .arrow::after, .wrapper-bootstrap .bs-popover-auto[x-placement^="bottom"] > .arrow::after {
        top: 1px;
        border-width: 0 0.5rem 0.5rem 0.5rem;
        border-bottom-color: #fff; }
    .wrapper-bootstrap .bs-popover-bottom .popover-header::before, .wrapper-bootstrap .bs-popover-auto[x-placement^="bottom"] .popover-header::before {
      position: absolute;
      top: 0;
      left: 50%;
      display: block;
      width: 1rem;
      margin-left: -0.5rem;
      content: "";
      border-bottom: 1px solid #f7f7f7; }
  .wrapper-bootstrap .bs-popover-left, .wrapper-bootstrap .bs-popover-auto[x-placement^="left"] {
    margin-right: 0.5rem; }
    .wrapper-bootstrap .bs-popover-left > .arrow, .wrapper-bootstrap .bs-popover-auto[x-placement^="left"] > .arrow {
      right: calc((0.5rem + 1px) * -1);
      width: 0.5rem;
      height: 1rem;
      margin: 0.3rem 0; }
      .wrapper-bootstrap .bs-popover-left > .arrow::before, .wrapper-bootstrap .bs-popover-auto[x-placement^="left"] > .arrow::before {
        right: 0;
        border-width: 0.5rem 0 0.5rem 0.5rem;
        border-left-color: rgba(0, 0, 0, 0.25); }
      .wrapper-bootstrap .bs-popover-left > .arrow::after, .wrapper-bootstrap .bs-popover-auto[x-placement^="left"] > .arrow::after {
        right: 1px;
        border-width: 0.5rem 0 0.5rem 0.5rem;
        border-left-color: #fff; }
  .wrapper-bootstrap .popover-header {
    padding: 0.5rem 0.75rem;
    margin-bottom: 0;
    font-size: 1rem;
    background-color: #f7f7f7;
    border-bottom: 1px solid #ebebeb;
    border-top-left-radius: calc(0.3rem - 1px);
    border-top-right-radius: calc(0.3rem - 1px); }
    .wrapper-bootstrap .popover-header:empty {
      display: none; }
  .wrapper-bootstrap .popover-body {
    padding: 0.5rem 0.75rem;
    color: #212529; }
  .wrapper-bootstrap .carousel {
    position: relative; }
  .wrapper-bootstrap .carousel.pointer-event {
    -ms-touch-action: pan-y;
    touch-action: pan-y; }
  .wrapper-bootstrap .carousel-inner {
    position: relative;
    width: 100%;
    overflow: hidden; }
    .wrapper-bootstrap .carousel-inner::after {
      display: block;
      clear: both;
      content: ""; }
  .wrapper-bootstrap .carousel-item {
    position: relative;
    display: none;
    float: left;
    width: 100%;
    margin-right: -100%;
    -webkit-backface-visibility: hidden;
    backface-visibility: hidden;
    -webkit-transition: -webkit-transform 0.6s ease-in-out;
    transition: -webkit-transform 0.6s ease-in-out;
    transition: transform 0.6s ease-in-out;
    transition: transform 0.6s ease-in-out, -webkit-transform 0.6s ease-in-out; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .carousel-item {
        -webkit-transition: none;
        transition: none; } }
  .wrapper-bootstrap .carousel-item.active,
  .wrapper-bootstrap .carousel-item-next,
  .wrapper-bootstrap .carousel-item-prev {
    display: block; }
  .wrapper-bootstrap .carousel-item-next:not(.carousel-item-left),
  .wrapper-bootstrap .active.carousel-item-right {
    -webkit-transform: translateX(100%);
    transform: translateX(100%); }
  .wrapper-bootstrap .carousel-item-prev:not(.carousel-item-right),
  .wrapper-bootstrap .active.carousel-item-left {
    -webkit-transform: translateX(-100%);
    transform: translateX(-100%); }
  .wrapper-bootstrap .carousel-fade .carousel-item {
    opacity: 0;
    -webkit-transition-property: opacity;
    transition-property: opacity;
    -webkit-transform: none;
    transform: none; }
  .wrapper-bootstrap .carousel-fade .carousel-item.active,
  .wrapper-bootstrap .carousel-fade .carousel-item-next.carousel-item-left,
  .wrapper-bootstrap .carousel-fade .carousel-item-prev.carousel-item-right {
    z-index: 1;
    opacity: 1; }
  .wrapper-bootstrap .carousel-fade .active.carousel-item-left,
  .wrapper-bootstrap .carousel-fade .active.carousel-item-right {
    z-index: 0;
    opacity: 0;
    -webkit-transition: 0s 0.6s opacity;
    transition: 0s 0.6s opacity; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .carousel-fade .active.carousel-item-left,
      .wrapper-bootstrap .carousel-fade .active.carousel-item-right {
        -webkit-transition: none;
        transition: none; } }
  .wrapper-bootstrap .carousel-control-prev,
  .wrapper-bootstrap .carousel-control-next {
    position: absolute;
    top: 0;
    bottom: 0;
    z-index: 1;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-align: center;
    -ms-flex-align: center;
    align-items: center;
    -webkit-box-pack: center;
    -ms-flex-pack: center;
    justify-content: center;
    width: 15%;
    color: #fff;
    text-align: center;
    opacity: 0.5;
    -webkit-transition: opacity 0.15s ease;
    transition: opacity 0.15s ease; }
    @media (prefers-reduced-motion: reduce) {
      .wrapper-bootstrap .carousel-control-prev,
      .wrapper-bootstrap .carousel-control-next {
        -webkit-transition: none;
        transition: none; } }
    .wrapper-bootstrap .carousel-control-prev:hover, .wrapper-bootstrap .carousel-control-prev:focus,
    .wrapper-bootstrap .carousel-control-next:hover,
    .wrapper-bootstrap .carousel-control-next:focus {
      color: #fff;
      text-decoration: none;
      outline: 0;
      opacity: 0.9; }
  .wrapper-bootstrap .carousel-control-prev {
    left: 0; }
  .wrapper-bootstrap .carousel-control-next {
    right: 0; }
  .wrapper-bootstrap .carousel-control-prev-icon,
  .wrapper-bootstrap .carousel-control-next-icon {
    display: inline-block;
    width: 20px;
    height: 20px;
    background: no-repeat 50% / 100% 100%; }
  .wrapper-bootstrap .carousel-control-prev-icon {
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23fff' viewBox='0 0 8 8'%3e%3cpath d='M5.25 0l-4 4 4 4 1.5-1.5-2.5-2.5 2.5-2.5-1.5-1.5z'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .carousel-control-next-icon {
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='%23fff' viewBox='0 0 8 8'%3e%3cpath d='M2.75 0l-1.5 1.5 2.5 2.5-2.5 2.5 1.5 1.5 4-4-4-4z'/%3e%3c/svg%3e"); }
  .wrapper-bootstrap .carousel-indicators {
    position: absolute;
    right: 0;
    bottom: 0;
    left: 0;
    z-index: 15;
    display: -webkit-box;
    display: -ms-flexbox;
    display: flex;
    -webkit-box-pack: center;
    -ms-flex-pack: center;
    justify-content: center;
    padding-left: 0;
    margin-right: 15%;
    margin-left: 15%;
    list-style: none; }
    .wrapper-bootstrap .carousel-indicators li {
      -webkit-box-sizing: content-box;
      box-sizing: content-box;
      -webkit-box-flex: 0;
      -ms-flex: 0 1 auto;
      flex: 0 1 auto;
      width: 30px;
      height: 3px;
      margin-right: 3px;
      margin-left: 3px;
      text-indent: -999px;
      cursor: pointer;
      background-color: #fff;
      background-clip: padding-box;
      border-top: 10px solid transparent;
      border-bottom: 10px solid transparent;
      opacity: .5;
      -webkit-transition: opacity 0.6s ease;
      transition: opacity 0.6s ease; }
      @media (prefers-reduced-motion: reduce) {
        .wrapper-bootstrap .carousel-indicators li {
          -webkit-transition: none;
          transition: none; } }
    .wrapper-bootstrap .carousel-indicators .active {
      opacity: 1; }
  .wrapper-bootstrap .carousel-caption {
    position: absolute;
    right: 15%;
    bottom: 20px;
    left: 15%;
    z-index: 10;
    padding-top: 20px;
    padding-bottom: 20px;
    color: #fff;
    text-align: center; }

@-webkit-keyframes spinner-border {
  to {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg); } }

@keyframes spinner-border {
  to {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg); } }
  .wrapper-bootstrap .spinner-border {
    display: inline-block;
    width: 2rem;
    height: 2rem;
    vertical-align: text-bottom;
    border: 0.25em solid currentColor;
    border-right-color: transparent;
    border-radius: 50%;
    -webkit-animation: spinner-border .75s linear infinite;
    animation: spinner-border .75s linear infinite; }
  .wrapper-bootstrap .spinner-border-sm {
    width: 1rem;
    height: 1rem;
    border-width: 0.2em; }

@-webkit-keyframes spinner-grow {
  0% {
    -webkit-transform: scale(0);
    transform: scale(0); }
  50% {
    opacity: 1; } }

@keyframes spinner-grow {
  0% {
    -webkit-transform: scale(0);
    transform: scale(0); }
  50% {
    opacity: 1; } }
  .wrapper-bootstrap .spinner-grow {
    display: inline-block;
    width: 2rem;
    height: 2rem;
    vertical-align: text-bottom;
    background-color: currentColor;
    border-radius: 50%;
    opacity: 0;
    -webkit-animation: spinner-grow .75s linear infinite;
    animation: spinner-grow .75s linear infinite; }
  .wrapper-bootstrap .spinner-grow-sm {
    width: 1rem;
    height: 1rem; }
  .wrapper-bootstrap .align-baseline {
    vertical-align: baseline !important; }
  .wrapper-bootstrap .align-top {
    vertical-align: top !important; }
  .wrapper-bootstrap .align-middle {
    vertical-align: middle !important; }
  .wrapper-bootstrap .align-bottom {
    vertical-align: bottom !important; }
  .wrapper-bootstrap .align-text-bottom {
    vertical-align: text-bottom !important; }
  .wrapper-bootstrap .align-text-top {
    vertical-align: text-top !important; }
  .wrapper-bootstrap .bg-primary {
    background-color: #007bff !important; }
  .wrapper-bootstrap a.bg-primary:hover, .wrapper-bootstrap a.bg-primary:focus,
  .wrapper-bootstrap button.bg-primary:hover,
  .wrapper-bootstrap button.bg-primary:focus {
    background-color: #0062cc !important; }
  .wrapper-bootstrap .bg-secondary {
    background-color: #6c757d !important; }
  .wrapper-bootstrap a.bg-secondary:hover, .wrapper-bootstrap a.bg-secondary:focus,
  .wrapper-bootstrap button.bg-secondary:hover,
  .wrapper-bootstrap button.bg-secondary:focus {
    background-color: #545b62 !important; }
  .wrapper-bootstrap .bg-success {
    background-color: #28a745 !important; }
  .wrapper-bootstrap a.bg-success:hover, .wrapper-bootstrap a.bg-success:focus,
  .wrapper-bootstrap button.bg-success:hover,
  .wrapper-bootstrap button.bg-success:focus {
    background-color: #1e7e34 !important; }
  .wrapper-bootstrap .bg-info {
    background-color: #17a2b8 !important; }
  .wrapper-bootstrap a.bg-info:hover, .wrapper-bootstrap a.bg-info:focus,
  .wrapper-bootstrap button.bg-info:hover,
  .wrapper-bootstrap button.bg-info:focus {
    background-color: #117a8b !important; }
  .wrapper-bootstrap .bg-warning {
    background-color: #ffc107 !important; }
  .wrapper-bootstrap a.bg-warning:hover, .wrapper-bootstrap a.bg-warning:focus,
  .wrapper-bootstrap button.bg-warning:hover,
  .wrapper-bootstrap button.bg-warning:focus {
    background-color: #d39e00 !important; }
  .wrapper-bootstrap .bg-danger {
    background-color: #dc3545 !important; }
  .wrapper-bootstrap a.bg-danger:hover, .wrapper-bootstrap a.bg-danger:focus,
  .wrapper-bootstrap button.bg-danger:hover,
  .wrapper-bootstrap button.bg-danger:focus {
    background-color: #bd2130 !important; }
  .wrapper-bootstrap .bg-light {
    background-color: #f8f9fa !important; }
  .wrapper-bootstrap a.bg-light:hover, .wrapper-bootstrap a.bg-light:focus,
  .wrapper-bootstrap button.bg-light:hover,
  .wrapper-bootstrap button.bg-light:focus {
    background-color: #dae0e5 !important; }
  .wrapper-bootstrap .bg-dark {
    background-color: #343a40 !important; }
  .wrapper-bootstrap a.bg-dark:hover, .wrapper-bootstrap a.bg-dark:focus,
  .wrapper-bootstrap button.bg-dark:hover,
  .wrapper-bootstrap button.bg-dark:focus {
    background-color: #1d2124 !important; }
  .wrapper-bootstrap .bg-white {
    background-color: #fff !important; }
  .wrapper-bootstrap .bg-transparent {
    background-color: transparent !important; }
  .wrapper-bootstrap .border {
    border: 1px solid #dee2e6 !important; }
  .wrapper-bootstrap .border-top {
    border-top: 1px solid #dee2e6 !important; }
  .wrapper-bootstrap .border-right {
    border-right: 1px solid #dee2e6 !important; }
  .wrapper-bootstrap .border-bottom {
    border-bottom: 1px solid #dee2e6 !important; }
  .wrapper-bootstrap .border-left {
    border-left: 1px solid #dee2e6 !important; }
  .wrapper-bootstrap .border-0 {
    border: 0 !important; }
  .wrapper-bootstrap .border-top-0 {
    border-top: 0 !important; }
  .wrapper-bootstrap .border-right-0 {
    border-right: 0 !important; }
  .wrapper-bootstrap .border-bottom-0 {
    border-bottom: 0 !important; }
  .wrapper-bootstrap .border-left-0 {
    border-left: 0 !important; }
  .wrapper-bootstrap .border-primary {
    border-color: #007bff !important; }
  .wrapper-bootstrap .border-secondary {
    border-color: #6c757d !important; }
  .wrapper-bootstrap .border-success {
    border-color: #28a745 !important; }
  .wrapper-bootstrap .border-info {
    border-color: #17a2b8 !important; }
  .wrapper-bootstrap .border-warning {
    border-color: #ffc107 !important; }
  .wrapper-bootstrap .border-danger {
    border-color: #dc3545 !important; }
  .wrapper-bootstrap .border-light {
    border-color: #f8f9fa !important; }
  .wrapper-bootstrap .border-dark {
    border-color: #343a40 !important; }
  .wrapper-bootstrap .border-white {
    border-color: #fff !important; }
  .wrapper-bootstrap .rounded-sm {
    border-radius: 0.2rem !important; }
  .wrapper-bootstrap .rounded {
    border-radius: 0.25rem !important; }
  .wrapper-bootstrap .rounded-top {
    border-top-left-radius: 0.25rem !important;
    border-top-right-radius: 0.25rem !important; }
  .wrapper-bootstrap .rounded-right {
    border-top-right-radius: 0.25rem !important;
    border-bottom-right-radius: 0.25rem !important; }
  .wrapper-bootstrap .rounded-bottom {
    border-bottom-right-radius: 0.25rem !important;
    border-bottom-left-radius: 0.25rem !important; }
  .wrapper-bootstrap .rounded-left {
    border-top-left-radius: 0.25rem !important;
    border-bottom-left-radius: 0.25rem !important; }
  .wrapper-bootstrap .rounded-lg {
    border-radius: 0.3rem !important; }
  .wrapper-bootstrap .rounded-circle {
    border-radius: 50% !important; }
  .wrapper-bootstrap .rounded-pill {
    border-radius: 50rem !important; }
  .wrapper-bootstrap .rounded-0 {
    border-radius: 0 !important; }
  .wrapper-bootstrap .clearfix::after {
    display: block;
    clear: both;
    content: ""; }
  .wrapper-bootstrap .d-none {
    display: none !important; }
  .wrapper-bootstrap .d-inline {
    display: inline !important; }
  .wrapper-bootstrap .d-inline-block {
    display: inline-block !important; }
  .wrapper-bootstrap .d-block {
    display: block !important; }
  .wrapper-bootstrap .d-table {
    display: table !important; }
  .wrapper-bootstrap .d-table-row {
    display: table-row !important; }
  .wrapper-bootstrap .d-table-cell {
    display: table-cell !important; }
  .wrapper-bootstrap .d-flex {
    display: -webkit-box !important;
    display: -ms-flexbox !important;
    display: flex !important; }
  .wrapper-bootstrap .d-inline-flex {
    display: -webkit-inline-box !important;
    display: -ms-inline-flexbox !important;
    display: inline-flex !important; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .d-sm-none {
      display: none !important; }
    .wrapper-bootstrap .d-sm-inline {
      display: inline !important; }
    .wrapper-bootstrap .d-sm-inline-block {
      display: inline-block !important; }
    .wrapper-bootstrap .d-sm-block {
      display: block !important; }
    .wrapper-bootstrap .d-sm-table {
      display: table !important; }
    .wrapper-bootstrap .d-sm-table-row {
      display: table-row !important; }
    .wrapper-bootstrap .d-sm-table-cell {
      display: table-cell !important; }
    .wrapper-bootstrap .d-sm-flex {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important; }
    .wrapper-bootstrap .d-sm-inline-flex {
      display: -webkit-inline-box !important;
      display: -ms-inline-flexbox !important;
      display: inline-flex !important; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .d-md-none {
      display: none !important; }
    .wrapper-bootstrap .d-md-inline {
      display: inline !important; }
    .wrapper-bootstrap .d-md-inline-block {
      display: inline-block !important; }
    .wrapper-bootstrap .d-md-block {
      display: block !important; }
    .wrapper-bootstrap .d-md-table {
      display: table !important; }
    .wrapper-bootstrap .d-md-table-row {
      display: table-row !important; }
    .wrapper-bootstrap .d-md-table-cell {
      display: table-cell !important; }
    .wrapper-bootstrap .d-md-flex {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important; }
    .wrapper-bootstrap .d-md-inline-flex {
      display: -webkit-inline-box !important;
      display: -ms-inline-flexbox !important;
      display: inline-flex !important; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .d-lg-none {
      display: none !important; }
    .wrapper-bootstrap .d-lg-inline {
      display: inline !important; }
    .wrapper-bootstrap .d-lg-inline-block {
      display: inline-block !important; }
    .wrapper-bootstrap .d-lg-block {
      display: block !important; }
    .wrapper-bootstrap .d-lg-table {
      display: table !important; }
    .wrapper-bootstrap .d-lg-table-row {
      display: table-row !important; }
    .wrapper-bootstrap .d-lg-table-cell {
      display: table-cell !important; }
    .wrapper-bootstrap .d-lg-flex {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important; }
    .wrapper-bootstrap .d-lg-inline-flex {
      display: -webkit-inline-box !important;
      display: -ms-inline-flexbox !important;
      display: inline-flex !important; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .d-xl-none {
      display: none !important; }
    .wrapper-bootstrap .d-xl-inline {
      display: inline !important; }
    .wrapper-bootstrap .d-xl-inline-block {
      display: inline-block !important; }
    .wrapper-bootstrap .d-xl-block {
      display: block !important; }
    .wrapper-bootstrap .d-xl-table {
      display: table !important; }
    .wrapper-bootstrap .d-xl-table-row {
      display: table-row !important; }
    .wrapper-bootstrap .d-xl-table-cell {
      display: table-cell !important; }
    .wrapper-bootstrap .d-xl-flex {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important; }
    .wrapper-bootstrap .d-xl-inline-flex {
      display: -webkit-inline-box !important;
      display: -ms-inline-flexbox !important;
      display: inline-flex !important; } }
  @media print {
    .wrapper-bootstrap .d-print-none {
      display: none !important; }
    .wrapper-bootstrap .d-print-inline {
      display: inline !important; }
    .wrapper-bootstrap .d-print-inline-block {
      display: inline-block !important; }
    .wrapper-bootstrap .d-print-block {
      display: block !important; }
    .wrapper-bootstrap .d-print-table {
      display: table !important; }
    .wrapper-bootstrap .d-print-table-row {
      display: table-row !important; }
    .wrapper-bootstrap .d-print-table-cell {
      display: table-cell !important; }
    .wrapper-bootstrap .d-print-flex {
      display: -webkit-box !important;
      display: -ms-flexbox !important;
      display: flex !important; }
    .wrapper-bootstrap .d-print-inline-flex {
      display: -webkit-inline-box !important;
      display: -ms-inline-flexbox !important;
      display: inline-flex !important; } }
  .wrapper-bootstrap .embed-responsive {
    position: relative;
    display: block;
    width: 100%;
    padding: 0;
    overflow: hidden; }
    .wrapper-bootstrap .embed-responsive::before {
      display: block;
      content: ""; }
    .wrapper-bootstrap .embed-responsive .embed-responsive-item,
    .wrapper-bootstrap .embed-responsive iframe,
    .wrapper-bootstrap .embed-responsive embed,
    .wrapper-bootstrap .embed-responsive object,
    .wrapper-bootstrap .embed-responsive video {
      position: absolute;
      top: 0;
      bottom: 0;
      left: 0;
      width: 100%;
      height: 100%;
      border: 0; }
  .wrapper-bootstrap .embed-responsive-21by9::before {
    padding-top: 42.85714%; }
  .wrapper-bootstrap .embed-responsive-16by9::before {
    padding-top: 56.25%; }
  .wrapper-bootstrap .embed-responsive-4by3::before {
    padding-top: 75%; }
  .wrapper-bootstrap .embed-responsive-1by1::before {
    padding-top: 100%; }
  .wrapper-bootstrap .flex-row {
    -webkit-box-orient: horizontal !important;
    -webkit-box-direction: normal !important;
    -ms-flex-direction: row !important;
    flex-direction: row !important; }
  .wrapper-bootstrap .flex-column {
    -webkit-box-orient: vertical !important;
    -webkit-box-direction: normal !important;
    -ms-flex-direction: column !important;
    flex-direction: column !important; }
  .wrapper-bootstrap .flex-row-reverse {
    -webkit-box-orient: horizontal !important;
    -webkit-box-direction: reverse !important;
    -ms-flex-direction: row-reverse !important;
    flex-direction: row-reverse !important; }
  .wrapper-bootstrap .flex-column-reverse {
    -webkit-box-orient: vertical !important;
    -webkit-box-direction: reverse !important;
    -ms-flex-direction: column-reverse !important;
    flex-direction: column-reverse !important; }
  .wrapper-bootstrap .flex-wrap {
    -ms-flex-wrap: wrap !important;
    flex-wrap: wrap !important; }
  .wrapper-bootstrap .flex-nowrap {
    -ms-flex-wrap: nowrap !important;
    flex-wrap: nowrap !important; }
  .wrapper-bootstrap .flex-wrap-reverse {
    -ms-flex-wrap: wrap-reverse !important;
    flex-wrap: wrap-reverse !important; }
  .wrapper-bootstrap .flex-fill {
    -webkit-box-flex: 1 !important;
    -ms-flex: 1 1 auto !important;
    flex: 1 1 auto !important; }
  .wrapper-bootstrap .flex-grow-0 {
    -webkit-box-flex: 0 !important;
    -ms-flex-positive: 0 !important;
    flex-grow: 0 !important; }
  .wrapper-bootstrap .flex-grow-1 {
    -webkit-box-flex: 1 !important;
    -ms-flex-positive: 1 !important;
    flex-grow: 1 !important; }
  .wrapper-bootstrap .flex-shrink-0 {
    -ms-flex-negative: 0 !important;
    flex-shrink: 0 !important; }
  .wrapper-bootstrap .flex-shrink-1 {
    -ms-flex-negative: 1 !important;
    flex-shrink: 1 !important; }
  .wrapper-bootstrap .justify-content-start {
    -webkit-box-pack: start !important;
    -ms-flex-pack: start !important;
    justify-content: flex-start !important; }
  .wrapper-bootstrap .justify-content-end {
    -webkit-box-pack: end !important;
    -ms-flex-pack: end !important;
    justify-content: flex-end !important; }
  .wrapper-bootstrap .justify-content-center {
    -webkit-box-pack: center !important;
    -ms-flex-pack: center !important;
    justify-content: center !important; }
  .wrapper-bootstrap .justify-content-between {
    -webkit-box-pack: justify !important;
    -ms-flex-pack: justify !important;
    justify-content: space-between !important; }
  .wrapper-bootstrap .justify-content-around {
    -ms-flex-pack: distribute !important;
    justify-content: space-around !important; }
  .wrapper-bootstrap .align-items-start {
    -webkit-box-align: start !important;
    -ms-flex-align: start !important;
    align-items: flex-start !important; }
  .wrapper-bootstrap .align-items-end {
    -webkit-box-align: end !important;
    -ms-flex-align: end !important;
    align-items: flex-end !important; }
  .wrapper-bootstrap .align-items-center {
    -webkit-box-align: center !important;
    -ms-flex-align: center !important;
    align-items: center !important; }
  .wrapper-bootstrap .align-items-baseline {
    -webkit-box-align: baseline !important;
    -ms-flex-align: baseline !important;
    align-items: baseline !important; }
  .wrapper-bootstrap .align-items-stretch {
    -webkit-box-align: stretch !important;
    -ms-flex-align: stretch !important;
    align-items: stretch !important; }
  .wrapper-bootstrap .align-content-start {
    -ms-flex-line-pack: start !important;
    align-content: flex-start !important; }
  .wrapper-bootstrap .align-content-end {
    -ms-flex-line-pack: end !important;
    align-content: flex-end !important; }
  .wrapper-bootstrap .align-content-center {
    -ms-flex-line-pack: center !important;
    align-content: center !important; }
  .wrapper-bootstrap .align-content-between {
    -ms-flex-line-pack: justify !important;
    align-content: space-between !important; }
  .wrapper-bootstrap .align-content-around {
    -ms-flex-line-pack: distribute !important;
    align-content: space-around !important; }
  .wrapper-bootstrap .align-content-stretch {
    -ms-flex-line-pack: stretch !important;
    align-content: stretch !important; }
  .wrapper-bootstrap .align-self-auto {
    -ms-flex-item-align: auto !important;
    align-self: auto !important; }
  .wrapper-bootstrap .align-self-start {
    -ms-flex-item-align: start !important;
    align-self: flex-start !important; }
  .wrapper-bootstrap .align-self-end {
    -ms-flex-item-align: end !important;
    align-self: flex-end !important; }
  .wrapper-bootstrap .align-self-center {
    -ms-flex-item-align: center !important;
    align-self: center !important; }
  .wrapper-bootstrap .align-self-baseline {
    -ms-flex-item-align: baseline !important;
    align-self: baseline !important; }
  .wrapper-bootstrap .align-self-stretch {
    -ms-flex-item-align: stretch !important;
    align-self: stretch !important; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .flex-sm-row {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: row !important;
      flex-direction: row !important; }
    .wrapper-bootstrap .flex-sm-column {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: column !important;
      flex-direction: column !important; }
    .wrapper-bootstrap .flex-sm-row-reverse {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: row-reverse !important;
      flex-direction: row-reverse !important; }
    .wrapper-bootstrap .flex-sm-column-reverse {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: column-reverse !important;
      flex-direction: column-reverse !important; }
    .wrapper-bootstrap .flex-sm-wrap {
      -ms-flex-wrap: wrap !important;
      flex-wrap: wrap !important; }
    .wrapper-bootstrap .flex-sm-nowrap {
      -ms-flex-wrap: nowrap !important;
      flex-wrap: nowrap !important; }
    .wrapper-bootstrap .flex-sm-wrap-reverse {
      -ms-flex-wrap: wrap-reverse !important;
      flex-wrap: wrap-reverse !important; }
    .wrapper-bootstrap .flex-sm-fill {
      -webkit-box-flex: 1 !important;
      -ms-flex: 1 1 auto !important;
      flex: 1 1 auto !important; }
    .wrapper-bootstrap .flex-sm-grow-0 {
      -webkit-box-flex: 0 !important;
      -ms-flex-positive: 0 !important;
      flex-grow: 0 !important; }
    .wrapper-bootstrap .flex-sm-grow-1 {
      -webkit-box-flex: 1 !important;
      -ms-flex-positive: 1 !important;
      flex-grow: 1 !important; }
    .wrapper-bootstrap .flex-sm-shrink-0 {
      -ms-flex-negative: 0 !important;
      flex-shrink: 0 !important; }
    .wrapper-bootstrap .flex-sm-shrink-1 {
      -ms-flex-negative: 1 !important;
      flex-shrink: 1 !important; }
    .wrapper-bootstrap .justify-content-sm-start {
      -webkit-box-pack: start !important;
      -ms-flex-pack: start !important;
      justify-content: flex-start !important; }
    .wrapper-bootstrap .justify-content-sm-end {
      -webkit-box-pack: end !important;
      -ms-flex-pack: end !important;
      justify-content: flex-end !important; }
    .wrapper-bootstrap .justify-content-sm-center {
      -webkit-box-pack: center !important;
      -ms-flex-pack: center !important;
      justify-content: center !important; }
    .wrapper-bootstrap .justify-content-sm-between {
      -webkit-box-pack: justify !important;
      -ms-flex-pack: justify !important;
      justify-content: space-between !important; }
    .wrapper-bootstrap .justify-content-sm-around {
      -ms-flex-pack: distribute !important;
      justify-content: space-around !important; }
    .wrapper-bootstrap .align-items-sm-start {
      -webkit-box-align: start !important;
      -ms-flex-align: start !important;
      align-items: flex-start !important; }
    .wrapper-bootstrap .align-items-sm-end {
      -webkit-box-align: end !important;
      -ms-flex-align: end !important;
      align-items: flex-end !important; }
    .wrapper-bootstrap .align-items-sm-center {
      -webkit-box-align: center !important;
      -ms-flex-align: center !important;
      align-items: center !important; }
    .wrapper-bootstrap .align-items-sm-baseline {
      -webkit-box-align: baseline !important;
      -ms-flex-align: baseline !important;
      align-items: baseline !important; }
    .wrapper-bootstrap .align-items-sm-stretch {
      -webkit-box-align: stretch !important;
      -ms-flex-align: stretch !important;
      align-items: stretch !important; }
    .wrapper-bootstrap .align-content-sm-start {
      -ms-flex-line-pack: start !important;
      align-content: flex-start !important; }
    .wrapper-bootstrap .align-content-sm-end {
      -ms-flex-line-pack: end !important;
      align-content: flex-end !important; }
    .wrapper-bootstrap .align-content-sm-center {
      -ms-flex-line-pack: center !important;
      align-content: center !important; }
    .wrapper-bootstrap .align-content-sm-between {
      -ms-flex-line-pack: justify !important;
      align-content: space-between !important; }
    .wrapper-bootstrap .align-content-sm-around {
      -ms-flex-line-pack: distribute !important;
      align-content: space-around !important; }
    .wrapper-bootstrap .align-content-sm-stretch {
      -ms-flex-line-pack: stretch !important;
      align-content: stretch !important; }
    .wrapper-bootstrap .align-self-sm-auto {
      -ms-flex-item-align: auto !important;
      align-self: auto !important; }
    .wrapper-bootstrap .align-self-sm-start {
      -ms-flex-item-align: start !important;
      align-self: flex-start !important; }
    .wrapper-bootstrap .align-self-sm-end {
      -ms-flex-item-align: end !important;
      align-self: flex-end !important; }
    .wrapper-bootstrap .align-self-sm-center {
      -ms-flex-item-align: center !important;
      align-self: center !important; }
    .wrapper-bootstrap .align-self-sm-baseline {
      -ms-flex-item-align: baseline !important;
      align-self: baseline !important; }
    .wrapper-bootstrap .align-self-sm-stretch {
      -ms-flex-item-align: stretch !important;
      align-self: stretch !important; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .flex-md-row {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: row !important;
      flex-direction: row !important; }
    .wrapper-bootstrap .flex-md-column {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: column !important;
      flex-direction: column !important; }
    .wrapper-bootstrap .flex-md-row-reverse {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: row-reverse !important;
      flex-direction: row-reverse !important; }
    .wrapper-bootstrap .flex-md-column-reverse {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: column-reverse !important;
      flex-direction: column-reverse !important; }
    .wrapper-bootstrap .flex-md-wrap {
      -ms-flex-wrap: wrap !important;
      flex-wrap: wrap !important; }
    .wrapper-bootstrap .flex-md-nowrap {
      -ms-flex-wrap: nowrap !important;
      flex-wrap: nowrap !important; }
    .wrapper-bootstrap .flex-md-wrap-reverse {
      -ms-flex-wrap: wrap-reverse !important;
      flex-wrap: wrap-reverse !important; }
    .wrapper-bootstrap .flex-md-fill {
      -webkit-box-flex: 1 !important;
      -ms-flex: 1 1 auto !important;
      flex: 1 1 auto !important; }
    .wrapper-bootstrap .flex-md-grow-0 {
      -webkit-box-flex: 0 !important;
      -ms-flex-positive: 0 !important;
      flex-grow: 0 !important; }
    .wrapper-bootstrap .flex-md-grow-1 {
      -webkit-box-flex: 1 !important;
      -ms-flex-positive: 1 !important;
      flex-grow: 1 !important; }
    .wrapper-bootstrap .flex-md-shrink-0 {
      -ms-flex-negative: 0 !important;
      flex-shrink: 0 !important; }
    .wrapper-bootstrap .flex-md-shrink-1 {
      -ms-flex-negative: 1 !important;
      flex-shrink: 1 !important; }
    .wrapper-bootstrap .justify-content-md-start {
      -webkit-box-pack: start !important;
      -ms-flex-pack: start !important;
      justify-content: flex-start !important; }
    .wrapper-bootstrap .justify-content-md-end {
      -webkit-box-pack: end !important;
      -ms-flex-pack: end !important;
      justify-content: flex-end !important; }
    .wrapper-bootstrap .justify-content-md-center {
      -webkit-box-pack: center !important;
      -ms-flex-pack: center !important;
      justify-content: center !important; }
    .wrapper-bootstrap .justify-content-md-between {
      -webkit-box-pack: justify !important;
      -ms-flex-pack: justify !important;
      justify-content: space-between !important; }
    .wrapper-bootstrap .justify-content-md-around {
      -ms-flex-pack: distribute !important;
      justify-content: space-around !important; }
    .wrapper-bootstrap .align-items-md-start {
      -webkit-box-align: start !important;
      -ms-flex-align: start !important;
      align-items: flex-start !important; }
    .wrapper-bootstrap .align-items-md-end {
      -webkit-box-align: end !important;
      -ms-flex-align: end !important;
      align-items: flex-end !important; }
    .wrapper-bootstrap .align-items-md-center {
      -webkit-box-align: center !important;
      -ms-flex-align: center !important;
      align-items: center !important; }
    .wrapper-bootstrap .align-items-md-baseline {
      -webkit-box-align: baseline !important;
      -ms-flex-align: baseline !important;
      align-items: baseline !important; }
    .wrapper-bootstrap .align-items-md-stretch {
      -webkit-box-align: stretch !important;
      -ms-flex-align: stretch !important;
      align-items: stretch !important; }
    .wrapper-bootstrap .align-content-md-start {
      -ms-flex-line-pack: start !important;
      align-content: flex-start !important; }
    .wrapper-bootstrap .align-content-md-end {
      -ms-flex-line-pack: end !important;
      align-content: flex-end !important; }
    .wrapper-bootstrap .align-content-md-center {
      -ms-flex-line-pack: center !important;
      align-content: center !important; }
    .wrapper-bootstrap .align-content-md-between {
      -ms-flex-line-pack: justify !important;
      align-content: space-between !important; }
    .wrapper-bootstrap .align-content-md-around {
      -ms-flex-line-pack: distribute !important;
      align-content: space-around !important; }
    .wrapper-bootstrap .align-content-md-stretch {
      -ms-flex-line-pack: stretch !important;
      align-content: stretch !important; }
    .wrapper-bootstrap .align-self-md-auto {
      -ms-flex-item-align: auto !important;
      align-self: auto !important; }
    .wrapper-bootstrap .align-self-md-start {
      -ms-flex-item-align: start !important;
      align-self: flex-start !important; }
    .wrapper-bootstrap .align-self-md-end {
      -ms-flex-item-align: end !important;
      align-self: flex-end !important; }
    .wrapper-bootstrap .align-self-md-center {
      -ms-flex-item-align: center !important;
      align-self: center !important; }
    .wrapper-bootstrap .align-self-md-baseline {
      -ms-flex-item-align: baseline !important;
      align-self: baseline !important; }
    .wrapper-bootstrap .align-self-md-stretch {
      -ms-flex-item-align: stretch !important;
      align-self: stretch !important; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .flex-lg-row {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: row !important;
      flex-direction: row !important; }
    .wrapper-bootstrap .flex-lg-column {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: column !important;
      flex-direction: column !important; }
    .wrapper-bootstrap .flex-lg-row-reverse {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: row-reverse !important;
      flex-direction: row-reverse !important; }
    .wrapper-bootstrap .flex-lg-column-reverse {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: column-reverse !important;
      flex-direction: column-reverse !important; }
    .wrapper-bootstrap .flex-lg-wrap {
      -ms-flex-wrap: wrap !important;
      flex-wrap: wrap !important; }
    .wrapper-bootstrap .flex-lg-nowrap {
      -ms-flex-wrap: nowrap !important;
      flex-wrap: nowrap !important; }
    .wrapper-bootstrap .flex-lg-wrap-reverse {
      -ms-flex-wrap: wrap-reverse !important;
      flex-wrap: wrap-reverse !important; }
    .wrapper-bootstrap .flex-lg-fill {
      -webkit-box-flex: 1 !important;
      -ms-flex: 1 1 auto !important;
      flex: 1 1 auto !important; }
    .wrapper-bootstrap .flex-lg-grow-0 {
      -webkit-box-flex: 0 !important;
      -ms-flex-positive: 0 !important;
      flex-grow: 0 !important; }
    .wrapper-bootstrap .flex-lg-grow-1 {
      -webkit-box-flex: 1 !important;
      -ms-flex-positive: 1 !important;
      flex-grow: 1 !important; }
    .wrapper-bootstrap .flex-lg-shrink-0 {
      -ms-flex-negative: 0 !important;
      flex-shrink: 0 !important; }
    .wrapper-bootstrap .flex-lg-shrink-1 {
      -ms-flex-negative: 1 !important;
      flex-shrink: 1 !important; }
    .wrapper-bootstrap .justify-content-lg-start {
      -webkit-box-pack: start !important;
      -ms-flex-pack: start !important;
      justify-content: flex-start !important; }
    .wrapper-bootstrap .justify-content-lg-end {
      -webkit-box-pack: end !important;
      -ms-flex-pack: end !important;
      justify-content: flex-end !important; }
    .wrapper-bootstrap .justify-content-lg-center {
      -webkit-box-pack: center !important;
      -ms-flex-pack: center !important;
      justify-content: center !important; }
    .wrapper-bootstrap .justify-content-lg-between {
      -webkit-box-pack: justify !important;
      -ms-flex-pack: justify !important;
      justify-content: space-between !important; }
    .wrapper-bootstrap .justify-content-lg-around {
      -ms-flex-pack: distribute !important;
      justify-content: space-around !important; }
    .wrapper-bootstrap .align-items-lg-start {
      -webkit-box-align: start !important;
      -ms-flex-align: start !important;
      align-items: flex-start !important; }
    .wrapper-bootstrap .align-items-lg-end {
      -webkit-box-align: end !important;
      -ms-flex-align: end !important;
      align-items: flex-end !important; }
    .wrapper-bootstrap .align-items-lg-center {
      -webkit-box-align: center !important;
      -ms-flex-align: center !important;
      align-items: center !important; }
    .wrapper-bootstrap .align-items-lg-baseline {
      -webkit-box-align: baseline !important;
      -ms-flex-align: baseline !important;
      align-items: baseline !important; }
    .wrapper-bootstrap .align-items-lg-stretch {
      -webkit-box-align: stretch !important;
      -ms-flex-align: stretch !important;
      align-items: stretch !important; }
    .wrapper-bootstrap .align-content-lg-start {
      -ms-flex-line-pack: start !important;
      align-content: flex-start !important; }
    .wrapper-bootstrap .align-content-lg-end {
      -ms-flex-line-pack: end !important;
      align-content: flex-end !important; }
    .wrapper-bootstrap .align-content-lg-center {
      -ms-flex-line-pack: center !important;
      align-content: center !important; }
    .wrapper-bootstrap .align-content-lg-between {
      -ms-flex-line-pack: justify !important;
      align-content: space-between !important; }
    .wrapper-bootstrap .align-content-lg-around {
      -ms-flex-line-pack: distribute !important;
      align-content: space-around !important; }
    .wrapper-bootstrap .align-content-lg-stretch {
      -ms-flex-line-pack: stretch !important;
      align-content: stretch !important; }
    .wrapper-bootstrap .align-self-lg-auto {
      -ms-flex-item-align: auto !important;
      align-self: auto !important; }
    .wrapper-bootstrap .align-self-lg-start {
      -ms-flex-item-align: start !important;
      align-self: flex-start !important; }
    .wrapper-bootstrap .align-self-lg-end {
      -ms-flex-item-align: end !important;
      align-self: flex-end !important; }
    .wrapper-bootstrap .align-self-lg-center {
      -ms-flex-item-align: center !important;
      align-self: center !important; }
    .wrapper-bootstrap .align-self-lg-baseline {
      -ms-flex-item-align: baseline !important;
      align-self: baseline !important; }
    .wrapper-bootstrap .align-self-lg-stretch {
      -ms-flex-item-align: stretch !important;
      align-self: stretch !important; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .flex-xl-row {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: row !important;
      flex-direction: row !important; }
    .wrapper-bootstrap .flex-xl-column {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: normal !important;
      -ms-flex-direction: column !important;
      flex-direction: column !important; }
    .wrapper-bootstrap .flex-xl-row-reverse {
      -webkit-box-orient: horizontal !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: row-reverse !important;
      flex-direction: row-reverse !important; }
    .wrapper-bootstrap .flex-xl-column-reverse {
      -webkit-box-orient: vertical !important;
      -webkit-box-direction: reverse !important;
      -ms-flex-direction: column-reverse !important;
      flex-direction: column-reverse !important; }
    .wrapper-bootstrap .flex-xl-wrap {
      -ms-flex-wrap: wrap !important;
      flex-wrap: wrap !important; }
    .wrapper-bootstrap .flex-xl-nowrap {
      -ms-flex-wrap: nowrap !important;
      flex-wrap: nowrap !important; }
    .wrapper-bootstrap .flex-xl-wrap-reverse {
      -ms-flex-wrap: wrap-reverse !important;
      flex-wrap: wrap-reverse !important; }
    .wrapper-bootstrap .flex-xl-fill {
      -webkit-box-flex: 1 !important;
      -ms-flex: 1 1 auto !important;
      flex: 1 1 auto !important; }
    .wrapper-bootstrap .flex-xl-grow-0 {
      -webkit-box-flex: 0 !important;
      -ms-flex-positive: 0 !important;
      flex-grow: 0 !important; }
    .wrapper-bootstrap .flex-xl-grow-1 {
      -webkit-box-flex: 1 !important;
      -ms-flex-positive: 1 !important;
      flex-grow: 1 !important; }
    .wrapper-bootstrap .flex-xl-shrink-0 {
      -ms-flex-negative: 0 !important;
      flex-shrink: 0 !important; }
    .wrapper-bootstrap .flex-xl-shrink-1 {
      -ms-flex-negative: 1 !important;
      flex-shrink: 1 !important; }
    .wrapper-bootstrap .justify-content-xl-start {
      -webkit-box-pack: start !important;
      -ms-flex-pack: start !important;
      justify-content: flex-start !important; }
    .wrapper-bootstrap .justify-content-xl-end {
      -webkit-box-pack: end !important;
      -ms-flex-pack: end !important;
      justify-content: flex-end !important; }
    .wrapper-bootstrap .justify-content-xl-center {
      -webkit-box-pack: center !important;
      -ms-flex-pack: center !important;
      justify-content: center !important; }
    .wrapper-bootstrap .justify-content-xl-between {
      -webkit-box-pack: justify !important;
      -ms-flex-pack: justify !important;
      justify-content: space-between !important; }
    .wrapper-bootstrap .justify-content-xl-around {
      -ms-flex-pack: distribute !important;
      justify-content: space-around !important; }
    .wrapper-bootstrap .align-items-xl-start {
      -webkit-box-align: start !important;
      -ms-flex-align: start !important;
      align-items: flex-start !important; }
    .wrapper-bootstrap .align-items-xl-end {
      -webkit-box-align: end !important;
      -ms-flex-align: end !important;
      align-items: flex-end !important; }
    .wrapper-bootstrap .align-items-xl-center {
      -webkit-box-align: center !important;
      -ms-flex-align: center !important;
      align-items: center !important; }
    .wrapper-bootstrap .align-items-xl-baseline {
      -webkit-box-align: baseline !important;
      -ms-flex-align: baseline !important;
      align-items: baseline !important; }
    .wrapper-bootstrap .align-items-xl-stretch {
      -webkit-box-align: stretch !important;
      -ms-flex-align: stretch !important;
      align-items: stretch !important; }
    .wrapper-bootstrap .align-content-xl-start {
      -ms-flex-line-pack: start !important;
      align-content: flex-start !important; }
    .wrapper-bootstrap .align-content-xl-end {
      -ms-flex-line-pack: end !important;
      align-content: flex-end !important; }
    .wrapper-bootstrap .align-content-xl-center {
      -ms-flex-line-pack: center !important;
      align-content: center !important; }
    .wrapper-bootstrap .align-content-xl-between {
      -ms-flex-line-pack: justify !important;
      align-content: space-between !important; }
    .wrapper-bootstrap .align-content-xl-around {
      -ms-flex-line-pack: distribute !important;
      align-content: space-around !important; }
    .wrapper-bootstrap .align-content-xl-stretch {
      -ms-flex-line-pack: stretch !important;
      align-content: stretch !important; }
    .wrapper-bootstrap .align-self-xl-auto {
      -ms-flex-item-align: auto !important;
      align-self: auto !important; }
    .wrapper-bootstrap .align-self-xl-start {
      -ms-flex-item-align: start !important;
      align-self: flex-start !important; }
    .wrapper-bootstrap .align-self-xl-end {
      -ms-flex-item-align: end !important;
      align-self: flex-end !important; }
    .wrapper-bootstrap .align-self-xl-center {
      -ms-flex-item-align: center !important;
      align-self: center !important; }
    .wrapper-bootstrap .align-self-xl-baseline {
      -ms-flex-item-align: baseline !important;
      align-self: baseline !important; }
    .wrapper-bootstrap .align-self-xl-stretch {
      -ms-flex-item-align: stretch !important;
      align-self: stretch !important; } }
  .wrapper-bootstrap .float-left {
    float: left !important; }
  .wrapper-bootstrap .float-right {
    float: right !important; }
  .wrapper-bootstrap .float-none {
    float: none !important; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .float-sm-left {
      float: left !important; }
    .wrapper-bootstrap .float-sm-right {
      float: right !important; }
    .wrapper-bootstrap .float-sm-none {
      float: none !important; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .float-md-left {
      float: left !important; }
    .wrapper-bootstrap .float-md-right {
      float: right !important; }
    .wrapper-bootstrap .float-md-none {
      float: none !important; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .float-lg-left {
      float: left !important; }
    .wrapper-bootstrap .float-lg-right {
      float: right !important; }
    .wrapper-bootstrap .float-lg-none {
      float: none !important; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .float-xl-left {
      float: left !important; }
    .wrapper-bootstrap .float-xl-right {
      float: right !important; }
    .wrapper-bootstrap .float-xl-none {
      float: none !important; } }
  .wrapper-bootstrap .overflow-auto {
    overflow: auto !important; }
  .wrapper-bootstrap .overflow-hidden {
    overflow: hidden !important; }
  .wrapper-bootstrap .position-static {
    position: static !important; }
  .wrapper-bootstrap .position-relative {
    position: relative !important; }
  .wrapper-bootstrap .position-absolute {
    position: absolute !important; }
  .wrapper-bootstrap .position-fixed {
    position: fixed !important; }
  .wrapper-bootstrap .position-sticky {
    position: -webkit-sticky !important;
    position: sticky !important; }
  .wrapper-bootstrap .fixed-top {
    position: fixed;
    top: 0;
    right: 0;
    left: 0;
    z-index: 1030; }
  .wrapper-bootstrap .fixed-bottom {
    position: fixed;
    right: 0;
    bottom: 0;
    left: 0;
    z-index: 1030; }
  @supports ((position: -webkit-sticky) or (position: sticky)) {
    .wrapper-bootstrap .sticky-top {
      position: -webkit-sticky;
      position: sticky;
      top: 0;
      z-index: 1020; } }
  .wrapper-bootstrap .sr-only {
    position: absolute;
    width: 1px;
    height: 1px;
    padding: 0;
    overflow: hidden;
    clip: rect(0, 0, 0, 0);
    white-space: nowrap;
    border: 0; }
  .wrapper-bootstrap .sr-only-focusable:active, .wrapper-bootstrap .sr-only-focusable:focus {
    position: static;
    width: auto;
    height: auto;
    overflow: visible;
    clip: auto;
    white-space: normal; }
  .wrapper-bootstrap .shadow-sm {
    -webkit-box-shadow: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075) !important;
    box-shadow: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075) !important; }
  .wrapper-bootstrap .shadow {
    -webkit-box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15) !important;
    box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15) !important; }
  .wrapper-bootstrap .shadow-lg {
    -webkit-box-shadow: 0 1rem 3rem rgba(0, 0, 0, 0.175) !important;
    box-shadow: 0 1rem 3rem rgba(0, 0, 0, 0.175) !important; }
  .wrapper-bootstrap .shadow-none {
    -webkit-box-shadow: none !important;
    box-shadow: none !important; }
  .wrapper-bootstrap .w-25 {
    width: 25% !important; }
  .wrapper-bootstrap .w-50 {
    width: 50% !important; }
  .wrapper-bootstrap .w-75 {
    width: 75% !important; }
  .wrapper-bootstrap .w-100 {
    width: 100% !important; }
  .wrapper-bootstrap .w-auto {
    width: auto !important; }
  .wrapper-bootstrap .h-25 {
    height: 25% !important; }
  .wrapper-bootstrap .h-50 {
    height: 50% !important; }
  .wrapper-bootstrap .h-75 {
    height: 75% !important; }
  .wrapper-bootstrap .h-100 {
    height: 100% !important; }
  .wrapper-bootstrap .h-auto {
    height: auto !important; }
  .wrapper-bootstrap .mw-100 {
    max-width: 100% !important; }
  .wrapper-bootstrap .mh-100 {
    max-height: 100% !important; }
  .wrapper-bootstrap .min-vw-100 {
    min-width: 100vw !important; }
  .wrapper-bootstrap .min-vh-100 {
    min-height: 100vh !important; }
  .wrapper-bootstrap .vw-100 {
    width: 100vw !important; }
  .wrapper-bootstrap .vh-100 {
    height: 100vh !important; }
  .wrapper-bootstrap .stretched-link::after {
    position: absolute;
    top: 0;
    right: 0;
    bottom: 0;
    left: 0;
    z-index: 1;
    pointer-events: auto;
    content: "";
    background-color: rgba(0, 0, 0, 0); }
  .wrapper-bootstrap .m-0 {
    margin: 0 !important; }
  .wrapper-bootstrap .mt-0,
  .wrapper-bootstrap .my-0 {
    margin-top: 0 !important; }
  .wrapper-bootstrap .mr-0,
  .wrapper-bootstrap .mx-0 {
    margin-right: 0 !important; }
  .wrapper-bootstrap .mb-0,
  .wrapper-bootstrap .my-0 {
    margin-bottom: 0 !important; }
  .wrapper-bootstrap .ml-0,
  .wrapper-bootstrap .mx-0 {
    margin-left: 0 !important; }
  .wrapper-bootstrap .m-1 {
    margin: 0.25rem !important; }
  .wrapper-bootstrap .mt-1,
  .wrapper-bootstrap .my-1 {
    margin-top: 0.25rem !important; }
  .wrapper-bootstrap .mr-1,
  .wrapper-bootstrap .mx-1 {
    margin-right: 0.25rem !important; }
  .wrapper-bootstrap .mb-1,
  .wrapper-bootstrap .my-1 {
    margin-bottom: 0.25rem !important; }
  .wrapper-bootstrap .ml-1,
  .wrapper-bootstrap .mx-1 {
    margin-left: 0.25rem !important; }
  .wrapper-bootstrap .m-2 {
    margin: 0.5rem !important; }
  .wrapper-bootstrap .mt-2,
  .wrapper-bootstrap .my-2 {
    margin-top: 0.5rem !important; }
  .wrapper-bootstrap .mr-2,
  .wrapper-bootstrap .mx-2 {
    margin-right: 0.5rem !important; }
  .wrapper-bootstrap .mb-2,
  .wrapper-bootstrap .my-2 {
    margin-bottom: 0.5rem !important; }
  .wrapper-bootstrap .ml-2,
  .wrapper-bootstrap .mx-2 {
    margin-left: 0.5rem !important; }
  .wrapper-bootstrap .m-3 {
    margin: 1rem !important; }
  .wrapper-bootstrap .mt-3,
  .wrapper-bootstrap .my-3 {
    margin-top: 1rem !important; }
  .wrapper-bootstrap .mr-3,
  .wrapper-bootstrap .mx-3 {
    margin-right: 1rem !important; }
  .wrapper-bootstrap .mb-3,
  .wrapper-bootstrap .my-3 {
    margin-bottom: 1rem !important; }
  .wrapper-bootstrap .ml-3,
  .wrapper-bootstrap .mx-3 {
    margin-left: 1rem !important; }
  .wrapper-bootstrap .m-4 {
    margin: 1.5rem !important; }
  .wrapper-bootstrap .mt-4,
  .wrapper-bootstrap .my-4 {
    margin-top: 1.5rem !important; }
  .wrapper-bootstrap .mr-4,
  .wrapper-bootstrap .mx-4 {
    margin-right: 1.5rem !important; }
  .wrapper-bootstrap .mb-4,
  .wrapper-bootstrap .my-4 {
    margin-bottom: 1.5rem !important; }
  .wrapper-bootstrap .ml-4,
  .wrapper-bootstrap .mx-4 {
    margin-left: 1.5rem !important; }
  .wrapper-bootstrap .m-5 {
    margin: 3rem !important; }
  .wrapper-bootstrap .mt-5,
  .wrapper-bootstrap .my-5 {
    margin-top: 3rem !important; }
  .wrapper-bootstrap .mr-5,
  .wrapper-bootstrap .mx-5 {
    margin-right: 3rem !important; }
  .wrapper-bootstrap .mb-5,
  .wrapper-bootstrap .my-5 {
    margin-bottom: 3rem !important; }
  .wrapper-bootstrap .ml-5,
  .wrapper-bootstrap .mx-5 {
    margin-left: 3rem !important; }
  .wrapper-bootstrap .p-0 {
    padding: 0 !important; }
  .wrapper-bootstrap .pt-0,
  .wrapper-bootstrap .py-0 {
    padding-top: 0 !important; }
  .wrapper-bootstrap .pr-0,
  .wrapper-bootstrap .px-0 {
    padding-right: 0 !important; }
  .wrapper-bootstrap .pb-0,
  .wrapper-bootstrap .py-0 {
    padding-bottom: 0 !important; }
  .wrapper-bootstrap .pl-0,
  .wrapper-bootstrap .px-0 {
    padding-left: 0 !important; }
  .wrapper-bootstrap .p-1 {
    padding: 0.25rem !important; }
  .wrapper-bootstrap .pt-1,
  .wrapper-bootstrap .py-1 {
    padding-top: 0.25rem !important; }
  .wrapper-bootstrap .pr-1,
  .wrapper-bootstrap .px-1 {
    padding-right: 0.25rem !important; }
  .wrapper-bootstrap .pb-1,
  .wrapper-bootstrap .py-1 {
    padding-bottom: 0.25rem !important; }
  .wrapper-bootstrap .pl-1,
  .wrapper-bootstrap .px-1 {
    padding-left: 0.25rem !important; }
  .wrapper-bootstrap .p-2 {
    padding: 0.5rem !important; }
  .wrapper-bootstrap .pt-2,
  .wrapper-bootstrap .py-2 {
    padding-top: 0.5rem !important; }
  .wrapper-bootstrap .pr-2,
  .wrapper-bootstrap .px-2 {
    padding-right: 0.5rem !important; }
  .wrapper-bootstrap .pb-2,
  .wrapper-bootstrap .py-2 {
    padding-bottom: 0.5rem !important; }
  .wrapper-bootstrap .pl-2,
  .wrapper-bootstrap .px-2 {
    padding-left: 0.5rem !important; }
  .wrapper-bootstrap .p-3 {
    padding: 1rem !important; }
  .wrapper-bootstrap .pt-3,
  .wrapper-bootstrap .py-3 {
    padding-top: 1rem !important; }
  .wrapper-bootstrap .pr-3,
  .wrapper-bootstrap .px-3 {
    padding-right: 1rem !important; }
  .wrapper-bootstrap .pb-3,
  .wrapper-bootstrap .py-3 {
    padding-bottom: 1rem !important; }
  .wrapper-bootstrap .pl-3,
  .wrapper-bootstrap .px-3 {
    padding-left: 1rem !important; }
  .wrapper-bootstrap .p-4 {
    padding: 1.5rem !important; }
  .wrapper-bootstrap .pt-4,
  .wrapper-bootstrap .py-4 {
    padding-top: 1.5rem !important; }
  .wrapper-bootstrap .pr-4,
  .wrapper-bootstrap .px-4 {
    padding-right: 1.5rem !important; }
  .wrapper-bootstrap .pb-4,
  .wrapper-bootstrap .py-4 {
    padding-bottom: 1.5rem !important; }
  .wrapper-bootstrap .pl-4,
  .wrapper-bootstrap .px-4 {
    padding-left: 1.5rem !important; }
  .wrapper-bootstrap .p-5 {
    padding: 3rem !important; }
  .wrapper-bootstrap .pt-5,
  .wrapper-bootstrap .py-5 {
    padding-top: 3rem !important; }
  .wrapper-bootstrap .pr-5,
  .wrapper-bootstrap .px-5 {
    padding-right: 3rem !important; }
  .wrapper-bootstrap .pb-5,
  .wrapper-bootstrap .py-5 {
    padding-bottom: 3rem !important; }
  .wrapper-bootstrap .pl-5,
  .wrapper-bootstrap .px-5 {
    padding-left: 3rem !important; }
  .wrapper-bootstrap .m-n1 {
    margin: -0.25rem !important; }
  .wrapper-bootstrap .mt-n1,
  .wrapper-bootstrap .my-n1 {
    margin-top: -0.25rem !important; }
  .wrapper-bootstrap .mr-n1,
  .wrapper-bootstrap .mx-n1 {
    margin-right: -0.25rem !important; }
  .wrapper-bootstrap .mb-n1,
  .wrapper-bootstrap .my-n1 {
    margin-bottom: -0.25rem !important; }
  .wrapper-bootstrap .ml-n1,
  .wrapper-bootstrap .mx-n1 {
    margin-left: -0.25rem !important; }
  .wrapper-bootstrap .m-n2 {
    margin: -0.5rem !important; }
  .wrapper-bootstrap .mt-n2,
  .wrapper-bootstrap .my-n2 {
    margin-top: -0.5rem !important; }
  .wrapper-bootstrap .mr-n2,
  .wrapper-bootstrap .mx-n2 {
    margin-right: -0.5rem !important; }
  .wrapper-bootstrap .mb-n2,
  .wrapper-bootstrap .my-n2 {
    margin-bottom: -0.5rem !important; }
  .wrapper-bootstrap .ml-n2,
  .wrapper-bootstrap .mx-n2 {
    margin-left: -0.5rem !important; }
  .wrapper-bootstrap .m-n3 {
    margin: -1rem !important; }
  .wrapper-bootstrap .mt-n3,
  .wrapper-bootstrap .my-n3 {
    margin-top: -1rem !important; }
  .wrapper-bootstrap .mr-n3,
  .wrapper-bootstrap .mx-n3 {
    margin-right: -1rem !important; }
  .wrapper-bootstrap .mb-n3,
  .wrapper-bootstrap .my-n3 {
    margin-bottom: -1rem !important; }
  .wrapper-bootstrap .ml-n3,
  .wrapper-bootstrap .mx-n3 {
    margin-left: -1rem !important; }
  .wrapper-bootstrap .m-n4 {
    margin: -1.5rem !important; }
  .wrapper-bootstrap .mt-n4,
  .wrapper-bootstrap .my-n4 {
    margin-top: -1.5rem !important; }
  .wrapper-bootstrap .mr-n4,
  .wrapper-bootstrap .mx-n4 {
    margin-right: -1.5rem !important; }
  .wrapper-bootstrap .mb-n4,
  .wrapper-bootstrap .my-n4 {
    margin-bottom: -1.5rem !important; }
  .wrapper-bootstrap .ml-n4,
  .wrapper-bootstrap .mx-n4 {
    margin-left: -1.5rem !important; }
  .wrapper-bootstrap .m-n5 {
    margin: -3rem !important; }
  .wrapper-bootstrap .mt-n5,
  .wrapper-bootstrap .my-n5 {
    margin-top: -3rem !important; }
  .wrapper-bootstrap .mr-n5,
  .wrapper-bootstrap .mx-n5 {
    margin-right: -3rem !important; }
  .wrapper-bootstrap .mb-n5,
  .wrapper-bootstrap .my-n5 {
    margin-bottom: -3rem !important; }
  .wrapper-bootstrap .ml-n5,
  .wrapper-bootstrap .mx-n5 {
    margin-left: -3rem !important; }
  .wrapper-bootstrap .m-auto {
    margin: auto !important; }
  .wrapper-bootstrap .mt-auto,
  .wrapper-bootstrap .my-auto {
    margin-top: auto !important; }
  .wrapper-bootstrap .mr-auto,
  .wrapper-bootstrap .mx-auto {
    margin-right: auto !important; }
  .wrapper-bootstrap .mb-auto,
  .wrapper-bootstrap .my-auto {
    margin-bottom: auto !important; }
  .wrapper-bootstrap .ml-auto,
  .wrapper-bootstrap .mx-auto {
    margin-left: auto !important; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .m-sm-0 {
      margin: 0 !important; }
    .wrapper-bootstrap .mt-sm-0,
    .wrapper-bootstrap .my-sm-0 {
      margin-top: 0 !important; }
    .wrapper-bootstrap .mr-sm-0,
    .wrapper-bootstrap .mx-sm-0 {
      margin-right: 0 !important; }
    .wrapper-bootstrap .mb-sm-0,
    .wrapper-bootstrap .my-sm-0 {
      margin-bottom: 0 !important; }
    .wrapper-bootstrap .ml-sm-0,
    .wrapper-bootstrap .mx-sm-0 {
      margin-left: 0 !important; }
    .wrapper-bootstrap .m-sm-1 {
      margin: 0.25rem !important; }
    .wrapper-bootstrap .mt-sm-1,
    .wrapper-bootstrap .my-sm-1 {
      margin-top: 0.25rem !important; }
    .wrapper-bootstrap .mr-sm-1,
    .wrapper-bootstrap .mx-sm-1 {
      margin-right: 0.25rem !important; }
    .wrapper-bootstrap .mb-sm-1,
    .wrapper-bootstrap .my-sm-1 {
      margin-bottom: 0.25rem !important; }
    .wrapper-bootstrap .ml-sm-1,
    .wrapper-bootstrap .mx-sm-1 {
      margin-left: 0.25rem !important; }
    .wrapper-bootstrap .m-sm-2 {
      margin: 0.5rem !important; }
    .wrapper-bootstrap .mt-sm-2,
    .wrapper-bootstrap .my-sm-2 {
      margin-top: 0.5rem !important; }
    .wrapper-bootstrap .mr-sm-2,
    .wrapper-bootstrap .mx-sm-2 {
      margin-right: 0.5rem !important; }
    .wrapper-bootstrap .mb-sm-2,
    .wrapper-bootstrap .my-sm-2 {
      margin-bottom: 0.5rem !important; }
    .wrapper-bootstrap .ml-sm-2,
    .wrapper-bootstrap .mx-sm-2 {
      margin-left: 0.5rem !important; }
    .wrapper-bootstrap .m-sm-3 {
      margin: 1rem !important; }
    .wrapper-bootstrap .mt-sm-3,
    .wrapper-bootstrap .my-sm-3 {
      margin-top: 1rem !important; }
    .wrapper-bootstrap .mr-sm-3,
    .wrapper-bootstrap .mx-sm-3 {
      margin-right: 1rem !important; }
    .wrapper-bootstrap .mb-sm-3,
    .wrapper-bootstrap .my-sm-3 {
      margin-bottom: 1rem !important; }
    .wrapper-bootstrap .ml-sm-3,
    .wrapper-bootstrap .mx-sm-3 {
      margin-left: 1rem !important; }
    .wrapper-bootstrap .m-sm-4 {
      margin: 1.5rem !important; }
    .wrapper-bootstrap .mt-sm-4,
    .wrapper-bootstrap .my-sm-4 {
      margin-top: 1.5rem !important; }
    .wrapper-bootstrap .mr-sm-4,
    .wrapper-bootstrap .mx-sm-4 {
      margin-right: 1.5rem !important; }
    .wrapper-bootstrap .mb-sm-4,
    .wrapper-bootstrap .my-sm-4 {
      margin-bottom: 1.5rem !important; }
    .wrapper-bootstrap .ml-sm-4,
    .wrapper-bootstrap .mx-sm-4 {
      margin-left: 1.5rem !important; }
    .wrapper-bootstrap .m-sm-5 {
      margin: 3rem !important; }
    .wrapper-bootstrap .mt-sm-5,
    .wrapper-bootstrap .my-sm-5 {
      margin-top: 3rem !important; }
    .wrapper-bootstrap .mr-sm-5,
    .wrapper-bootstrap .mx-sm-5 {
      margin-right: 3rem !important; }
    .wrapper-bootstrap .mb-sm-5,
    .wrapper-bootstrap .my-sm-5 {
      margin-bottom: 3rem !important; }
    .wrapper-bootstrap .ml-sm-5,
    .wrapper-bootstrap .mx-sm-5 {
      margin-left: 3rem !important; }
    .wrapper-bootstrap .p-sm-0 {
      padding: 0 !important; }
    .wrapper-bootstrap .pt-sm-0,
    .wrapper-bootstrap .py-sm-0 {
      padding-top: 0 !important; }
    .wrapper-bootstrap .pr-sm-0,
    .wrapper-bootstrap .px-sm-0 {
      padding-right: 0 !important; }
    .wrapper-bootstrap .pb-sm-0,
    .wrapper-bootstrap .py-sm-0 {
      padding-bottom: 0 !important; }
    .wrapper-bootstrap .pl-sm-0,
    .wrapper-bootstrap .px-sm-0 {
      padding-left: 0 !important; }
    .wrapper-bootstrap .p-sm-1 {
      padding: 0.25rem !important; }
    .wrapper-bootstrap .pt-sm-1,
    .wrapper-bootstrap .py-sm-1 {
      padding-top: 0.25rem !important; }
    .wrapper-bootstrap .pr-sm-1,
    .wrapper-bootstrap .px-sm-1 {
      padding-right: 0.25rem !important; }
    .wrapper-bootstrap .pb-sm-1,
    .wrapper-bootstrap .py-sm-1 {
      padding-bottom: 0.25rem !important; }
    .wrapper-bootstrap .pl-sm-1,
    .wrapper-bootstrap .px-sm-1 {
      padding-left: 0.25rem !important; }
    .wrapper-bootstrap .p-sm-2 {
      padding: 0.5rem !important; }
    .wrapper-bootstrap .pt-sm-2,
    .wrapper-bootstrap .py-sm-2 {
      padding-top: 0.5rem !important; }
    .wrapper-bootstrap .pr-sm-2,
    .wrapper-bootstrap .px-sm-2 {
      padding-right: 0.5rem !important; }
    .wrapper-bootstrap .pb-sm-2,
    .wrapper-bootstrap .py-sm-2 {
      padding-bottom: 0.5rem !important; }
    .wrapper-bootstrap .pl-sm-2,
    .wrapper-bootstrap .px-sm-2 {
      padding-left: 0.5rem !important; }
    .wrapper-bootstrap .p-sm-3 {
      padding: 1rem !important; }
    .wrapper-bootstrap .pt-sm-3,
    .wrapper-bootstrap .py-sm-3 {
      padding-top: 1rem !important; }
    .wrapper-bootstrap .pr-sm-3,
    .wrapper-bootstrap .px-sm-3 {
      padding-right: 1rem !important; }
    .wrapper-bootstrap .pb-sm-3,
    .wrapper-bootstrap .py-sm-3 {
      padding-bottom: 1rem !important; }
    .wrapper-bootstrap .pl-sm-3,
    .wrapper-bootstrap .px-sm-3 {
      padding-left: 1rem !important; }
    .wrapper-bootstrap .p-sm-4 {
      padding: 1.5rem !important; }
    .wrapper-bootstrap .pt-sm-4,
    .wrapper-bootstrap .py-sm-4 {
      padding-top: 1.5rem !important; }
    .wrapper-bootstrap .pr-sm-4,
    .wrapper-bootstrap .px-sm-4 {
      padding-right: 1.5rem !important; }
    .wrapper-bootstrap .pb-sm-4,
    .wrapper-bootstrap .py-sm-4 {
      padding-bottom: 1.5rem !important; }
    .wrapper-bootstrap .pl-sm-4,
    .wrapper-bootstrap .px-sm-4 {
      padding-left: 1.5rem !important; }
    .wrapper-bootstrap .p-sm-5 {
      padding: 3rem !important; }
    .wrapper-bootstrap .pt-sm-5,
    .wrapper-bootstrap .py-sm-5 {
      padding-top: 3rem !important; }
    .wrapper-bootstrap .pr-sm-5,
    .wrapper-bootstrap .px-sm-5 {
      padding-right: 3rem !important; }
    .wrapper-bootstrap .pb-sm-5,
    .wrapper-bootstrap .py-sm-5 {
      padding-bottom: 3rem !important; }
    .wrapper-bootstrap .pl-sm-5,
    .wrapper-bootstrap .px-sm-5 {
      padding-left: 3rem !important; }
    .wrapper-bootstrap .m-sm-n1 {
      margin: -0.25rem !important; }
    .wrapper-bootstrap .mt-sm-n1,
    .wrapper-bootstrap .my-sm-n1 {
      margin-top: -0.25rem !important; }
    .wrapper-bootstrap .mr-sm-n1,
    .wrapper-bootstrap .mx-sm-n1 {
      margin-right: -0.25rem !important; }
    .wrapper-bootstrap .mb-sm-n1,
    .wrapper-bootstrap .my-sm-n1 {
      margin-bottom: -0.25rem !important; }
    .wrapper-bootstrap .ml-sm-n1,
    .wrapper-bootstrap .mx-sm-n1 {
      margin-left: -0.25rem !important; }
    .wrapper-bootstrap .m-sm-n2 {
      margin: -0.5rem !important; }
    .wrapper-bootstrap .mt-sm-n2,
    .wrapper-bootstrap .my-sm-n2 {
      margin-top: -0.5rem !important; }
    .wrapper-bootstrap .mr-sm-n2,
    .wrapper-bootstrap .mx-sm-n2 {
      margin-right: -0.5rem !important; }
    .wrapper-bootstrap .mb-sm-n2,
    .wrapper-bootstrap .my-sm-n2 {
      margin-bottom: -0.5rem !important; }
    .wrapper-bootstrap .ml-sm-n2,
    .wrapper-bootstrap .mx-sm-n2 {
      margin-left: -0.5rem !important; }
    .wrapper-bootstrap .m-sm-n3 {
      margin: -1rem !important; }
    .wrapper-bootstrap .mt-sm-n3,
    .wrapper-bootstrap .my-sm-n3 {
      margin-top: -1rem !important; }
    .wrapper-bootstrap .mr-sm-n3,
    .wrapper-bootstrap .mx-sm-n3 {
      margin-right: -1rem !important; }
    .wrapper-bootstrap .mb-sm-n3,
    .wrapper-bootstrap .my-sm-n3 {
      margin-bottom: -1rem !important; }
    .wrapper-bootstrap .ml-sm-n3,
    .wrapper-bootstrap .mx-sm-n3 {
      margin-left: -1rem !important; }
    .wrapper-bootstrap .m-sm-n4 {
      margin: -1.5rem !important; }
    .wrapper-bootstrap .mt-sm-n4,
    .wrapper-bootstrap .my-sm-n4 {
      margin-top: -1.5rem !important; }
    .wrapper-bootstrap .mr-sm-n4,
    .wrapper-bootstrap .mx-sm-n4 {
      margin-right: -1.5rem !important; }
    .wrapper-bootstrap .mb-sm-n4,
    .wrapper-bootstrap .my-sm-n4 {
      margin-bottom: -1.5rem !important; }
    .wrapper-bootstrap .ml-sm-n4,
    .wrapper-bootstrap .mx-sm-n4 {
      margin-left: -1.5rem !important; }
    .wrapper-bootstrap .m-sm-n5 {
      margin: -3rem !important; }
    .wrapper-bootstrap .mt-sm-n5,
    .wrapper-bootstrap .my-sm-n5 {
      margin-top: -3rem !important; }
    .wrapper-bootstrap .mr-sm-n5,
    .wrapper-bootstrap .mx-sm-n5 {
      margin-right: -3rem !important; }
    .wrapper-bootstrap .mb-sm-n5,
    .wrapper-bootstrap .my-sm-n5 {
      margin-bottom: -3rem !important; }
    .wrapper-bootstrap .ml-sm-n5,
    .wrapper-bootstrap .mx-sm-n5 {
      margin-left: -3rem !important; }
    .wrapper-bootstrap .m-sm-auto {
      margin: auto !important; }
    .wrapper-bootstrap .mt-sm-auto,
    .wrapper-bootstrap .my-sm-auto {
      margin-top: auto !important; }
    .wrapper-bootstrap .mr-sm-auto,
    .wrapper-bootstrap .mx-sm-auto {
      margin-right: auto !important; }
    .wrapper-bootstrap .mb-sm-auto,
    .wrapper-bootstrap .my-sm-auto {
      margin-bottom: auto !important; }
    .wrapper-bootstrap .ml-sm-auto,
    .wrapper-bootstrap .mx-sm-auto {
      margin-left: auto !important; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .m-md-0 {
      margin: 0 !important; }
    .wrapper-bootstrap .mt-md-0,
    .wrapper-bootstrap .my-md-0 {
      margin-top: 0 !important; }
    .wrapper-bootstrap .mr-md-0,
    .wrapper-bootstrap .mx-md-0 {
      margin-right: 0 !important; }
    .wrapper-bootstrap .mb-md-0,
    .wrapper-bootstrap .my-md-0 {
      margin-bottom: 0 !important; }
    .wrapper-bootstrap .ml-md-0,
    .wrapper-bootstrap .mx-md-0 {
      margin-left: 0 !important; }
    .wrapper-bootstrap .m-md-1 {
      margin: 0.25rem !important; }
    .wrapper-bootstrap .mt-md-1,
    .wrapper-bootstrap .my-md-1 {
      margin-top: 0.25rem !important; }
    .wrapper-bootstrap .mr-md-1,
    .wrapper-bootstrap .mx-md-1 {
      margin-right: 0.25rem !important; }
    .wrapper-bootstrap .mb-md-1,
    .wrapper-bootstrap .my-md-1 {
      margin-bottom: 0.25rem !important; }
    .wrapper-bootstrap .ml-md-1,
    .wrapper-bootstrap .mx-md-1 {
      margin-left: 0.25rem !important; }
    .wrapper-bootstrap .m-md-2 {
      margin: 0.5rem !important; }
    .wrapper-bootstrap .mt-md-2,
    .wrapper-bootstrap .my-md-2 {
      margin-top: 0.5rem !important; }
    .wrapper-bootstrap .mr-md-2,
    .wrapper-bootstrap .mx-md-2 {
      margin-right: 0.5rem !important; }
    .wrapper-bootstrap .mb-md-2,
    .wrapper-bootstrap .my-md-2 {
      margin-bottom: 0.5rem !important; }
    .wrapper-bootstrap .ml-md-2,
    .wrapper-bootstrap .mx-md-2 {
      margin-left: 0.5rem !important; }
    .wrapper-bootstrap .m-md-3 {
      margin: 1rem !important; }
    .wrapper-bootstrap .mt-md-3,
    .wrapper-bootstrap .my-md-3 {
      margin-top: 1rem !important; }
    .wrapper-bootstrap .mr-md-3,
    .wrapper-bootstrap .mx-md-3 {
      margin-right: 1rem !important; }
    .wrapper-bootstrap .mb-md-3,
    .wrapper-bootstrap .my-md-3 {
      margin-bottom: 1rem !important; }
    .wrapper-bootstrap .ml-md-3,
    .wrapper-bootstrap .mx-md-3 {
      margin-left: 1rem !important; }
    .wrapper-bootstrap .m-md-4 {
      margin: 1.5rem !important; }
    .wrapper-bootstrap .mt-md-4,
    .wrapper-bootstrap .my-md-4 {
      margin-top: 1.5rem !important; }
    .wrapper-bootstrap .mr-md-4,
    .wrapper-bootstrap .mx-md-4 {
      margin-right: 1.5rem !important; }
    .wrapper-bootstrap .mb-md-4,
    .wrapper-bootstrap .my-md-4 {
      margin-bottom: 1.5rem !important; }
    .wrapper-bootstrap .ml-md-4,
    .wrapper-bootstrap .mx-md-4 {
      margin-left: 1.5rem !important; }
    .wrapper-bootstrap .m-md-5 {
      margin: 3rem !important; }
    .wrapper-bootstrap .mt-md-5,
    .wrapper-bootstrap .my-md-5 {
      margin-top: 3rem !important; }
    .wrapper-bootstrap .mr-md-5,
    .wrapper-bootstrap .mx-md-5 {
      margin-right: 3rem !important; }
    .wrapper-bootstrap .mb-md-5,
    .wrapper-bootstrap .my-md-5 {
      margin-bottom: 3rem !important; }
    .wrapper-bootstrap .ml-md-5,
    .wrapper-bootstrap .mx-md-5 {
      margin-left: 3rem !important; }
    .wrapper-bootstrap .p-md-0 {
      padding: 0 !important; }
    .wrapper-bootstrap .pt-md-0,
    .wrapper-bootstrap .py-md-0 {
      padding-top: 0 !important; }
    .wrapper-bootstrap .pr-md-0,
    .wrapper-bootstrap .px-md-0 {
      padding-right: 0 !important; }
    .wrapper-bootstrap .pb-md-0,
    .wrapper-bootstrap .py-md-0 {
      padding-bottom: 0 !important; }
    .wrapper-bootstrap .pl-md-0,
    .wrapper-bootstrap .px-md-0 {
      padding-left: 0 !important; }
    .wrapper-bootstrap .p-md-1 {
      padding: 0.25rem !important; }
    .wrapper-bootstrap .pt-md-1,
    .wrapper-bootstrap .py-md-1 {
      padding-top: 0.25rem !important; }
    .wrapper-bootstrap .pr-md-1,
    .wrapper-bootstrap .px-md-1 {
      padding-right: 0.25rem !important; }
    .wrapper-bootstrap .pb-md-1,
    .wrapper-bootstrap .py-md-1 {
      padding-bottom: 0.25rem !important; }
    .wrapper-bootstrap .pl-md-1,
    .wrapper-bootstrap .px-md-1 {
      padding-left: 0.25rem !important; }
    .wrapper-bootstrap .p-md-2 {
      padding: 0.5rem !important; }
    .wrapper-bootstrap .pt-md-2,
    .wrapper-bootstrap .py-md-2 {
      padding-top: 0.5rem !important; }
    .wrapper-bootstrap .pr-md-2,
    .wrapper-bootstrap .px-md-2 {
      padding-right: 0.5rem !important; }
    .wrapper-bootstrap .pb-md-2,
    .wrapper-bootstrap .py-md-2 {
      padding-bottom: 0.5rem !important; }
    .wrapper-bootstrap .pl-md-2,
    .wrapper-bootstrap .px-md-2 {
      padding-left: 0.5rem !important; }
    .wrapper-bootstrap .p-md-3 {
      padding: 1rem !important; }
    .wrapper-bootstrap .pt-md-3,
    .wrapper-bootstrap .py-md-3 {
      padding-top: 1rem !important; }
    .wrapper-bootstrap .pr-md-3,
    .wrapper-bootstrap .px-md-3 {
      padding-right: 1rem !important; }
    .wrapper-bootstrap .pb-md-3,
    .wrapper-bootstrap .py-md-3 {
      padding-bottom: 1rem !important; }
    .wrapper-bootstrap .pl-md-3,
    .wrapper-bootstrap .px-md-3 {
      padding-left: 1rem !important; }
    .wrapper-bootstrap .p-md-4 {
      padding: 1.5rem !important; }
    .wrapper-bootstrap .pt-md-4,
    .wrapper-bootstrap .py-md-4 {
      padding-top: 1.5rem !important; }
    .wrapper-bootstrap .pr-md-4,
    .wrapper-bootstrap .px-md-4 {
      padding-right: 1.5rem !important; }
    .wrapper-bootstrap .pb-md-4,
    .wrapper-bootstrap .py-md-4 {
      padding-bottom: 1.5rem !important; }
    .wrapper-bootstrap .pl-md-4,
    .wrapper-bootstrap .px-md-4 {
      padding-left: 1.5rem !important; }
    .wrapper-bootstrap .p-md-5 {
      padding: 3rem !important; }
    .wrapper-bootstrap .pt-md-5,
    .wrapper-bootstrap .py-md-5 {
      padding-top: 3rem !important; }
    .wrapper-bootstrap .pr-md-5,
    .wrapper-bootstrap .px-md-5 {
      padding-right: 3rem !important; }
    .wrapper-bootstrap .pb-md-5,
    .wrapper-bootstrap .py-md-5 {
      padding-bottom: 3rem !important; }
    .wrapper-bootstrap .pl-md-5,
    .wrapper-bootstrap .px-md-5 {
      padding-left: 3rem !important; }
    .wrapper-bootstrap .m-md-n1 {
      margin: -0.25rem !important; }
    .wrapper-bootstrap .mt-md-n1,
    .wrapper-bootstrap .my-md-n1 {
      margin-top: -0.25rem !important; }
    .wrapper-bootstrap .mr-md-n1,
    .wrapper-bootstrap .mx-md-n1 {
      margin-right: -0.25rem !important; }
    .wrapper-bootstrap .mb-md-n1,
    .wrapper-bootstrap .my-md-n1 {
      margin-bottom: -0.25rem !important; }
    .wrapper-bootstrap .ml-md-n1,
    .wrapper-bootstrap .mx-md-n1 {
      margin-left: -0.25rem !important; }
    .wrapper-bootstrap .m-md-n2 {
      margin: -0.5rem !important; }
    .wrapper-bootstrap .mt-md-n2,
    .wrapper-bootstrap .my-md-n2 {
      margin-top: -0.5rem !important; }
    .wrapper-bootstrap .mr-md-n2,
    .wrapper-bootstrap .mx-md-n2 {
      margin-right: -0.5rem !important; }
    .wrapper-bootstrap .mb-md-n2,
    .wrapper-bootstrap .my-md-n2 {
      margin-bottom: -0.5rem !important; }
    .wrapper-bootstrap .ml-md-n2,
    .wrapper-bootstrap .mx-md-n2 {
      margin-left: -0.5rem !important; }
    .wrapper-bootstrap .m-md-n3 {
      margin: -1rem !important; }
    .wrapper-bootstrap .mt-md-n3,
    .wrapper-bootstrap .my-md-n3 {
      margin-top: -1rem !important; }
    .wrapper-bootstrap .mr-md-n3,
    .wrapper-bootstrap .mx-md-n3 {
      margin-right: -1rem !important; }
    .wrapper-bootstrap .mb-md-n3,
    .wrapper-bootstrap .my-md-n3 {
      margin-bottom: -1rem !important; }
    .wrapper-bootstrap .ml-md-n3,
    .wrapper-bootstrap .mx-md-n3 {
      margin-left: -1rem !important; }
    .wrapper-bootstrap .m-md-n4 {
      margin: -1.5rem !important; }
    .wrapper-bootstrap .mt-md-n4,
    .wrapper-bootstrap .my-md-n4 {
      margin-top: -1.5rem !important; }
    .wrapper-bootstrap .mr-md-n4,
    .wrapper-bootstrap .mx-md-n4 {
      margin-right: -1.5rem !important; }
    .wrapper-bootstrap .mb-md-n4,
    .wrapper-bootstrap .my-md-n4 {
      margin-bottom: -1.5rem !important; }
    .wrapper-bootstrap .ml-md-n4,
    .wrapper-bootstrap .mx-md-n4 {
      margin-left: -1.5rem !important; }
    .wrapper-bootstrap .m-md-n5 {
      margin: -3rem !important; }
    .wrapper-bootstrap .mt-md-n5,
    .wrapper-bootstrap .my-md-n5 {
      margin-top: -3rem !important; }
    .wrapper-bootstrap .mr-md-n5,
    .wrapper-bootstrap .mx-md-n5 {
      margin-right: -3rem !important; }
    .wrapper-bootstrap .mb-md-n5,
    .wrapper-bootstrap .my-md-n5 {
      margin-bottom: -3rem !important; }
    .wrapper-bootstrap .ml-md-n5,
    .wrapper-bootstrap .mx-md-n5 {
      margin-left: -3rem !important; }
    .wrapper-bootstrap .m-md-auto {
      margin: auto !important; }
    .wrapper-bootstrap .mt-md-auto,
    .wrapper-bootstrap .my-md-auto {
      margin-top: auto !important; }
    .wrapper-bootstrap .mr-md-auto,
    .wrapper-bootstrap .mx-md-auto {
      margin-right: auto !important; }
    .wrapper-bootstrap .mb-md-auto,
    .wrapper-bootstrap .my-md-auto {
      margin-bottom: auto !important; }
    .wrapper-bootstrap .ml-md-auto,
    .wrapper-bootstrap .mx-md-auto {
      margin-left: auto !important; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .m-lg-0 {
      margin: 0 !important; }
    .wrapper-bootstrap .mt-lg-0,
    .wrapper-bootstrap .my-lg-0 {
      margin-top: 0 !important; }
    .wrapper-bootstrap .mr-lg-0,
    .wrapper-bootstrap .mx-lg-0 {
      margin-right: 0 !important; }
    .wrapper-bootstrap .mb-lg-0,
    .wrapper-bootstrap .my-lg-0 {
      margin-bottom: 0 !important; }
    .wrapper-bootstrap .ml-lg-0,
    .wrapper-bootstrap .mx-lg-0 {
      margin-left: 0 !important; }
    .wrapper-bootstrap .m-lg-1 {
      margin: 0.25rem !important; }
    .wrapper-bootstrap .mt-lg-1,
    .wrapper-bootstrap .my-lg-1 {
      margin-top: 0.25rem !important; }
    .wrapper-bootstrap .mr-lg-1,
    .wrapper-bootstrap .mx-lg-1 {
      margin-right: 0.25rem !important; }
    .wrapper-bootstrap .mb-lg-1,
    .wrapper-bootstrap .my-lg-1 {
      margin-bottom: 0.25rem !important; }
    .wrapper-bootstrap .ml-lg-1,
    .wrapper-bootstrap .mx-lg-1 {
      margin-left: 0.25rem !important; }
    .wrapper-bootstrap .m-lg-2 {
      margin: 0.5rem !important; }
    .wrapper-bootstrap .mt-lg-2,
    .wrapper-bootstrap .my-lg-2 {
      margin-top: 0.5rem !important; }
    .wrapper-bootstrap .mr-lg-2,
    .wrapper-bootstrap .mx-lg-2 {
      margin-right: 0.5rem !important; }
    .wrapper-bootstrap .mb-lg-2,
    .wrapper-bootstrap .my-lg-2 {
      margin-bottom: 0.5rem !important; }
    .wrapper-bootstrap .ml-lg-2,
    .wrapper-bootstrap .mx-lg-2 {
      margin-left: 0.5rem !important; }
    .wrapper-bootstrap .m-lg-3 {
      margin: 1rem !important; }
    .wrapper-bootstrap .mt-lg-3,
    .wrapper-bootstrap .my-lg-3 {
      margin-top: 1rem !important; }
    .wrapper-bootstrap .mr-lg-3,
    .wrapper-bootstrap .mx-lg-3 {
      margin-right: 1rem !important; }
    .wrapper-bootstrap .mb-lg-3,
    .wrapper-bootstrap .my-lg-3 {
      margin-bottom: 1rem !important; }
    .wrapper-bootstrap .ml-lg-3,
    .wrapper-bootstrap .mx-lg-3 {
      margin-left: 1rem !important; }
    .wrapper-bootstrap .m-lg-4 {
      margin: 1.5rem !important; }
    .wrapper-bootstrap .mt-lg-4,
    .wrapper-bootstrap .my-lg-4 {
      margin-top: 1.5rem !important; }
    .wrapper-bootstrap .mr-lg-4,
    .wrapper-bootstrap .mx-lg-4 {
      margin-right: 1.5rem !important; }
    .wrapper-bootstrap .mb-lg-4,
    .wrapper-bootstrap .my-lg-4 {
      margin-bottom: 1.5rem !important; }
    .wrapper-bootstrap .ml-lg-4,
    .wrapper-bootstrap .mx-lg-4 {
      margin-left: 1.5rem !important; }
    .wrapper-bootstrap .m-lg-5 {
      margin: 3rem !important; }
    .wrapper-bootstrap .mt-lg-5,
    .wrapper-bootstrap .my-lg-5 {
      margin-top: 3rem !important; }
    .wrapper-bootstrap .mr-lg-5,
    .wrapper-bootstrap .mx-lg-5 {
      margin-right: 3rem !important; }
    .wrapper-bootstrap .mb-lg-5,
    .wrapper-bootstrap .my-lg-5 {
      margin-bottom: 3rem !important; }
    .wrapper-bootstrap .ml-lg-5,
    .wrapper-bootstrap .mx-lg-5 {
      margin-left: 3rem !important; }
    .wrapper-bootstrap .p-lg-0 {
      padding: 0 !important; }
    .wrapper-bootstrap .pt-lg-0,
    .wrapper-bootstrap .py-lg-0 {
      padding-top: 0 !important; }
    .wrapper-bootstrap .pr-lg-0,
    .wrapper-bootstrap .px-lg-0 {
      padding-right: 0 !important; }
    .wrapper-bootstrap .pb-lg-0,
    .wrapper-bootstrap .py-lg-0 {
      padding-bottom: 0 !important; }
    .wrapper-bootstrap .pl-lg-0,
    .wrapper-bootstrap .px-lg-0 {
      padding-left: 0 !important; }
    .wrapper-bootstrap .p-lg-1 {
      padding: 0.25rem !important; }
    .wrapper-bootstrap .pt-lg-1,
    .wrapper-bootstrap .py-lg-1 {
      padding-top: 0.25rem !important; }
    .wrapper-bootstrap .pr-lg-1,
    .wrapper-bootstrap .px-lg-1 {
      padding-right: 0.25rem !important; }
    .wrapper-bootstrap .pb-lg-1,
    .wrapper-bootstrap .py-lg-1 {
      padding-bottom: 0.25rem !important; }
    .wrapper-bootstrap .pl-lg-1,
    .wrapper-bootstrap .px-lg-1 {
      padding-left: 0.25rem !important; }
    .wrapper-bootstrap .p-lg-2 {
      padding: 0.5rem !important; }
    .wrapper-bootstrap .pt-lg-2,
    .wrapper-bootstrap .py-lg-2 {
      padding-top: 0.5rem !important; }
    .wrapper-bootstrap .pr-lg-2,
    .wrapper-bootstrap .px-lg-2 {
      padding-right: 0.5rem !important; }
    .wrapper-bootstrap .pb-lg-2,
    .wrapper-bootstrap .py-lg-2 {
      padding-bottom: 0.5rem !important; }
    .wrapper-bootstrap .pl-lg-2,
    .wrapper-bootstrap .px-lg-2 {
      padding-left: 0.5rem !important; }
    .wrapper-bootstrap .p-lg-3 {
      padding: 1rem !important; }
    .wrapper-bootstrap .pt-lg-3,
    .wrapper-bootstrap .py-lg-3 {
      padding-top: 1rem !important; }
    .wrapper-bootstrap .pr-lg-3,
    .wrapper-bootstrap .px-lg-3 {
      padding-right: 1rem !important; }
    .wrapper-bootstrap .pb-lg-3,
    .wrapper-bootstrap .py-lg-3 {
      padding-bottom: 1rem !important; }
    .wrapper-bootstrap .pl-lg-3,
    .wrapper-bootstrap .px-lg-3 {
      padding-left: 1rem !important; }
    .wrapper-bootstrap .p-lg-4 {
      padding: 1.5rem !important; }
    .wrapper-bootstrap .pt-lg-4,
    .wrapper-bootstrap .py-lg-4 {
      padding-top: 1.5rem !important; }
    .wrapper-bootstrap .pr-lg-4,
    .wrapper-bootstrap .px-lg-4 {
      padding-right: 1.5rem !important; }
    .wrapper-bootstrap .pb-lg-4,
    .wrapper-bootstrap .py-lg-4 {
      padding-bottom: 1.5rem !important; }
    .wrapper-bootstrap .pl-lg-4,
    .wrapper-bootstrap .px-lg-4 {
      padding-left: 1.5rem !important; }
    .wrapper-bootstrap .p-lg-5 {
      padding: 3rem !important; }
    .wrapper-bootstrap .pt-lg-5,
    .wrapper-bootstrap .py-lg-5 {
      padding-top: 3rem !important; }
    .wrapper-bootstrap .pr-lg-5,
    .wrapper-bootstrap .px-lg-5 {
      padding-right: 3rem !important; }
    .wrapper-bootstrap .pb-lg-5,
    .wrapper-bootstrap .py-lg-5 {
      padding-bottom: 3rem !important; }
    .wrapper-bootstrap .pl-lg-5,
    .wrapper-bootstrap .px-lg-5 {
      padding-left: 3rem !important; }
    .wrapper-bootstrap .m-lg-n1 {
      margin: -0.25rem !important; }
    .wrapper-bootstrap .mt-lg-n1,
    .wrapper-bootstrap .my-lg-n1 {
      margin-top: -0.25rem !important; }
    .wrapper-bootstrap .mr-lg-n1,
    .wrapper-bootstrap .mx-lg-n1 {
      margin-right: -0.25rem !important; }
    .wrapper-bootstrap .mb-lg-n1,
    .wrapper-bootstrap .my-lg-n1 {
      margin-bottom: -0.25rem !important; }
    .wrapper-bootstrap .ml-lg-n1,
    .wrapper-bootstrap .mx-lg-n1 {
      margin-left: -0.25rem !important; }
    .wrapper-bootstrap .m-lg-n2 {
      margin: -0.5rem !important; }
    .wrapper-bootstrap .mt-lg-n2,
    .wrapper-bootstrap .my-lg-n2 {
      margin-top: -0.5rem !important; }
    .wrapper-bootstrap .mr-lg-n2,
    .wrapper-bootstrap .mx-lg-n2 {
      margin-right: -0.5rem !important; }
    .wrapper-bootstrap .mb-lg-n2,
    .wrapper-bootstrap .my-lg-n2 {
      margin-bottom: -0.5rem !important; }
    .wrapper-bootstrap .ml-lg-n2,
    .wrapper-bootstrap .mx-lg-n2 {
      margin-left: -0.5rem !important; }
    .wrapper-bootstrap .m-lg-n3 {
      margin: -1rem !important; }
    .wrapper-bootstrap .mt-lg-n3,
    .wrapper-bootstrap .my-lg-n3 {
      margin-top: -1rem !important; }
    .wrapper-bootstrap .mr-lg-n3,
    .wrapper-bootstrap .mx-lg-n3 {
      margin-right: -1rem !important; }
    .wrapper-bootstrap .mb-lg-n3,
    .wrapper-bootstrap .my-lg-n3 {
      margin-bottom: -1rem !important; }
    .wrapper-bootstrap .ml-lg-n3,
    .wrapper-bootstrap .mx-lg-n3 {
      margin-left: -1rem !important; }
    .wrapper-bootstrap .m-lg-n4 {
      margin: -1.5rem !important; }
    .wrapper-bootstrap .mt-lg-n4,
    .wrapper-bootstrap .my-lg-n4 {
      margin-top: -1.5rem !important; }
    .wrapper-bootstrap .mr-lg-n4,
    .wrapper-bootstrap .mx-lg-n4 {
      margin-right: -1.5rem !important; }
    .wrapper-bootstrap .mb-lg-n4,
    .wrapper-bootstrap .my-lg-n4 {
      margin-bottom: -1.5rem !important; }
    .wrapper-bootstrap .ml-lg-n4,
    .wrapper-bootstrap .mx-lg-n4 {
      margin-left: -1.5rem !important; }
    .wrapper-bootstrap .m-lg-n5 {
      margin: -3rem !important; }
    .wrapper-bootstrap .mt-lg-n5,
    .wrapper-bootstrap .my-lg-n5 {
      margin-top: -3rem !important; }
    .wrapper-bootstrap .mr-lg-n5,
    .wrapper-bootstrap .mx-lg-n5 {
      margin-right: -3rem !important; }
    .wrapper-bootstrap .mb-lg-n5,
    .wrapper-bootstrap .my-lg-n5 {
      margin-bottom: -3rem !important; }
    .wrapper-bootstrap .ml-lg-n5,
    .wrapper-bootstrap .mx-lg-n5 {
      margin-left: -3rem !important; }
    .wrapper-bootstrap .m-lg-auto {
      margin: auto !important; }
    .wrapper-bootstrap .mt-lg-auto,
    .wrapper-bootstrap .my-lg-auto {
      margin-top: auto !important; }
    .wrapper-bootstrap .mr-lg-auto,
    .wrapper-bootstrap .mx-lg-auto {
      margin-right: auto !important; }
    .wrapper-bootstrap .mb-lg-auto,
    .wrapper-bootstrap .my-lg-auto {
      margin-bottom: auto !important; }
    .wrapper-bootstrap .ml-lg-auto,
    .wrapper-bootstrap .mx-lg-auto {
      margin-left: auto !important; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .m-xl-0 {
      margin: 0 !important; }
    .wrapper-bootstrap .mt-xl-0,
    .wrapper-bootstrap .my-xl-0 {
      margin-top: 0 !important; }
    .wrapper-bootstrap .mr-xl-0,
    .wrapper-bootstrap .mx-xl-0 {
      margin-right: 0 !important; }
    .wrapper-bootstrap .mb-xl-0,
    .wrapper-bootstrap .my-xl-0 {
      margin-bottom: 0 !important; }
    .wrapper-bootstrap .ml-xl-0,
    .wrapper-bootstrap .mx-xl-0 {
      margin-left: 0 !important; }
    .wrapper-bootstrap .m-xl-1 {
      margin: 0.25rem !important; }
    .wrapper-bootstrap .mt-xl-1,
    .wrapper-bootstrap .my-xl-1 {
      margin-top: 0.25rem !important; }
    .wrapper-bootstrap .mr-xl-1,
    .wrapper-bootstrap .mx-xl-1 {
      margin-right: 0.25rem !important; }
    .wrapper-bootstrap .mb-xl-1,
    .wrapper-bootstrap .my-xl-1 {
      margin-bottom: 0.25rem !important; }
    .wrapper-bootstrap .ml-xl-1,
    .wrapper-bootstrap .mx-xl-1 {
      margin-left: 0.25rem !important; }
    .wrapper-bootstrap .m-xl-2 {
      margin: 0.5rem !important; }
    .wrapper-bootstrap .mt-xl-2,
    .wrapper-bootstrap .my-xl-2 {
      margin-top: 0.5rem !important; }
    .wrapper-bootstrap .mr-xl-2,
    .wrapper-bootstrap .mx-xl-2 {
      margin-right: 0.5rem !important; }
    .wrapper-bootstrap .mb-xl-2,
    .wrapper-bootstrap .my-xl-2 {
      margin-bottom: 0.5rem !important; }
    .wrapper-bootstrap .ml-xl-2,
    .wrapper-bootstrap .mx-xl-2 {
      margin-left: 0.5rem !important; }
    .wrapper-bootstrap .m-xl-3 {
      margin: 1rem !important; }
    .wrapper-bootstrap .mt-xl-3,
    .wrapper-bootstrap .my-xl-3 {
      margin-top: 1rem !important; }
    .wrapper-bootstrap .mr-xl-3,
    .wrapper-bootstrap .mx-xl-3 {
      margin-right: 1rem !important; }
    .wrapper-bootstrap .mb-xl-3,
    .wrapper-bootstrap .my-xl-3 {
      margin-bottom: 1rem !important; }
    .wrapper-bootstrap .ml-xl-3,
    .wrapper-bootstrap .mx-xl-3 {
      margin-left: 1rem !important; }
    .wrapper-bootstrap .m-xl-4 {
      margin: 1.5rem !important; }
    .wrapper-bootstrap .mt-xl-4,
    .wrapper-bootstrap .my-xl-4 {
      margin-top: 1.5rem !important; }
    .wrapper-bootstrap .mr-xl-4,
    .wrapper-bootstrap .mx-xl-4 {
      margin-right: 1.5rem !important; }
    .wrapper-bootstrap .mb-xl-4,
    .wrapper-bootstrap .my-xl-4 {
      margin-bottom: 1.5rem !important; }
    .wrapper-bootstrap .ml-xl-4,
    .wrapper-bootstrap .mx-xl-4 {
      margin-left: 1.5rem !important; }
    .wrapper-bootstrap .m-xl-5 {
      margin: 3rem !important; }
    .wrapper-bootstrap .mt-xl-5,
    .wrapper-bootstrap .my-xl-5 {
      margin-top: 3rem !important; }
    .wrapper-bootstrap .mr-xl-5,
    .wrapper-bootstrap .mx-xl-5 {
      margin-right: 3rem !important; }
    .wrapper-bootstrap .mb-xl-5,
    .wrapper-bootstrap .my-xl-5 {
      margin-bottom: 3rem !important; }
    .wrapper-bootstrap .ml-xl-5,
    .wrapper-bootstrap .mx-xl-5 {
      margin-left: 3rem !important; }
    .wrapper-bootstrap .p-xl-0 {
      padding: 0 !important; }
    .wrapper-bootstrap .pt-xl-0,
    .wrapper-bootstrap .py-xl-0 {
      padding-top: 0 !important; }
    .wrapper-bootstrap .pr-xl-0,
    .wrapper-bootstrap .px-xl-0 {
      padding-right: 0 !important; }
    .wrapper-bootstrap .pb-xl-0,
    .wrapper-bootstrap .py-xl-0 {
      padding-bottom: 0 !important; }
    .wrapper-bootstrap .pl-xl-0,
    .wrapper-bootstrap .px-xl-0 {
      padding-left: 0 !important; }
    .wrapper-bootstrap .p-xl-1 {
      padding: 0.25rem !important; }
    .wrapper-bootstrap .pt-xl-1,
    .wrapper-bootstrap .py-xl-1 {
      padding-top: 0.25rem !important; }
    .wrapper-bootstrap .pr-xl-1,
    .wrapper-bootstrap .px-xl-1 {
      padding-right: 0.25rem !important; }
    .wrapper-bootstrap .pb-xl-1,
    .wrapper-bootstrap .py-xl-1 {
      padding-bottom: 0.25rem !important; }
    .wrapper-bootstrap .pl-xl-1,
    .wrapper-bootstrap .px-xl-1 {
      padding-left: 0.25rem !important; }
    .wrapper-bootstrap .p-xl-2 {
      padding: 0.5rem !important; }
    .wrapper-bootstrap .pt-xl-2,
    .wrapper-bootstrap .py-xl-2 {
      padding-top: 0.5rem !important; }
    .wrapper-bootstrap .pr-xl-2,
    .wrapper-bootstrap .px-xl-2 {
      padding-right: 0.5rem !important; }
    .wrapper-bootstrap .pb-xl-2,
    .wrapper-bootstrap .py-xl-2 {
      padding-bottom: 0.5rem !important; }
    .wrapper-bootstrap .pl-xl-2,
    .wrapper-bootstrap .px-xl-2 {
      padding-left: 0.5rem !important; }
    .wrapper-bootstrap .p-xl-3 {
      padding: 1rem !important; }
    .wrapper-bootstrap .pt-xl-3,
    .wrapper-bootstrap .py-xl-3 {
      padding-top: 1rem !important; }
    .wrapper-bootstrap .pr-xl-3,
    .wrapper-bootstrap .px-xl-3 {
      padding-right: 1rem !important; }
    .wrapper-bootstrap .pb-xl-3,
    .wrapper-bootstrap .py-xl-3 {
      padding-bottom: 1rem !important; }
    .wrapper-bootstrap .pl-xl-3,
    .wrapper-bootstrap .px-xl-3 {
      padding-left: 1rem !important; }
    .wrapper-bootstrap .p-xl-4 {
      padding: 1.5rem !important; }
    .wrapper-bootstrap .pt-xl-4,
    .wrapper-bootstrap .py-xl-4 {
      padding-top: 1.5rem !important; }
    .wrapper-bootstrap .pr-xl-4,
    .wrapper-bootstrap .px-xl-4 {
      padding-right: 1.5rem !important; }
    .wrapper-bootstrap .pb-xl-4,
    .wrapper-bootstrap .py-xl-4 {
      padding-bottom: 1.5rem !important; }
    .wrapper-bootstrap .pl-xl-4,
    .wrapper-bootstrap .px-xl-4 {
      padding-left: 1.5rem !important; }
    .wrapper-bootstrap .p-xl-5 {
      padding: 3rem !important; }
    .wrapper-bootstrap .pt-xl-5,
    .wrapper-bootstrap .py-xl-5 {
      padding-top: 3rem !important; }
    .wrapper-bootstrap .pr-xl-5,
    .wrapper-bootstrap .px-xl-5 {
      padding-right: 3rem !important; }
    .wrapper-bootstrap .pb-xl-5,
    .wrapper-bootstrap .py-xl-5 {
      padding-bottom: 3rem !important; }
    .wrapper-bootstrap .pl-xl-5,
    .wrapper-bootstrap .px-xl-5 {
      padding-left: 3rem !important; }
    .wrapper-bootstrap .m-xl-n1 {
      margin: -0.25rem !important; }
    .wrapper-bootstrap .mt-xl-n1,
    .wrapper-bootstrap .my-xl-n1 {
      margin-top: -0.25rem !important; }
    .wrapper-bootstrap .mr-xl-n1,
    .wrapper-bootstrap .mx-xl-n1 {
      margin-right: -0.25rem !important; }
    .wrapper-bootstrap .mb-xl-n1,
    .wrapper-bootstrap .my-xl-n1 {
      margin-bottom: -0.25rem !important; }
    .wrapper-bootstrap .ml-xl-n1,
    .wrapper-bootstrap .mx-xl-n1 {
      margin-left: -0.25rem !important; }
    .wrapper-bootstrap .m-xl-n2 {
      margin: -0.5rem !important; }
    .wrapper-bootstrap .mt-xl-n2,
    .wrapper-bootstrap .my-xl-n2 {
      margin-top: -0.5rem !important; }
    .wrapper-bootstrap .mr-xl-n2,
    .wrapper-bootstrap .mx-xl-n2 {
      margin-right: -0.5rem !important; }
    .wrapper-bootstrap .mb-xl-n2,
    .wrapper-bootstrap .my-xl-n2 {
      margin-bottom: -0.5rem !important; }
    .wrapper-bootstrap .ml-xl-n2,
    .wrapper-bootstrap .mx-xl-n2 {
      margin-left: -0.5rem !important; }
    .wrapper-bootstrap .m-xl-n3 {
      margin: -1rem !important; }
    .wrapper-bootstrap .mt-xl-n3,
    .wrapper-bootstrap .my-xl-n3 {
      margin-top: -1rem !important; }
    .wrapper-bootstrap .mr-xl-n3,
    .wrapper-bootstrap .mx-xl-n3 {
      margin-right: -1rem !important; }
    .wrapper-bootstrap .mb-xl-n3,
    .wrapper-bootstrap .my-xl-n3 {
      margin-bottom: -1rem !important; }
    .wrapper-bootstrap .ml-xl-n3,
    .wrapper-bootstrap .mx-xl-n3 {
      margin-left: -1rem !important; }
    .wrapper-bootstrap .m-xl-n4 {
      margin: -1.5rem !important; }
    .wrapper-bootstrap .mt-xl-n4,
    .wrapper-bootstrap .my-xl-n4 {
      margin-top: -1.5rem !important; }
    .wrapper-bootstrap .mr-xl-n4,
    .wrapper-bootstrap .mx-xl-n4 {
      margin-right: -1.5rem !important; }
    .wrapper-bootstrap .mb-xl-n4,
    .wrapper-bootstrap .my-xl-n4 {
      margin-bottom: -1.5rem !important; }
    .wrapper-bootstrap .ml-xl-n4,
    .wrapper-bootstrap .mx-xl-n4 {
      margin-left: -1.5rem !important; }
    .wrapper-bootstrap .m-xl-n5 {
      margin: -3rem !important; }
    .wrapper-bootstrap .mt-xl-n5,
    .wrapper-bootstrap .my-xl-n5 {
      margin-top: -3rem !important; }
    .wrapper-bootstrap .mr-xl-n5,
    .wrapper-bootstrap .mx-xl-n5 {
      margin-right: -3rem !important; }
    .wrapper-bootstrap .mb-xl-n5,
    .wrapper-bootstrap .my-xl-n5 {
      margin-bottom: -3rem !important; }
    .wrapper-bootstrap .ml-xl-n5,
    .wrapper-bootstrap .mx-xl-n5 {
      margin-left: -3rem !important; }
    .wrapper-bootstrap .m-xl-auto {
      margin: auto !important; }
    .wrapper-bootstrap .mt-xl-auto,
    .wrapper-bootstrap .my-xl-auto {
      margin-top: auto !important; }
    .wrapper-bootstrap .mr-xl-auto,
    .wrapper-bootstrap .mx-xl-auto {
      margin-right: auto !important; }
    .wrapper-bootstrap .mb-xl-auto,
    .wrapper-bootstrap .my-xl-auto {
      margin-bottom: auto !important; }
    .wrapper-bootstrap .ml-xl-auto,
    .wrapper-bootstrap .mx-xl-auto {
      margin-left: auto !important; } }
  .wrapper-bootstrap .text-monospace {
    font-family: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace !important; }
  .wrapper-bootstrap .text-justify {
    text-align: justify !important; }
  .wrapper-bootstrap .text-wrap {
    white-space: normal !important; }
  .wrapper-bootstrap .text-nowrap {
    white-space: nowrap !important; }
  .wrapper-bootstrap .text-truncate {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap; }
  .wrapper-bootstrap .text-left {
    text-align: left !important; }
  .wrapper-bootstrap .text-right {
    text-align: right !important; }
  .wrapper-bootstrap .text-center {
    text-align: center !important; }
  @media (min-width: 576px) {
    .wrapper-bootstrap .text-sm-left {
      text-align: left !important; }
    .wrapper-bootstrap .text-sm-right {
      text-align: right !important; }
    .wrapper-bootstrap .text-sm-center {
      text-align: center !important; } }
  @media (min-width: 768px) {
    .wrapper-bootstrap .text-md-left {
      text-align: left !important; }
    .wrapper-bootstrap .text-md-right {
      text-align: right !important; }
    .wrapper-bootstrap .text-md-center {
      text-align: center !important; } }
  @media (min-width: 992px) {
    .wrapper-bootstrap .text-lg-left {
      text-align: left !important; }
    .wrapper-bootstrap .text-lg-right {
      text-align: right !important; }
    .wrapper-bootstrap .text-lg-center {
      text-align: center !important; } }
  @media (min-width: 1200px) {
    .wrapper-bootstrap .text-xl-left {
      text-align: left !important; }
    .wrapper-bootstrap .text-xl-right {
      text-align: right !important; }
    .wrapper-bootstrap .text-xl-center {
      text-align: center !important; } }
  .wrapper-bootstrap .text-lowercase {
    text-transform: lowercase !important; }
  .wrapper-bootstrap .text-uppercase {
    text-transform: uppercase !important; }
  .wrapper-bootstrap .text-capitalize {
    text-transform: capitalize !important; }
  .wrapper-bootstrap .font-weight-light {
    font-weight: 300 !important; }
  .wrapper-bootstrap .font-weight-lighter {
    font-weight: lighter !important; }
  .wrapper-bootstrap .font-weight-normal {
    font-weight: 400 !important; }
  .wrapper-bootstrap .font-weight-bold {
    font-weight: 700 !important; }
  .wrapper-bootstrap .font-weight-bolder {
    font-weight: bolder !important; }
  .wrapper-bootstrap .font-italic {
    font-style: italic !important; }
  .wrapper-bootstrap .text-white {
    color: #fff !important; }
  .wrapper-bootstrap .text-primary {
    color: #007bff !important; }
  .wrapper-bootstrap a.text-primary:hover, .wrapper-bootstrap a.text-primary:focus {
    color: #0056b3 !important; }
  .wrapper-bootstrap .text-secondary {
    color: #6c757d !important; }
  .wrapper-bootstrap a.text-secondary:hover, .wrapper-bootstrap a.text-secondary:focus {
    color: #494f54 !important; }
  .wrapper-bootstrap .text-success {
    color: #28a745 !important; }
  .wrapper-bootstrap a.text-success:hover, .wrapper-bootstrap a.text-success:focus {
    color: #19692c !important; }
  .wrapper-bootstrap .text-info {
    color: #17a2b8 !important; }
  .wrapper-bootstrap a.text-info:hover, .wrapper-bootstrap a.text-info:focus {
    color: #0f6674 !important; }
  .wrapper-bootstrap .text-warning {
    color: #ffc107 !important; }
  .wrapper-bootstrap a.text-warning:hover, .wrapper-bootstrap a.text-warning:focus {
    color: #ba8b00 !important; }
  .wrapper-bootstrap .text-danger {
    color: #dc3545 !important; }
  .wrapper-bootstrap a.text-danger:hover, .wrapper-bootstrap a.text-danger:focus {
    color: #a71d2a !important; }
  .wrapper-bootstrap .text-light {
    color: #f8f9fa !important; }
  .wrapper-bootstrap a.text-light:hover, .wrapper-bootstrap a.text-light:focus {
    color: #cbd3da !important; }
  .wrapper-bootstrap .text-dark {
    color: #343a40 !important; }
  .wrapper-bootstrap a.text-dark:hover, .wrapper-bootstrap a.text-dark:focus {
    color: #121416 !important; }
  .wrapper-bootstrap .text-body {
    color: #212529 !important; }
  .wrapper-bootstrap .text-muted {
    color: #6c757d !important; }
  .wrapper-bootstrap .text-black-50 {
    color: rgba(0, 0, 0, 0.5) !important; }
  .wrapper-bootstrap .text-white-50 {
    color: rgba(255, 255, 255, 0.5) !important; }
  .wrapper-bootstrap .text-hide {
    font: 0/0 a;
    color: transparent;
    text-shadow: none;
    background-color: transparent;
    border: 0; }
  .wrapper-bootstrap .text-decoration-none {
    text-decoration: none !important; }
  .wrapper-bootstrap .text-break {
    word-break: break-word !important;
    overflow-wrap: break-word !important; }
  .wrapper-bootstrap .text-reset {
    color: inherit !important; }
  .wrapper-bootstrap .visible {
    visibility: visible !important; }
  .wrapper-bootstrap .invisible {
    visibility: hidden !important; }
  @media print {
    .wrapper-bootstrap *,
    .wrapper-bootstrap *::before,
    .wrapper-bootstrap *::after {
      text-shadow: none !important;
      -webkit-box-shadow: none !important;
      box-shadow: none !important; }
    .wrapper-bootstrap a:not(.btn) {
      text-decoration: underline; }
    .wrapper-bootstrap abbr[title]::after {
      content: " (" attr(title) ")"; }
    .wrapper-bootstrap pre {
      white-space: pre-wrap !important; }
    .wrapper-bootstrap pre,
    .wrapper-bootstrap blockquote {
      border: 1px solid #adb5bd;
      page-break-inside: avoid; }
    .wrapper-bootstrap thead {
      display: table-header-group; }
    .wrapper-bootstrap tr,
    .wrapper-bootstrap img {
      page-break-inside: avoid; }
    .wrapper-bootstrap p,
    .wrapper-bootstrap h2,
    .wrapper-bootstrap h3 {
      orphans: 3;
      widows: 3; }
    .wrapper-bootstrap h2,
    .wrapper-bootstrap h3 {
      page-break-after: avoid; }
    @page {
      .wrapper-bootstrap {
        size: a3; } }
    .wrapper-bootstrap body {
      min-width: 992px !important; }
    .wrapper-bootstrap .container {
      min-width: 992px !important; }
    .wrapper-bootstrap .navbar {
      display: none; }
    .wrapper-bootstrap .badge {
      border: 1px solid #000; }
    .wrapper-bootstrap .table {
      border-collapse: collapse !important; }
      .wrapper-bootstrap .table td,
      .wrapper-bootstrap .table th {
        background-color: #fff !important; }
    .wrapper-bootstrap .table-bordered th,
    .wrapper-bootstrap .table-bordered td {
      border: 1px solid #dee2e6 !important; }
    .wrapper-bootstrap .table-dark {
      color: inherit; }
      .wrapper-bootstrap .table-dark th,
      .wrapper-bootstrap .table-dark td,
      .wrapper-bootstrap .table-dark thead th,
      .wrapper-bootstrap .table-dark tbody + tbody {
        border-color: #dee2e6; }
    .wrapper-bootstrap .table .thead-dark th {
      color: inherit;
      border-color: #dee2e6; } }

body {
  font-family: "Roboto", sans-serif; }

.aligned-row {
  display: -webkit-box;
  display: -ms-flexbox;
  display: flex;
  -webkit-box-orient: horizontal;
  -webkit-box-direction: normal;
  -ms-flex-flow: row wrap;
  flex-flow: row wrap; }
  .aligned-row::before {
    display: block; }

.heading, h1,
h2,
h3,
h4,
h5,
h6 {
  line-height: 1;
  color: inherit;
  font-weight: 500; }
  .heading strong, h1 strong, h2 strong, h3 strong, h4 strong, h5 strong, h6 strong, .heading b, h1 b, h2 b, h3 b, h4 b, h5 b, h6 b {
    font-weight: 700; }

.heading-1 {
  font-size: 45px; }

.heading-2 {
  font-size: 35px; }

.heading-3 {
  font-size: 25px; }

p {
  font-size: 16px;
  font-weight: 300;
  color: inherit;
  margin: 0; }

footer {
  margin: 40px 0 20px; }

header {
  margin: 2% 0; }

table .th-lg {
  min-width: 20rem; }

table .th-md {
  min-width: 12rem; }

/*# sourceMappingURL=app.css.map */
